@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -73,6 +73,57 @@ static const unsigned char dsa_p[] = { 0x90, 0x63, 0x82, 0xae, 0x72, 0x71, 0xac,
     dsa_g[] = { 0x40, 0x6d, 0xfb, 0x6d, 0xb6, 0x6, 0x32, 0xcc, 0xf0, 0xe9, 0x84, 0x16, 0x1e, 0xe1, 0x21, 0xcd, 0x34, 0xe7, 0xbb, 0x6c, 0x98, 0xff, 0xa9, 0xb9, 0xae, 0xe4, 0x6a, 0x61, 0x51, 0xf8, 0x66, 0x83, 0xa4, 0x34, 0x63, 0x81, 0xc4, 0x5f, 0xee, 0x85, 0x74, 0xee, 0x2a, 0x63, 0x9d, 0xcf, 0x97, 0x50, 0xb8, 0x9f, 0x76, 0xd9, 0xe, 0x58, 0xab, 0xac, 0x2e, 0x23, 0xac, 0x95, 0xc3, 0xb7, 0x14, 0xd6, 0x69, 0xff, 0x36, 0xef, 0xa4, 0xa9, 0xe1, 0xd6, 0x7a, 0xfd, 0x9d, 0x68, 0x91, 0xcf, 0x2d, 0xcd, 0x98, 0xc5, 0xe6, 0xf4, 0x1e, 0xde, 0xf8, 0x65, 0x6b, 0xeb, 0x80, 0x41, 0xab, 0xc7, 0x97, 0xcb, 0xbb, 0xc5, 0x5, 0x7, 0x22, 0x81, 0x58, 0x63, 0xf9, 0x67, 0xd4, 0x7c, 0xb6, 0x21, 0x17, 0xea, 0x62, 0xe3, 0xe8, 0x3f, 0x60, 0xb1, 0x51, 0x51, 0x4, 0xf2, 0x6f, 0x5c, 0x47, 0x69, 0x6b, 0xc1 },
     dsa_pub_key[] = { 0x76, 0xe, 0xf, 0x36, 0x77, 0x6, 0x9d, 0xb1, 0xf1, 0x4e, 0x9a, 0x95, 0xae, 0xc9, 0x39, 0xdf, 0x90, 0xd3, 0x94, 0x54, 0xf8, 0xf6, 0x89, 0xc8, 0x11, 0x8d, 0x2e, 0x92, 0x81, 0x6b, 0x2c, 0x37, 0x4, 0x9d, 0x97, 0x9a, 0x43, 0xa3, 0x2e, 0xed, 0x9a, 0x99, 0xb0, 0xcb, 0x9f, 0x4e, 0xea, 0x4f, 0xb7, 0xd5, 0x93, 0xc0, 0x86, 0x1b, 0xc7, 0x97, 0x5f, 0x5, 0xb4, 0xf1, 0xf9, 0xd6, 0x73, 0x97, 0x19, 0xe3, 0xc9, 0xda, 0xfc, 0x39, 0xe0, 0x37, 0xed, 0x7a, 0x62, 0xcb, 0xe3, 0x17, 0x9b, 0x64, 0x3c, 0x46, 0x86, 0x3f, 0x32, 0xec, 0x70, 0xab, 0x5b, 0x87, 0x5a, 0x6e, 0xc3, 0x37, 0xeb, 0x92, 0x58, 0x6c, 0x9e, 0x25, 0x1f, 0x37, 0x4b, 0xcd, 0xb5, 0x22, 0x62, 0x1d, 0x1b, 0x1c, 0xb1, 0x5d, 0xa1, 0xef, 0x50, 0xc3, 0x75, 0xff, 0x2, 0x24, 0x8c, 0xd7, 0x3b, 0x91, 0x77, 0xef, 0x94, 0x76 };
 
+
+
+// Backward compatibility with older openssl versions
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+
+static int DSA_set0_pqg(DSA *d, BIGNUM *p, BIGNUM *q, BIGNUM *g)
+{
+   /* If the fields p, q and g in d are NULL, the corresponding input
+    * parameters MUST be non-NULL.
+    */
+   if ((d->p == NULL && p == NULL)
+       || (d->q == NULL && q == NULL)
+       || (d->g == NULL && g == NULL))
+       return 0;
+   if (p != NULL) {
+       BN_free(d->p);
+       d->p = p;
+   }
+   if (q != NULL) {
+       BN_free(d->q);
+       d->q = q;
+   }
+   if (g != NULL) {
+       BN_free(d->g);
+       d->g = g;
+   }
+   return 1;
+}
+
+static int DSA_set0_key(DSA *d, BIGNUM *pub_key, BIGNUM *priv_key)
+{
+   /* If the field pub_key in d is NULL, the corresponding input
+    * parameters MUST be non-NULL.  The priv_key field may
+    * be left NULL.
+    */
+   if (d->pub_key == NULL && pub_key == NULL)
+       return 0;
+   if (pub_key != NULL) {
+       BN_free(d->pub_key);
+       d->pub_key = pub_key;
+   }
+   if (priv_key != NULL) {
+       BN_free(d->priv_key);
+       d->priv_key = priv_key;
+   }
+   return 1;
+}
+
+#endif
+
+
 /** Big-endian decoding of a 4-byte integer */
 static unsigned int decode_int(const unsigned char *from)
 {
@@ -156,10 +207,8 @@ static void check_license_signature(license_raw *lptr)
     SHA1_Update(&sha_ctx, lptr, sizeof(*lptr) - sizeof(lptr->dsa_signature));
     SHA1_Final(message_digest, &sha_ctx);
 
-    dsa->p = BN_bin2bn(dsa_p, sizeof(dsa_p), NULL);
-    dsa->q = BN_bin2bn(dsa_q, sizeof(dsa_q), NULL);
-    dsa->g = BN_bin2bn(dsa_g, sizeof(dsa_g), NULL);
-    dsa->pub_key = BN_bin2bn(dsa_pub_key, sizeof(dsa_pub_key), NULL);
+    DSA_set0_pqg(dsa, BN_bin2bn(dsa_p, sizeof(dsa_p), NULL), BN_bin2bn(dsa_q, sizeof(dsa_q), NULL), BN_bin2bn(dsa_g, sizeof(dsa_g), NULL));
+    DSA_set0_key(dsa, BN_bin2bn(dsa_pub_key, sizeof(dsa_pub_key), NULL), NULL);
 
   // calculate the right len of the signiture
   DSA_SIG *temp_sig=DSA_SIG_new();
@@ -489,10 +538,6 @@ void free_openssl()
     ERR_free_strings();
 }
 
-const char * openssl_version_str(void) {
-  return SSLeay_version(SSLEAY_VERSION);
-}
-
 #if defined(WIN32) || defined(INTERIX)
 
 #ifdef INTERIX
