@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -177,7 +177,8 @@ namespace Common {
     // run delayed Type::chk_coding() calls
     if (!delayed_type_enc_v.empty()) {
       for (size_t i = 0; i < delayed_type_enc_v.size(); ++i) {
-        delayed_type_enc_v[i]->t->chk_coding(delayed_type_enc_v[i]->enc, true);
+        delayed_type_enc_v[i]->t->chk_coding(delayed_type_enc_v[i]->enc,
+          delayed_type_enc_v[i]->mod, true);
         delete delayed_type_enc_v[i];
       }
       delayed_type_enc_v.clear();
@@ -303,10 +304,19 @@ namespace Common {
     }
   }
   
-  void Modules::delay_type_encode_check(Type* p_type, bool p_encode)
+  void Modules::delay_type_encode_check(Type* p_type, Module* p_module, bool p_encode)
   {
+    for (size_t i = 0; i < delayed_type_enc_v.size(); ++i) {
+      if (delayed_type_enc_v[i]->t == p_type &&
+          delayed_type_enc_v[i]->mod == p_module &&
+          delayed_type_enc_v[i]->enc == p_encode) {
+        // it's already in the list of delayed checks
+        return;
+      }
+    }
     type_enc_t* elem = new type_enc_t;
     elem->t = p_type;
+    elem->mod = p_module;
     elem->enc = p_encode;
     delayed_type_enc_v.add(elem);
   }
@@ -1897,10 +1907,10 @@ namespace Common {
     return 0;
   }
 
-  bool Assignment::get_lazy_eval() const
+  param_eval_t Assignment::get_eval_type() const
   {
-    FATAL_ERROR("Common::Assignment::get_lazy_eval()");
-    return false;
+    FATAL_ERROR("Common::Assignment::get_eval_type()");
+    return NORMAL_EVAL;
   }
 
   Ttcn::FormalParList *Assignment::get_FormalParList()
@@ -1917,6 +1927,11 @@ namespace Common {
   {
     return 0;
   }
+  
+  Type *Assignment::get_PortType()
+  {
+    return 0;
+  }
 
   void Assignment::chk_ttcn_id()
   {
@@ -1948,12 +1963,12 @@ namespace Common {
     }
     if (p_prefix) ret_val += p_prefix;
     ret_val += get_genname();
-    // add the cast to real type if its a lazy formal paramter
+    // add the cast to real type if it's a lazy or fuzzy formal parameter
     switch (asstype) {
     case A_PAR_VAL:
     case A_PAR_VAL_IN:
     case A_PAR_TEMPL_IN:
-      if (get_lazy_eval() && p_prefix==NULL) {
+      if (get_eval_type() != NORMAL_EVAL && p_prefix==NULL) {
         Type* type = get_Type();
         string type_genname = (asstype==A_PAR_TEMPL_IN) ? type->get_genname_template(p_scope) : type->get_genname_value(p_scope);
         ret_val = string("((") + type_genname + string("&)") + ret_val + string(")");
