@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -29,7 +29,7 @@ namespace Common {
   // ===== Code
   // =================================
 
-  void Code::init_output(output_struct *output)
+  void Code::init_output(output_struct *output, boolean no_alloc)
   {
     output->header.includes = NULL;
     output->header.class_decls = NULL;
@@ -65,7 +65,7 @@ namespace Common {
     output->intervals.function_bodies_max_size = 1;
     output->intervals.static_conversion_function_bodies_max_size = 1;
     output->intervals.static_function_bodies_max_size = 1;
-    if (CodeGenHelper::GetInstance().get_split_mode() == CodeGenHelper::SPLIT_TO_SLICES) {
+    if (!no_alloc && CodeGenHelper::GetInstance().get_split_mode() == CodeGenHelper::SPLIT_TO_SLICES) {
       output->intervals.methods = (size_t*)Malloc(output->intervals.methods_max_size * sizeof(size_t));
       output->intervals.function_bodies = (size_t*)Malloc(output->intervals.function_bodies_max_size * sizeof(size_t));
       output->intervals.static_conversion_function_bodies = (size_t*)Malloc(output->intervals.static_conversion_function_bodies_max_size * sizeof(size_t));
@@ -173,7 +173,7 @@ namespace Common {
     Free(output->intervals.function_bodies);
     Free(output->intervals.static_conversion_function_bodies);
     Free(output->intervals.static_function_bodies);
-    init_output(output);
+    init_output(output, TRUE);
   }
 
   void Code::init_cdef(const_def *cdef)
@@ -219,8 +219,7 @@ namespace Common {
     Free(expr->postamble);
   }
 
-  char* Code::merge_free_expr(char* str, expression_struct *expr,
-                              bool is_block)
+  char* Code::merge_free_expr(char* str, expression_struct *expr)
   {
     if (expr->preamble || expr->postamble) {
       // open a statement block if the expression has a preamble or postamble
@@ -230,9 +229,8 @@ namespace Common {
     }
     // append the expression itself
     str = mputstr(str, expr->expr);
-    // terminate it with a bracket or semi-colon
-    if (is_block) str = mputstr(str, "}\n");
-    else str = mputstr(str, ";\n");
+    // terminate it with a semi-colon
+    str = mputstr(str, ";\n");
     if (expr->preamble || expr->postamble) {
       // append the postamble if present
       if (expr->postamble) str = mputstr(str, expr->postamble);
