@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -766,7 +766,7 @@ string* remove_bom(const string& encoded_value)
   string str_uppercase(encoded_value);
   size_t enough = length > sizeof(utf32be)-1 ? sizeof(utf32be)-1 : length;
   for (size_t i = 0; i < enough; ++i) {
-    str_uppercase[i] = toupper(encoded_value[i]);
+    str_uppercase[i] = (char)toupper(encoded_value[i]);
   }
 
   if      (str_uppercase.find(utf32be, 0) < length) length_of_BOM = sizeof(utf32be)-1;
@@ -794,12 +794,12 @@ static CharCoding::CharCodingType is_ascii (size_t length, const unsigned char*
 
 static CharCoding::CharCodingType is_utf8(size_t length, const unsigned char* strptr)
 {
-  const char MSB = 1 << 7; // MSB is 1 in case of non ASCII character  
-  const char MSBmin1 = 1 << 6; // 0100 0000   
+  const unsigned char MSB = 1 << 7; // MSB is 1 in case of non ASCII character  
+  const unsigned char MSBmin1 = 1 << 6; // 0100 0000   
   size_t i = 0;
   while (length > i) {
     if ( strptr[i] & MSB) { // non ASCII char
-    char maskUTF8 = 1 << 6; // 111x xxxx shows how many additional bytes are there
+    unsigned char maskUTF8 = 1 << 6; // 111x xxxx shows how many additional bytes are there
       if (!(strptr[i] & maskUTF8)) return CharCoding::UNKNOWN; // accepted 11xxx xxxx but received 10xx xxxx
       unsigned int noofUTF8 = 0; // 11xx xxxxx -> 2 bytes, 111x xxxxx -> 3 bytes , 1111 xxxxx -> 4 bytes in UTF-8
       while (strptr[i] & maskUTF8) {
@@ -833,7 +833,7 @@ string* get_stringencoding(const string& encoded_value)
   string str_uppercase(encoded_value);
   size_t enough = length > sizeof(utf32be)-1 ? sizeof(utf32be)-1 : length;
   for (size_t i = 0; i < enough; ++i) {
-    str_uppercase[i] = toupper(encoded_value[i]);
+    str_uppercase[i] = (char)toupper(encoded_value[i]);
   }
 
   if      (str_uppercase.find(utf32be, 0) < length) return new string("UTF-32BE");
@@ -971,7 +971,7 @@ static void fill_continuing_octets(int n_continuing, unsigned char *continuing_p
   }
 }
 
-ustring decode_utf8(const string & ostr, CharCoding::CharCodingType expected_coding)
+ustring decode_utf8(const string & ostr, CharCoding::CharCodingType /*expected_coding*/)
 {
   size_t length = ostr.size();
   if (0 == length) return ustring();
