@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,6 +18,7 @@
  *   Kovacs, Ferenc
  *   Raduly, Csaba
  *   Szabados, Kristof
+ *   Szabo, Bence Janos
  *   Szabo, Janos Zoltan – initial implementation
  *   Tatarka, Gabor
  *
@@ -36,6 +37,7 @@
 #include "Int.hh"
 #include "main.hh"
 #include "ttcn3/profiler.h"
+#include "ttcn3/Attributes.hh"
 
 namespace Common {
 
@@ -481,6 +483,22 @@ namespace Common {
   // =================================
   // ===== GovernedSimple
   // =================================
+  
+  GovernedSimple::~GovernedSimple()
+  {
+    delete err_descrs;
+  }
+  
+  void GovernedSimple::add_err_descr(Ttcn::Statement* p_update_statement,
+                                     Ttcn::ErroneousDescriptor* p_err_descr)
+  {
+    if (p_err_descr != NULL) {
+      if (err_descrs == NULL) {
+        err_descrs = new Ttcn::ErroneousDescriptors;
+      }
+      err_descrs->add(p_update_statement, p_err_descr);
+    }
+  }
 
   string GovernedSimple::get_lhs_name() const
   {
@@ -554,6 +572,12 @@ namespace Common {
     if (parent_scope) return parent_scope->get_scope_runs_on();
     else return 0;
   }
+  
+  Ttcn::PortScope *Scope::get_scope_port()
+  {
+    if (parent_scope) return parent_scope->get_scope_port();
+    else return 0;
+  }
 
   Assignments *Scope::get_scope_asss()
   {
