@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -557,7 +557,7 @@ namespace Asn {
     json.put_next_token(JSON_TOKEN_OBJECT_END);
   }
   
-  void Module::generate_debugger_init(output_struct *output)
+  void Module::generate_debugger_init(output_struct */*output*/)
   {
     // no debugging in ASN.1 modules
   }
@@ -931,7 +931,7 @@ namespace Asn {
 
   void Assignments::dump(unsigned level) const
   {
-    DEBUG(level, "Assignments (%lu pcs.)", (unsigned long) asss_v.size());
+    DEBUG(level, "Assignments (%lu pcs.)", static_cast<unsigned long>( asss_v.size()));
     for(size_t i = 0; i < asss_v.size(); i++) asss_v[i]->dump(level + 1);
   }
 
