@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -241,7 +241,7 @@ void defEnumClass(const enum_def *edef, output_struct *output)
   /* Conversion function: enum_to_str */
   def = mputprintf(def, "static const char *enum_to_str(%s enum_par%s);\n",
     enum_type,
-    edef->xerText ? ", boolean txt = false" : "");
+    edef->xerText ? ", boolean txt = FALSE" : "");
   src = mputprintf(src, "const char *%s::enum_to_str(%s enum_par%s)\n"
     "{\n"
     "switch (enum_par) {\n", name, enum_type,
@@ -655,7 +655,7 @@ void defEnumClass(const enum_def *edef, output_struct *output)
      */
     src = mputprintf(src,
       "boolean %s::can_start(const char *name, const char *uri, "
-      "const XERdescriptor_t& xd, unsigned int flavor) {\n"
+      "const XERdescriptor_t& xd, unsigned int flavor, unsigned int) {\n"
       "  boolean exer = is_exer(flavor);\n"
       "  return check_name(name, xd, exer) && (!exer || check_namespace(uri, xd));\n"
       "}\n\n"
@@ -664,12 +664,12 @@ void defEnumClass(const enum_def *edef, output_struct *output)
 
     src = mputprintf(src,
       "int %s::XER_encode(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,"
-      " unsigned int p_flavor, int p_indent, embed_values_enc_struct_t*) const\n"
+      " unsigned int p_flavor, unsigned int, int p_indent, embed_values_enc_struct_t*) const\n"
       "{\n"
       "  int encoded_length=(int)p_buf.get_len();\n"
       "  const boolean e_xer = is_exer(p_flavor);\n"
       "  p_flavor |= (SIMPLE_TYPE | BXER_EMPTY_ELEM);\n"
-      "  if (begin_xml(p_td, p_buf, p_flavor, p_indent, false) == -1) "
+      "  if (begin_xml(p_td, p_buf, p_flavor, p_indent, FALSE) == -1) "
       "--encoded_length;\n"
       "  if (!e_xer) p_buf.put_c('<');\n"
       , name
@@ -690,7 +690,7 @@ void defEnumClass(const enum_def *edef, output_struct *output)
       "    p_buf.put_s(strlen(enumval), (const unsigned char*)enumval);\n"
       "  }\n"
       "  if (!e_xer) p_buf.put_s(2, (const unsigned char*)\"/>\");\n"
-      "  end_xml(p_td, p_buf, p_flavor, p_indent, false);\n"
+      "  end_xml(p_td, p_buf, p_flavor, p_indent, FALSE);\n"
       , edef->xerText ? ", e_xer" : ""
     );
     src = mputstr(src,
@@ -706,7 +706,7 @@ void defEnumClass(const enum_def *edef, output_struct *output)
       "{\n"
       "  int rd_ok = 1, type;\n"
 
-      "  const int e_xer = is_exer(p_flavor);\n"
+      "  const boolean e_xer = is_exer(p_flavor);\n"
       "  const boolean name_tag = !((!e_xer && is_record_of(p_flavor)) || (e_xer && ((p_td.xer_bits & UNTAGGED) ||(is_record_of(p_flavor) && is_exerlist(p_flavor)))));\n"
       "  if (e_xer && ((p_td.xer_bits & XER_ATTRIBUTE) || is_exerlist(p_flavor))) {\n"
       "    if ((p_td.xer_bits & XER_ATTRIBUTE)) verify_name(p_reader, p_td, e_xer);\n"
@@ -809,12 +809,12 @@ void defEnumClass(const enum_def *edef, output_struct *output)
       "  json_token_t token = JSON_TOKEN_NONE;\n"
       "  char* value = 0;\n"
       "  size_t value_len = 0;\n"
-      "  boolean error = false;\n"
-      "  int dec_len = 0;\n"
+      "  boolean error = FALSE;\n"
+      "  size_t dec_len = 0;\n"
       "  boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();\n"
       "  if (use_default) {\n"
       // No JSON data in the buffer -> use default value
-      "    value = (char*)p_td.json->default_value;\n"
+      "    value = const_cast<char*>(p_td.json->default_value);\n"
       "    value_len = strlen(value);\n"
       "  } else {\n"
       "    dec_len = p_tok.get_next_token(&token, &value, &value_len);\n"
@@ -829,10 +829,10 @@ void defEnumClass(const enum_def *edef, output_struct *output)
       "      enum_value = str_to_enum(value + (use_default ? 0 : 1));\n"
       "      if (!use_default) value[value_len - 1] = '\\\"';\n"
       "      if (%s == enum_value) {\n"
-      "        error = true;\n"
+      "        error = TRUE;\n"
       "      }\n"
       "    } else {\n"
-      "      error = true;\n"
+      "      error = TRUE;\n"
       "    }\n"
       "  } else {\n"
       "    enum_value = %s;\n"
@@ -843,7 +843,7 @@ void defEnumClass(const enum_def *edef, output_struct *output)
       "    enum_value = %s;\n"
       "    return JSON_ERROR_FATAL;\n"
       "  }\n"
-      "  return dec_len;\n"
+      "  return (int)dec_len;\n"
       "}\n\n"
       , name, unknown_value, unbound_value, unbound_value);
   }
