@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -69,6 +69,8 @@
 
 #include "ttcn3/profiler.h"
 
+#include "../common/openssl_version.h"
+
 #ifdef LICENSE
 #include "../common/license.h"
 #endif
@@ -77,8 +79,8 @@ using namespace Common;
 
 const char *output_dir = NULL;
 const char *tcov_file_name = NULL;
-const char *profiler_file_name = NULL;
-const char *file_list_file_name = NULL;
+static const char *profiler_file_name = NULL;
+static const char *file_list_file_name = NULL;
 tcov_file_list *tcov_files = NULL;
 expstring_t effective_module_lines = NULL;
 expstring_t effective_module_functions = NULL;
@@ -96,7 +98,8 @@ boolean generate_skeleton = FALSE, force_overwrite = FALSE,
   implicit_json_encoding = FALSE, json_refs_for_all_types = TRUE,
   force_gen_seof = FALSE, omit_in_value_list = FALSE,
   warnings_for_bad_variants = FALSE, debugger_active = FALSE,
-  legacy_unbound_union_fields = FALSE, split_to_slices = FALSE;
+  legacy_unbound_union_fields = FALSE, split_to_slices = FALSE,
+  legacy_untagged_union;
 
 // Default code splitting mode is set to 'no splitting'.
 CodeGenHelper::split_type code_splitting_mode = CodeGenHelper::SPLIT_NONE;
@@ -385,7 +388,7 @@ static boolean is_valid_asn1_filename(const char* file_name)
 static void usage()
 {
   fprintf(stderr, "\n"
-    "usage: %s [-abcdEfgijlLMnOpqrRsStuwxXyY] [-J file] [-K file] [-z file] [-V verb_level]\n"
+    "usage: %s [-abcdEfgijlLMnNOpqrRsStuwxXyY] [-J file] [-K file] [-z file] [-V verb_level]\n"
     "	[-o dir] [-U none|type|'number'] [-P modulename.top_level_pdu_name] [-Q number] ...\n"
     "	[-T] module.ttcn [-A] module.asn ...\n"
     "	or  %s -v\n"
@@ -408,6 +411,7 @@ static void usage()
     "	-L:		add source line info for logging\n"
     "	-M:		allow 'omit' in template value lists (legacy behavior)\n"
     "	-n:		activate debugger (generates extra code for debugging)\n"
+    "	-N:		ignore UNTAGGED encoding instruction on top level unions (legacy behaviour)\n"
     "	-o dir:		output files will be placed into dir\n"
     "	-p:		parse only (no semantic check or code generation)\n"
     "	-P pduname:	define top-level pdu\n"
@@ -485,7 +489,7 @@ int main(int argc, char *argv[])
     s0flag = false, Cflag = false, yflag = false, Uflag = false, Qflag = false,
     Sflag = false, Kflag = false, jflag = false, zflag = false, Fflag = false,
     Mflag = false, Eflag = false, nflag = false, Bflag = false, errflag = false,
-    print_usage = false, ttcn2json = false; 
+    print_usage = false, ttcn2json = false, Nflag = false; 
 
   CodeGenHelper cgh;
 
@@ -579,7 +583,7 @@ int main(int argc, char *argv[])
 
   if (!ttcn2json) {
     for ( ; ; ) {
-      int c = getopt(argc, argv, "aA:bBcC:dEfFgijJ:K:lLMno:pP:qQ:rRsStT:uU:vV:wxXyYz:0-");
+      int c = getopt(argc, argv, "aA:bBcC:dEfFgijJ:K:lLMnNo:pP:qQ:rRsStT:uU:vV:wxXyYz:0-");
       if (c == -1) break;
       switch (c) {
       case 'a':
@@ -757,6 +761,10 @@ int main(int argc, char *argv[])
         SET_FLAG(n);
         debugger_active = TRUE;
         break;
+      case 'N':
+        SET_FLAG(N);
+        legacy_untagged_union = TRUE;
+        break;
       case 'B':
         SET_FLAG(B);
         legacy_unbound_union_fields = TRUE;
@@ -919,12 +927,12 @@ int main(int argc, char *argv[])
     fputs("TTCN-3 and ASN.1 Compiler for the TTCN-3 Test Executor\n"
 	  "Product number: " PRODUCT_NUMBER "\n"
 	  "Build date: " __DATE__ " " __TIME__ "\n"
-	  "Compiled with: " C_COMPILER_VERSION "\n\n"
-	  COPYRIGHT_STRING "\n\n", stderr);
+	  "Compiled with: " C_COMPILER_VERSION "\n", stderr);
+	  fputs("Using ", stderr);
+	  fputs(openssl_version_str(), stderr);
+	  fputs("\n\n" COPYRIGHT_STRING "\n\n", stderr);
 #ifdef LICENSE
     print_license_info();
-    fputs("\nUsing ", stderr);
-    fputs(openssl_version_str(), stderr);
     fputs("\n\n", stderr);
 #endif
     return EXIT_SUCCESS;
@@ -1152,6 +1160,11 @@ int main(int argc, char *argv[])
   unsigned int error_count = Error_Context::get_error_count();
   if (error_count > 0) ret_val = EXIT_FAILURE;
 
+#ifdef USAGE_STATS
+  pthread_t stats_thread = 0;
+  thread_data* stats_data = NULL;
+#endif
+  
   if (parse_only || semantic_check_only) {
     // print detailed statistics
     Error_Context::print_error_statistics();
@@ -1184,8 +1197,9 @@ int main(int argc, char *argv[])
         }
       }
 
-      HttpSender *sender = new HttpSender;
-      UsageData::getInstance().sendDataThreaded(stream.str(), sender);
+      stats_data = new thread_data;
+      stats_data->sndr = new HttpSender;
+      stats_thread = UsageData::getInstance().sendDataThreaded(stream.str(), stats_data);
 #endif
       if (ttcn2json) {
         NOTIFY("Generating JSON schema...");
@@ -1233,5 +1247,19 @@ int main(int argc, char *argv[])
 
   // dbgnew.hh already does it: check_mem_leak(argv[0]);
 
+#ifdef USAGE_STATS
+  if (stats_thread != 0) {
+    // cancel the usage stats thread if it hasn't finished yet
+    pthread_cancel(stats_thread);
+    pthread_join(stats_thread, NULL);
+  }
+  if (stats_data != NULL) {
+    if (stats_data->sndr != NULL) {
+      delete stats_data->sndr;
+    }
+    delete stats_data;
+  }
+#endif
+  
   return ret_val;
 }
