@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -23,6 +23,7 @@
 #include "../Type.hh"
 #include "../main.hh"
 #include "TtcnTemplate.hh"
+#include "Statement.hh"
 
 namespace Ttcn {
 
@@ -144,7 +145,7 @@ namespace Ttcn {
   void Qualifiers::dump(unsigned level) const
   {
     DEBUG(level, "has %lu qualifiers",
-        (unsigned long)get_nof_qualifiers());
+        static_cast<unsigned long>( get_nof_qualifiers() ));
     for(size_t i = 0; i < qualifiers.size(); i++)
       qualifiers[i]->dump(level);
   }
@@ -226,7 +227,7 @@ namespace Ttcn {
     return (tmpl_inst->get_Template()->get_templatetype()==Template::OMIT_VALUE);
   }
 
-  void ErroneousAttributeSpec::chk()
+  void ErroneousAttributeSpec::chk(bool in_update_stmt)
   {
     if (get_is_omit()) { // special case, no type needed
       if ((indicator==I_BEFORE)||(indicator==I_AFTER)) {
@@ -290,7 +291,10 @@ namespace Ttcn {
       value = templ->get_Value();
       value->set_my_governor(type);
       type->chk_this_value_ref(value);
-      type->chk_this_value(value, 0, Type::EXPECTED_CONSTANT,
+      // dynamic values are allowed for attribute specs in '@update' statements,
+      // but not for the initial attribute specs of constants and templates
+      type->chk_this_value(value, 0, in_update_stmt ?
+        Type::EXPECTED_DYNAMIC_VALUE : Type::EXPECTED_CONSTANT,
         INCOMPLETE_NOT_ALLOWED, OMIT_NOT_ALLOWED, SUB_CHK);
       //{ FIXME: make this work
       //  ReferenceChain refch(type, "While checking embedded recursions");
@@ -315,21 +319,21 @@ namespace Ttcn {
     return "";
   }
 
-  char* ErroneousAttributeSpec::generate_code_str(char *str, char *& def, string genname, const bool embedded)
+  char* ErroneousAttributeSpec::generate_code_str(char *str, char *& def, string genname)
   {
     if (get_is_omit()) return str;
     if (!type) FATAL_ERROR("ErroneousAttributeSpec::generate_code_str()");
     if (!value) FATAL_ERROR("ErroneousAttributeSpec::generate_code_str()");
     if (first_genname.empty()) { // this is the first use
-      str = mputprintf(str, "%s%s %s;\n", split_to_slices && !embedded ? "" : "static ",
+      str = mputprintf(str, "%s%s %s;\n", split_to_slices ? "" : "static ",
         type->get_genname_value(value->get_my_scope()).c_str(), genname.c_str());
       first_genname = genname;
-      if (split_to_slices && !embedded) {
+      if (split_to_slices) {
         def = mputprintf(def, "extern %s %s;\n",
           type->get_genname_value(value->get_my_scope()).c_str(), genname.c_str());
       }
     } else {
-      str = mputprintf(str, "%s%s& %s = %s;\n", split_to_slices && !embedded ? "" : "static ",
+      str = mputprintf(str, "%s& %s = %s;\n",
         type->get_genname_value(value->get_my_scope()).c_str(),
         genname.c_str(), first_genname.c_str());
     }
@@ -364,22 +368,23 @@ namespace Ttcn {
 
   // ==== ErroneousValues ====
 
-  char* ErroneousValues::generate_code_embedded_str(char *str, char *& def, string genname, const bool embedded)
+  char* ErroneousValues::generate_code_embedded_str(char *str, char *& def, string genname)
   {
-    if (before) str = generate_code_embedded_str(str, def, genname+"_before", before, embedded);
-    if (value) str = generate_code_embedded_str(str, def, genname+"_value", value, embedded);
-    if (after) str = generate_code_embedded_str(str, def, genname+"_after", after, embedded);
+    if (before) str = generate_code_embedded_str(str, def, genname+"_before", before);
+    if (value) str = generate_code_embedded_str(str, def, genname+"_value", value);
+    if (after) str = generate_code_embedded_str(str, def, genname+"_after", after);
     return str;
   }
 
-  char* ErroneousValues::generate_code_embedded_str(char *str, char *& def, string genname, ErroneousAttributeSpec* attr_spec, const bool embedded)
+  char* ErroneousValues::generate_code_embedded_str(char *str, char *& def, string genname, ErroneousAttributeSpec* attr_spec)
   {
-    str = attr_spec->generate_code_str(str, def, genname+"_errval", embedded);
-    str = mputprintf(str, "%sErroneous_value_t %s = { %s, %s, %s };\n", split_to_slices && !embedded ? "" : "static ", genname.c_str(),
+    str = attr_spec->generate_code_str(str, def, genname+"_errval");
+    str = mputprintf(str, "%sErroneous_value_t %s = { %s, %s, %s };\n",
+      split_to_slices ? "" : "static ", genname.c_str(),
       attr_spec->get_is_raw() ? "true" : "false",
       attr_spec->get_is_omit() ? "NULL" : ("&"+genname+"_errval").c_str(),
       attr_spec->get_typedescriptor_str().c_str());
-    if (split_to_slices && !embedded) {
+    if (split_to_slices) {
       def = mputprintf(def, "extern Erroneous_value_t %s;\n", genname.c_str());
     }
     return str;
@@ -412,40 +417,42 @@ namespace Ttcn {
 
   // ==== ErroneousDescriptor ====
 
-  char* ErroneousDescriptor::generate_code_embedded_str(char *str, char *& def, string genname, const bool embedded)
+  char* ErroneousDescriptor::generate_code_embedded_str(char *str, char *& def, string genname)
   {
     // values
     for (size_t i=0; i<values_m.size(); i++) {
-      str = values_m.get_nth_elem(i)->generate_code_embedded_str(str, def, genname+"_v"+Int2string((int)values_m.get_nth_key(i)), embedded);
+      str = values_m.get_nth_elem(i)->generate_code_embedded_str(str, def, genname+"_v"+Int2string(static_cast<int>( values_m.get_nth_key(i) )));
     }
     // embedded descriptors
     for (size_t i=0; i<descr_m.size(); i++) {
-      str = descr_m.get_nth_elem(i)->generate_code_embedded_str(str, def, genname+"_d"+Int2string((int)descr_m.get_nth_key(i)), embedded);
+      str = descr_m.get_nth_elem(i)->generate_code_embedded_str(str, def, genname+"_d"+Int2string(static_cast<int>( descr_m.get_nth_key(i) )));
     }
     // values vector
     if (values_m.size()>0) {
-      str = mputprintf(str, "%sErroneous_values_t %s_valsvec[%d] = { ", split_to_slices && !embedded ? "" : "static ", genname.c_str(), (int)values_m.size());
+      str = mputprintf(str, "%sErroneous_values_t %s_valsvec[%d] = { ",
+        split_to_slices ? "" : "static ", genname.c_str(), static_cast<int>( values_m.size() ));
       for (size_t i=0; i<values_m.size(); i++) {
         if (i>0) str = mputstr(str, ", ");
-        int key_i = (int)values_m.get_nth_key(i);
+        int key_i = static_cast<int>( values_m.get_nth_key(i) );
         str = values_m.get_nth_elem(i)->generate_code_struct_str(str, genname+"_v"+Int2string(key_i), key_i);
       }
       str = mputstr(str, " };\n");
-      if (split_to_slices && !embedded) {
-        def = mputprintf(def, "extern Erroneous_values_t %s_valsvec[%d];\n", genname.c_str(), (int)values_m.size());
+      if (split_to_slices) {
+        def = mputprintf(def, "extern Erroneous_values_t %s_valsvec[%d];\n", genname.c_str(), static_cast<int>( values_m.size() ));
       }
     }
     // embedded descriptor vector
     if (descr_m.size()>0) {
-      str = mputprintf(str, "%sErroneous_descriptor_t %s_embvec[%d] = { ", split_to_slices && !embedded ? "" : "static ", genname.c_str(), (int)descr_m.size());
+      str = mputprintf(str, "%sErroneous_descriptor_t %s_embvec[%d] = { ",
+        split_to_slices ? "" : "static ", genname.c_str(), static_cast<int>( descr_m.size() ));
       for (size_t i=0; i<descr_m.size(); i++) {
         if (i>0) str = mputstr(str, ", ");
-        int key_i = (int)descr_m.get_nth_key(i);
+        int key_i = static_cast<int>( descr_m.get_nth_key(i) );
         str = descr_m.get_nth_elem(i)->generate_code_struct_str(str, def, genname+"_d"+Int2string(key_i), key_i);
       }
       str = mputstr(str, " };\n");
-      if (split_to_slices && !embedded) {
-        def = mputprintf(def, "extern Erroneous_descriptor_t %s_embvec[%d];\n", genname.c_str(), (int)descr_m.size());
+      if (split_to_slices) {
+        def = mputprintf(def, "extern Erroneous_descriptor_t %s_embvec[%d];\n", genname.c_str(), static_cast<int>( descr_m.size() ));
       }
     }
     return str;
@@ -454,15 +461,15 @@ namespace Ttcn {
   char* ErroneousDescriptor::generate_code_init_str(char *str, string genname)
   {
     for (size_t i=0; i<values_m.size(); i++) {
-      str = values_m.get_nth_elem(i)->generate_code_init_str(str, genname+"_v"+Int2string((int)values_m.get_nth_key(i)));
+      str = values_m.get_nth_elem(i)->generate_code_init_str(str, genname+"_v"+Int2string(static_cast<int>( values_m.get_nth_key(i) )));
     }
     for (size_t i=0; i<descr_m.size(); i++) {
-      str = descr_m.get_nth_elem(i)->generate_code_init_str(str, genname+"_d"+Int2string((int)descr_m.get_nth_key(i)));
+      str = descr_m.get_nth_elem(i)->generate_code_init_str(str, genname+"_d"+Int2string(static_cast<int>( descr_m.get_nth_key(i) )));
     }
     return str;
   }
 
-  char* ErroneousDescriptor::generate_code_struct_str(char *str, char *& def, string genname, int field_index)
+  char* ErroneousDescriptor::generate_code_struct_str(char *str, char *& /*def*/, string genname, int field_index)
   {
     string genname_values_vec = genname + "_valsvec";
     string genname_embedded_vec = genname + "_embvec";
@@ -470,8 +477,8 @@ namespace Ttcn {
       field_index,
       omit_before, (omit_before==-1)?"NULL":("\""+omit_before_name+"\"").c_str(),
       omit_after,  (omit_after==-1)?"NULL":("\""+omit_after_name+"\"").c_str(),
-      (int)values_m.size(), (values_m.size()>0) ? genname_values_vec.c_str(): "NULL",
-      (int)descr_m.size(), (descr_m.size()>0) ? genname_embedded_vec.c_str(): "NULL");
+      static_cast<int>( values_m.size() ), (values_m.size()>0) ? genname_values_vec.c_str(): "NULL",
+      static_cast<int>( descr_m.size() ), (descr_m.size()>0) ? genname_embedded_vec.c_str(): "NULL");
     return str;
   }
 
@@ -486,18 +493,83 @@ namespace Ttcn {
     }
   }
 
-  char* ErroneousDescriptor::generate_code_str(char *str, char *& def, string genname, const bool embedded)
+  char* ErroneousDescriptor::generate_code_str(char *str, char *& def, string genname)
   {
     genname += "_err_descr";
-    str = generate_code_embedded_str(str, def, genname, embedded);
-    str = mputprintf(str, "%sErroneous_descriptor_t %s = ", split_to_slices && !embedded ? "" : "static ", genname.c_str());
+    str = generate_code_embedded_str(str, def, genname);
+    str = mputprintf(str, "%sErroneous_descriptor_t %s = ",
+      split_to_slices ? "" : "static ", genname.c_str());
     str = generate_code_struct_str(str, def, genname, -1);
     str = mputstr(str, ";\n");
-    if (split_to_slices && !embedded) {
+    if (split_to_slices) {
       def = mputprintf(def, "extern Erroneous_descriptor_t %s;\n", genname.c_str());
     }
     return str;
   }
+  
+  ErroneousDescriptors::~ErroneousDescriptors()
+  {
+    descr_map.clear();
+  }
+  
+  void ErroneousDescriptors::add(Statement* p_update_statement, ErroneousDescriptor* p_descr)
+  {
+    descr_map.add(p_update_statement, p_descr);
+  }
+  
+  bool ErroneousDescriptors::has_descr(Statement* p_update_statement)
+  {
+    return descr_map.has_key(p_update_statement);
+  }
+  
+  size_t ErroneousDescriptors::get_descr_index(Statement* p_update_statement)
+  {
+    return descr_map.find_key(p_update_statement);
+  }
+  
+  char* ErroneousDescriptors::generate_code_init_str(Statement* p_update_statement, char *str, string genname)
+  {
+    size_t i = descr_map.find_key(p_update_statement);
+    return descr_map.get_nth_elem(i)->generate_code_init_str(str,
+      genname + string("_") + Int2string(static_cast<int>(i)) + string("_err_descr"));
+  }
+  
+  char* ErroneousDescriptors::generate_code_str(Statement* p_update_statement, char *str, char *& def, string genname)
+  {
+    size_t i = descr_map.find_key(p_update_statement);
+    return descr_map.get_nth_elem(i)->generate_code_str(str, def,
+      genname + string("_") + Int2string(static_cast<int>(i)));
+  }
+  
+  void ErroneousDescriptors::chk_recursions(ReferenceChain& refch)
+  {
+    for (size_t i = 0; i < descr_map.size(); ++i) {
+      descr_map.get_nth_elem(i)->chk_recursions(refch);
+    }
+  }
+  
+  boolean ErroneousDescriptors::can_have_err_attribs(Type* t)
+  {
+    if (t == NULL) {
+      FATAL_ERROR("ErroneousDescriptors::can_have_err_attribs");
+    }
+    t = t->get_type_refd_last();
+    switch (t->get_typetype_ttcn3()) {
+    case Type::T_SEQ_T:
+    case Type::T_SET_T:
+      if (t->get_nof_comps() == 0) {
+        // empty records/sets can't have erroneous attributes
+        return FALSE;
+      }
+      // else fall through
+    case Type::T_SEQOF:
+    case Type::T_SETOF:
+    case Type::T_CHOICE_T:
+      return use_runtime_2;
+    default:
+      return FALSE;
+    }
+  }
 
   // ==== ErroneousAttributes ====
 
@@ -567,8 +639,8 @@ namespace Ttcn {
         if (act_attr->get_is_raw()) {
           switch (ti_type->get_typetype_ttcn3()) {
           case Type::T_BSTR:
-            if (!type->has_encoding(Type::CT_PER) && !type->has_encoding(Type::CT_RAW)) {
-              act_attr->error("A `raw' %s value was used for erroneous type `%s' which has no RAW or PER encodings.",
+            if (!type->has_encoding(Type::CT_RAW)) {
+              act_attr->error("A `raw' %s value was used for erroneous type `%s' which has no RAW encoding.",
                               ti_type->get_typename().c_str(), type->get_typename().c_str());
             }
             break;
@@ -581,7 +653,7 @@ namespace Ttcn {
             break;
           case Type::T_USTR:
             if (!type->has_encoding(Type::CT_XER) && !type->has_encoding(Type::CT_JSON)) {
-              act_attr->error("A `raw' %s value was used for erroneous type `%s' which has no XER or JSON encoding.",
+              act_attr->error("A `raw' %s value was used for erroneous type `%s' which has no XER or JSON encodings.",
                               ti_type->get_typename().c_str(), type->get_typename().c_str());
             }
             break;
@@ -591,7 +663,6 @@ namespace Ttcn {
         } else {
           // the two types must have at least one common encoding
           if (!((type->has_encoding(Type::CT_BER)&&ti_type->has_encoding(Type::CT_BER)) ||
-                (type->has_encoding(Type::CT_PER)&&ti_type->has_encoding(Type::CT_PER)) ||
                 (type->has_encoding(Type::CT_RAW)&&ti_type->has_encoding(Type::CT_RAW)) ||
                 (type->has_encoding(Type::CT_TEXT)&&ti_type->has_encoding(Type::CT_TEXT)) ||
                 (type->has_encoding(Type::CT_XER)&&ti_type->has_encoding(Type::CT_XER)) ||
@@ -621,22 +692,22 @@ namespace Ttcn {
     map< size_t, dynamic_array<field_err_t> > embedded_fld_array_m; // used for recursive calls
     for (size_t i=0; i<fld_array.size(); i++) {
       field_err_t& act_field_err = fld_array[i];
-      ErroneousAttributeSpec::indicator_t act_indicator = act_field_err.err_attr->get_indicator();
-      bool is_omit = act_field_err.err_attr->get_is_omit();
       if (act_field_err.subrefs_array.size()<1) FATAL_ERROR("ErroneousAttributes::build_descr_tree()");
       size_t fld_idx = act_field_err.subrefs_array[0];
-      if (omit_before_qual && (err_descr->omit_before!=-1) && (err_descr->omit_before>(int)fld_idx)) {
+      if (omit_before_qual && (err_descr->omit_before!=-1) && (err_descr->omit_before>static_cast<int>(fld_idx))) {
         act_field_err.qualifier->error(
           "Field `%s' cannot be referenced because all fields before field `%s' have been omitted",
           act_field_err.qualifier->get_stringRepr().c_str(), omit_before_qual->get_stringRepr().c_str());
         continue;
       }
-      if (omit_after_qual && (err_descr->omit_after!=-1) && (err_descr->omit_after<(int)fld_idx)) {
+      if (omit_after_qual && (err_descr->omit_after!=-1) && (err_descr->omit_after<static_cast<int>(fld_idx))) {
         act_field_err.qualifier->error(
           "Field `%s' cannot be referenced because all fields after field `%s' have been omitted",
           act_field_err.qualifier->get_stringRepr().c_str(), omit_after_qual->get_stringRepr().c_str());
         continue;
       }
+      ErroneousAttributeSpec::indicator_t act_indicator = act_field_err.err_attr->get_indicator();
+      bool is_omit = act_field_err.err_attr->get_is_omit();
       if (act_field_err.subrefs_array.size()==1) { // erroneous value
         if (act_field_err.type_array.size()!=1) FATAL_ERROR("ErroneousAttributes::build_descr_tree()");
         if ((act_field_err.type_array[0]->get_typetype()==Type::T_SET_A) &&
@@ -960,7 +1031,7 @@ namespace Ttcn {
   void MultiWithAttrib::dump(unsigned level) const
   {
     DEBUG(level,"with attrib parameters (%lu pcs)",
-      (unsigned long) elements.size());
+      static_cast<unsigned long>( elements.size() ));
     for(size_t i = 0; i < elements.size(); i++)
     {
       elements[i]->dump(level + 1);
@@ -1017,13 +1088,30 @@ namespace Ttcn {
 
     attributes_checked = true;
   }
+  
+  void WithAttribPath::chk_only_erroneous()
+  {
+    if (attributes_checked || m_w_attrib == NULL) {
+      return;
+    }
+
+    for (size_t i = 0; i < m_w_attrib->get_nof_elements(); ++i) {
+      const SingleWithAttrib* attrib = m_w_attrib->get_element(i);
+      if (attrib->get_attribKeyword() != SingleWithAttrib::AT_ERRONEOUS) {
+        attrib->error("Only `erroneous' attributes are allowed in an `@update' "
+          "statement");
+      }
+    }
+
+    attributes_checked = true;
+  }
 
   void WithAttribPath::dump(unsigned int level) const
   {
     DEBUG(level, "WithAttribPath");
     if (!m_w_attrib) return;
     DEBUG(level+1, "%lu elements",
-      (unsigned long)m_w_attrib->get_nof_elements());
+      static_cast<unsigned long>( m_w_attrib->get_nof_elements() ));
     for (size_t i=0; i < m_w_attrib->get_nof_elements(); ++i) {
       const SingleWithAttrib* a = m_w_attrib->get_element(i);
       if (!a) continue;
@@ -1067,6 +1155,12 @@ namespace Ttcn {
       {
         case SingleWithAttrib::AT_ENCODE:
         {
+          if (temp_attrib->get_attribQualifiers() != NULL &&
+              temp_attrib->get_attribQualifiers()->get_nof_qualifiers() != 0) {
+            // TODO: check the attributes grouped per qualifier (for now
+            // ignoring this check for attributes with qualifiers is enough)
+            break;
+          }
           if(has_encode)
           {
             temp_attrib->warning("Only the last encode "
