@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,6 +12,7 @@
  *   Delic, Adam
  *   Kremer, Peter
  *   Raduly, Csaba
+ *   Szabo, Bence Janos
  *   Szabo, Janos Zoltan – initial implementation
  *
  ******************************************************************************/
@@ -47,6 +48,7 @@ typedef struct port_msg_type_mapping_target_tag {
   msg_mapping_type_t mapping_type;
   union {
     struct {
+      const char *dispname;
       const char *name;
       function_prototype_t prototype;
     } function;
@@ -88,6 +90,17 @@ typedef enum testport_type_t { NORMAL, INTERNAL, ADDRESS } testport_type_t;
 
 typedef enum port_type_t { REGULAR, PROVIDER, USER } port_type_t;
 
+typedef struct port_msg_provider {
+  const char *name; // provider type's name
+  size_t n_out_msg_type_names;
+  const char **out_msg_type_names; // provider's out message list
+} port_msg_prov;
+
+typedef struct port_msg_provider_list {
+  size_t nElements;
+  port_msg_prov *elements;
+} port_msg_prov_list;
+
 typedef struct port_def_tag {
   const char *name;
   const char *dispname;
@@ -101,9 +114,16 @@ typedef struct port_def_tag {
   port_proc_signature_list proc_out; /* from PortTypeBody::out_sigs */
   testport_type_t testport_type;
   port_type_t port_type;
-  const char *provider_name;
+  port_msg_prov_list provider_msg_outlist;
+  const char **mapper_name;
+  size_t n_mapper_name;
   port_msg_mapped_type_list provider_msg_in;
   boolean has_sliding;
+  boolean legacy;
+  char *var_decls;
+  char *var_defs;
+  char *mapping_func_decls;
+  char *mapping_func_defs;
 } port_def;
 
 #ifdef __cplusplus
