@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -84,7 +84,7 @@ const char* findLibraryName(const char* libraryName, const char* projName);
 #ifdef __cplusplus
 extern "C"
 #endif
-void erase_libs();
+void erase_libs(void);
 
 #ifdef __cplusplus
 extern "C"
@@ -124,7 +124,7 @@ boolean hasSubProject(const char* projName);
 #ifdef __cplusplus
 extern "C"
 #endif
-void print_libs();
+void print_libs(void);
 
 #ifdef __cplusplus
 extern "C"
@@ -270,9 +270,9 @@ extern "C"
 #else
 enum
 #endif
-tpd_result process_tpd(const char *p_tpd_name, const char *actcfg,
+tpd_result process_tpd(const char **p_tpd_name, const char *actcfg,
   const char *file_list_path,
-  int *argc, char ***argv,
+  int *argc, char ***argv, boolean* p_free_argv,
   int *optind, char **ets_name, char **project_name,
   boolean *gnu_make, boolean *single_mode,
   boolean *central_storage, boolean *absolute_paths,
@@ -282,11 +282,11 @@ tpd_result process_tpd(const char *p_tpd_name, const char *actcfg,
   const char *output_file, char** abs_work_dir, struct string_list* sub_project_dirs,
   const char* program_name, FILE* prj_graph_fp, struct string2_list* create_symlink_list, struct string_list* ttcn3_prep_includes,
   struct string_list* ttcn3_prep_defines, struct string_list* ttcn3_prep_undefines, struct string_list* prep_includes,
-  struct string_list* prep_defines, struct string_list* prep_undefines, boolean *codesplit, boolean *quietly, boolean *disablesubtypecheck,
+  struct string_list* prep_defines, struct string_list* prep_undefines, char **codesplit, boolean *quietly, boolean *disablesubtypecheck,
   char** cxxcompiler, char** optlevel, char** optflags, boolean *disableber, boolean *disableraw, boolean *disabletext, boolean *disablexer,
   boolean *disablejson, boolean *forcexerinasn, boolean *defaultasomit, boolean *gccmessageformat,
   boolean *linenumber, boolean *includesourceinfo, boolean *addsourcelineinfo, boolean *suppresswarnings,
-  boolean *outparamboundness, boolean *omit_in_value_list, boolean *warnings_for_bad_variants, boolean *activate_debugger,
+  boolean *outparamboundness, boolean *omit_in_value_list, boolean *warnings_for_bad_variants, boolean *activate_debugger, boolean* ignore_untagged_union,
   boolean *disable_predef_exp_folder, struct string_list* solspeclibs, struct string_list* sol8speclibs,
   struct string_list* linuxspeclibs, struct string_list* freebsdspeclibs, struct string_list* win32speclibs,
   char** ttcn3preprocessor, struct string_list* linkerlibs, struct string_list* additionalObjects, struct string_list* linkerlibsearchpath, boolean Vflag, boolean Dflag,
