@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -90,7 +90,7 @@ void Base_Type::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
     if(!p_td.xer) TTCN_EncDec_ErrorContext::error_internal(
       "No XER descriptor available for type '%s'.", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*(p_td.xer),p_buf, XER_coding, 0, 0);
+    XER_encode(*(p_td.xer),p_buf, XER_coding, 0, 0, 0);
     p_buf.put_c('\n');
     break;}
   case TTCN_EncDec::CT_JSON: {
@@ -174,7 +174,7 @@ void Base_Type::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -231,38 +231,33 @@ void Base_Type::XER_encode_chk_coding(unsigned& p_coding,
 static const cbyte empty_tag_end[4] = "/>\n";
 
 int Base_Type::begin_xml(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
-  unsigned int& flavor, int indent, bool empty,
-  collector_fn collector, const char *type_atr) const
+  unsigned int& flavor, int indent, boolean empty,
+  collector_fn collector, const char *type_atr, unsigned int flavor2) const
 {
-  const int indenting = !is_canonical(flavor);
   const int exer = is_exer(flavor);
-  int omit_tag = (indent != 0) // can never omit the tag at the toplevel
+  int omit_tag =
+    // can never omit the tag at the toplevel, except when the type is union
+    (indent != 0 || (flavor2 & THIS_UNION))
     && ( ((flavor & XER_RECOF) // can remove the tag even if not EXER
       && !(exer && (flavor & BXER_EMPTY_ELEM))) // except 26.6, 26.7
       || (exer /*&& */
         && ( (p_td.xer_bits & (UNTAGGED|ANY_ATTRIBUTES|ANY_ELEMENT))
           || (flavor & (EMBED_VALUES|XER_LIST|ANY_ATTRIBUTES|USE_NIL|USE_TYPE_ATTR)))));
 
-  // If a default namespace is in effect (uri but no prefix) and the type
-  // is unqualified, the default namespace must be canceled; otherwise
-  // an XML tag without a ns prefix looks like it belongs to the def.namespace
-  const boolean empty_ns_hack = exer && !omit_tag && (indent > 0)
-    && (p_td.xer_bits & FORM_UNQUALIFIED)
-    && (flavor & DEF_NS_PRESENT);
-
   if (exer && (p_td.xer_bits & XER_ATTRIBUTE)) {
     begin_attribute(p_td, p_buf);
   }
   else if (!omit_tag) { // full tag
+    const int indenting = !is_canonical(flavor);
     if (indenting) do_indent(p_buf, indent);
     p_buf.put_c('<');
     if (exer) write_ns_prefix(p_td, p_buf);
 
-    const namespace_t *ns_info = NULL;
-    bool namespaces_needed = false;
+    boolean namespaces_needed = FALSE;
     if (exer) {
+      const namespace_t *ns_info = NULL;
       if (p_td.my_module != NULL && p_td.ns_index != -1) {
-        ns_info = p_td.my_module->get_ns(p_td.ns_index);
+        ns_info = p_td.my_module->get_ns((size_t)p_td.ns_index);
       }
 
       namespaces_needed = exer && //!(p_td.xer_bits & FORM_UNQUALIFIED) &&
@@ -274,9 +269,9 @@ int Base_Type::begin_xml(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
 
     size_t num_collected = 0;
     char **collected_ns = NULL;
-    bool def_ns = false;
+    boolean def_ns = FALSE;
     if (namespaces_needed) {
-      collected_ns = (this->*collector)(p_td, num_collected, def_ns);
+      collected_ns = (this->*collector)(p_td, num_collected, def_ns, flavor2);
     }
 
     p_buf.put_s((size_t)p_td.namelens[exer] - 2, (cbyte*)p_td.names[exer]);
@@ -289,6 +284,13 @@ int Base_Type::begin_xml(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
       Free(collected_ns);
     }
 
+    // If a default namespace is in effect (uri but no prefix) and the type
+    // is unqualified, the default namespace must be canceled; otherwise
+    // an XML tag without a ns prefix looks like it belongs to the def.namespace
+    const boolean empty_ns_hack = exer && !omit_tag && (indent > 0)
+      && (p_td.xer_bits & FORM_UNQUALIFIED)
+      && (flavor & DEF_NS_PRESENT);
+
     if (empty_ns_hack) {
       p_buf.put_s(9, (cbyte*)" xmlns=''");
       flavor &= ~DEF_NS_PRESENT;
@@ -322,6 +324,10 @@ int Base_Type::begin_xml(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
       (p_buf.get_data()[buf_used] == '\n')) {
       p_buf.increase_length((size_t)-1); // back up over the newline
       omit_tag = -1; // to help fix HO85831
+    } else if (exer && (p_td.xer_bits & USE_TYPE_ATTR) && type_atr && (flavor2 & FROM_UNION_USETYPE)) {
+      p_buf.increase_length((size_t)-1); // back up over the endtag
+      p_buf.put_s(mstrlen(const_cast<char*>(type_atr)), (cbyte*)type_atr);
+      p_buf.put_c('>');
     }
   }
 
@@ -331,11 +337,12 @@ int Base_Type::begin_xml(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
 }
 
 void Base_Type::end_xml  (const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
-  unsigned int flavor, int indent, bool empty) const
+  unsigned int flavor, int indent, boolean empty, unsigned int flavor2) const
 {
   int exer = is_exer(flavor);
-  int indenting = !is_canonical(flavor);
-  bool omit_tag = (indent != 0) // can never omit the tag at the toplevel
+  boolean omit_tag =
+    // can never omit the tag at the toplevel, except when the type is union
+    (indent != 0 || (flavor2 & THIS_UNION))
     && ( ((flavor & XER_RECOF) // can remove the tag even if not EXER
       && !(exer && (flavor & BXER_EMPTY_ELEM))) // except 26.6, 26.7
       || (exer /*&& */
@@ -347,6 +354,7 @@ void Base_Type::end_xml  (const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
   }
   else if (!empty && !omit_tag) {
     // now close the tag
+    int indenting = !is_canonical(flavor);
     if (indenting && !(flavor & SIMPLE_TYPE)) do_indent(p_buf, indent);
     p_buf.put_s(2, (cbyte*)"</");
     if (exer) write_ns_prefix(p_td, p_buf);
@@ -511,20 +519,20 @@ ASN_BER_TLV_t *Base_Type::BER_encode_TLV_INTEGER(unsigned,
   unsigned char* bn_as_bin = (unsigned char*) Malloc(num_bytes);
   BN_bn2bin(D, bn_as_bin);
 
-  bool pad = false;
+  boolean pad = FALSE;
   if (BN_is_negative(D)) {
     for(size_t i = 0; i < num_bytes; ++i){
       bn_as_bin[i] = ~bn_as_bin[i];
     }
 
     // add one
-    bool stop = false;
+    boolean stop = FALSE;
     for (int i = num_bytes - 1; i >= 0 && !stop; --i) {
       for (int j = 0; j < 8 && !stop; ++j) {
         unsigned char mask = (0x1 << j);
         if (!(bn_as_bin[i] & mask)) {
           bn_as_bin[i] |= mask;
-          stop = true;
+          stop = TRUE;
         } else {
           bn_as_bin[i] ^= mask;
         }
@@ -747,20 +755,20 @@ boolean Base_Type::BER_decode_TLV_INTEGER(const ASN_BER_TLV_t& p_tlv,
 
   if (Vlen > sizeof(RInt)) { // Bignum
 
-    const bool negative = p_tlv.V.str.Vstr[0] & 0x80;
+    const boolean negative = p_tlv.V.str.Vstr[0] & 0x80;
     BIGNUM *D = BN_new();
 
     if (negative) {
       unsigned char* const Vstr = (unsigned char*) Malloc(Vlen);
       memcpy(Vstr, p_tlv.V.str.Vstr, Vlen);
       // -1 
-      bool stop = false;
+      boolean stop = FALSE;
       for (int i = Vlen - 1; i >= 0 && !stop; --i) {
         for(int j = 0; j < 8 && !stop; ++j) {
           unsigned char mask = (0x1 << j);
           if (Vstr[i] & mask) {
             Vstr[i] ^= mask;
-            stop = true;
+            stop = TRUE;
           } else {
             Vstr[i] |= mask;
           }
@@ -982,7 +990,7 @@ int Base_Type::RAW_decode(const TTCN_Typedescriptor_t& p_td,
 }
 
 int Base_Type::XER_encode(const XERdescriptor_t& p_td,
-                          TTCN_Buffer&, unsigned int, int, embed_values_enc_struct_t*) const
+                          TTCN_Buffer&, unsigned int, unsigned int, int, embed_values_enc_struct_t*) const
 {
   TTCN_error("XER encoding requested for type '%-.*s' which has no"
              " XER encoding method.", p_td.namelens[0]-2, p_td.names[0]);
@@ -1011,21 +1019,21 @@ int Base_Type::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer&, b
 }
 
 boolean Base_Type::can_start(const char *name, const char *uri,
-  XERdescriptor_t const& xd, unsigned int flavor)
+  XERdescriptor_t const& xd, unsigned int flavor, unsigned int /*flavor2*/)
 {
   boolean e_xer = is_exer(flavor);
   // Check the name. If EXER, check the namespace too.
   return check_name(name, xd, e_xer) && (!e_xer || check_namespace(uri, xd));
 }
 
-char ** Base_Type::collect_ns(const XERdescriptor_t& p_td, size_t& num, bool& def_ns) const
+char ** Base_Type::collect_ns(const XERdescriptor_t& p_td, size_t& num, bool& def_ns, unsigned int /* flavor */) const
 {
-  def_ns = false;
+  def_ns = FALSE;
   char *tmp = NULL;
   if (p_td.my_module != 0 && p_td.ns_index != -1
     && !(p_td.xer_bits & FORM_UNQUALIFIED)) {
-    const namespace_t *my_ns = p_td.my_module->get_ns(p_td.ns_index);
-    if (!*my_ns->px) def_ns = true;
+    const namespace_t *my_ns = p_td.my_module->get_ns((size_t)p_td.ns_index);
+    if (!*my_ns->px) def_ns = TRUE;
     tmp = mprintf(" xmlns%s%s='%s'",
       ((*my_ns->px) ? ":" : ""), my_ns->px,
       my_ns->ns
