@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -33,6 +33,7 @@
 #include "Logger.hh"
 #include "Encdec.hh"
 #include "Addfunc.hh"
+#include "Optional.hh"
 
 #include "../common/dbgnew.hh"
 
@@ -255,6 +256,16 @@ BITSTRING BITSTRING::operator+(const BITSTRING_ELEMENT& other_value) const
   return ret_val;
 }
 
+#ifdef TITAN_RUNTIME_2
+BITSTRING BITSTRING::operator+(const OPTIONAL<BITSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const BITSTRING&)other_value;
+  }
+  TTCN_error("Unbound or omitted right operand of bitstring concatenation.");
+}
+#endif
+
 BITSTRING BITSTRING::operator~() const
 {
   must_bound("Unbound bitstring operand of operator not4b.");
@@ -663,14 +674,14 @@ void BITSTRING::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
     RAW_enc_tr_pos rp;
     rp.level=0;
     rp.pos=NULL;
-    RAW_enc_tree root(true,NULL,&rp,1,p_td.raw);
+    RAW_enc_tree root(TRUE,NULL,&rp,1,p_td.raw);
     RAW_encode(p_td, root);
     root.put_to_buf(p_buf);
     break;}
   case TTCN_EncDec::CT_XER: {
     TTCN_EncDec_ErrorContext ec("While XER-encoding type '%s': ", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0);
+    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0, 0);
     break;}
   case TTCN_EncDec::CT_JSON: {
     TTCN_EncDec_ErrorContext ec("While JSON-encoding type '%s': ", p_td.name);
@@ -742,7 +753,7 @@ void BITSTRING::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -773,7 +784,7 @@ void BITSTRING::BER_encode_putbits(unsigned char *target,
     target[0]=0x00;
     return;
   }
-  target[0]=(unsigned char)(nof_octets*8-bit_count);
+  target[0] = static_cast<unsigned char>(nof_octets*8-bit_count);
   for(i=0; i<nof_octets-1; i++) {
     c=0;
     for(j=0; j<8; j++) {
@@ -993,15 +1004,15 @@ int BITSTRING::RAW_encode(const TTCN_Typedescriptor_t& p_td, RAW_enc_tree& mylea
   }
 //  myleaf.ext_bit=EXT_BIT_NO;
   if (myleaf.must_free) Free(myleaf.body.leaf.data_ptr);
-  myleaf.must_free = false;
-  myleaf.data_ptr_used = true;
+  myleaf.must_free = FALSE;
+  myleaf.data_ptr_used = TRUE;
   myleaf.body.leaf.data_ptr = val_ptr->bits_ptr;
-  bool orders = false;
-  if (p_td.raw->byteorder       == ORDER_MSB) orders = true;
+  boolean orders = FALSE;
+  if (p_td.raw->byteorder       == ORDER_MSB) orders = TRUE;
   if (p_td.raw->bitorderinfield == ORDER_LSB) orders = !orders;
   myleaf.coding_par.byteorder = orders ? ORDER_MSB : ORDER_LSB;
-  orders = false;
-  if (p_td.raw->bitorderinoctet == ORDER_MSB) orders = true;
+  orders = FALSE;
+  if (p_td.raw->bitorderinoctet == ORDER_MSB) orders = TRUE;
   if (p_td.raw->bitorderinfield == ORDER_LSB) orders = !orders;
   myleaf.coding_par.bitorder = orders ? ORDER_MSB : ORDER_LSB;
 
@@ -1030,12 +1041,12 @@ int BITSTRING::RAW_decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
   clean_up();
   init_struct(decode_length);
   RAW_coding_par cp;
-  bool orders = false;
-  if (p_td.raw->bitorderinoctet == ORDER_MSB) orders = true;
+  boolean orders = FALSE;
+  if (p_td.raw->bitorderinoctet == ORDER_MSB) orders = TRUE;
   if (p_td.raw->bitorderinfield == ORDER_LSB) orders = !orders;
   cp.bitorder = orders ? ORDER_MSB : ORDER_LSB;
-  orders = false;
-  if (p_td.raw->byteorder == ORDER_MSB) orders = true;
+  orders = FALSE;
+  if (p_td.raw->byteorder == ORDER_MSB) orders = TRUE;
   if (p_td.raw->bitorderinfield == ORDER_LSB) orders = !orders;
   cp.byteorder = orders ? ORDER_MSB : ORDER_LSB;
   cp.fieldorder = p_td.raw->fieldorder;
@@ -1066,7 +1077,7 @@ int BITSTRING::RAW_decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
 }
 
 int BITSTRING::XER_encode(const XERdescriptor_t& p_td,
-  TTCN_Buffer& p_buf, unsigned int flavor, int indent, embed_values_enc_struct_t*) const
+  TTCN_Buffer& p_buf, unsigned int flavor, unsigned int /*flavor2*/, int indent, embed_values_enc_struct_t*) const
 {
   if(!is_bound()) {
     TTCN_EncDec_ErrorContext::error
@@ -1092,7 +1103,7 @@ int BITSTRING::XER_encode(const XERdescriptor_t& p_td,
 int BITSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
                           unsigned int flavor, unsigned int /*flavor2*/, embed_values_dec_struct_t*)
 {
-  int exer  = is_exer(flavor);
+  boolean exer  = is_exer(flavor);
   int success = reader.Ok(), depth = -1, type;
   boolean own_tag = !is_exerlist(flavor) && !(exer && (p_td.xer_bits & UNTAGGED));
 
@@ -1188,12 +1199,12 @@ int BITSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_
   json_token_t token = JSON_TOKEN_NONE;
   char* value = 0;
   size_t value_len = 0;
-  boolean error = false;
-  int dec_len = 0;
+  boolean error = FALSE;
+  size_t dec_len = 0;
   boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
   if (use_default) {
     // No JSON data in the buffer -> use default value
-    value = (char*)p_td.json->default_value;
+    value = const_cast<char*>(p_td.json->default_value);
     value_len = strlen(value);
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
@@ -1203,23 +1214,44 @@ int BITSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_
     return JSON_ERROR_FATAL;
   }
   else if (JSON_TOKEN_STRING == token || use_default) {
-    if (use_default || (value_len > 2 && value[0] == '\"' && value[value_len - 1] == '\"')) {
+    if (use_default || (value_len >= 2 && value[0] == '\"' && value[value_len - 1] == '\"')) {
       if (!use_default) {
         // The default value doesn't have quotes around it
         value_len -= 2;
         ++value;
       }
-      init_struct(value_len);
+      // White spaces are ignored, so the resulting bitstring might be shorter
+      // than the extracted JSON string
+      int bits = value_len;
       for (size_t i = 0; i < value_len; ++i) {
-        if ('0' <= value[i] && '1' >= value[i]) {
-          set_bit(i, value[i] - '0');
-        } else {
-          error = true;
-          break;
+        if (value[i] == ' ') {
+          --bits;
+        }
+        else if (value[i] != '0' && value[i] != '1') {
+          if (value[i] == '\\' && i + 1 < value_len &&
+              (value[i + 1] == 'n' || value[i + 1] == 'r' || value[i + 1] == 't')) {
+            // Escaped white space character
+            ++i;
+            bits -= 2;
+          }
+          else {
+            error = TRUE;
+            break;
+          }
+        }
+      }
+      if (!error) {
+        init_struct(bits);
+        int bit_index = 0;
+        for (size_t i = 0; i < value_len; ++i) {
+          if (value[i] == '0' || value[i] == '1') {
+            set_bit(bit_index, value[i] - '0');
+            ++bit_index;
+          }
         }
       }
     } else {
-      error = true;
+      error = TRUE;
     }
   } else {
     return JSON_ERROR_INVALID_TOKEN;
@@ -1227,12 +1259,9 @@ int BITSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_
   
   if (error) {
     JSON_ERROR(TTCN_EncDec::ET_INVAL_MSG, JSON_DEC_FORMAT_ERROR, "string", "bitstring");
-    if (p_silent) {
-      clean_up();
-    }
     return JSON_ERROR_FATAL;    
   }
-  return dec_len;
+  return (int)dec_len;
 }
 
 
@@ -1314,6 +1343,17 @@ BITSTRING BITSTRING_ELEMENT::operator+(const BITSTRING_ELEMENT& other_value)
   return BITSTRING(2, &result);
 }
 
+#ifdef TITAN_RUNTIME_2
+BITSTRING BITSTRING_ELEMENT::operator+(
+  const OPTIONAL<BITSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const BITSTRING&)other_value;
+  }
+  TTCN_error("Unbound or omitted right operand of bitstring concatenation.");
+}
+#endif
+
 BITSTRING BITSTRING_ELEMENT::operator~() const
 {
   must_bound("Unbound bitstring element operand of operator not4b.");
@@ -1589,7 +1629,7 @@ BITSTRING_template::BITSTRING_template(const OPTIONAL<BITSTRING>& other_value)
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Creating a bitstring template from an unbound optional field.");
   }
 }
@@ -1652,7 +1692,7 @@ BITSTRING_template& BITSTRING_template::operator=
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Assignment of an unbound optional field to a bitstring "
       "template.");
   }
@@ -1669,6 +1709,253 @@ BITSTRING_template& BITSTRING_template::operator=
   return *this;
 }
 
+#ifdef TITAN_RUNTIME_2
+void BITSTRING_template::concat(Vector<unsigned char>& v) const
+{
+  switch (template_selection) {
+  case ANY_VALUE:
+  case ANY_OR_OMIT:
+    switch (length_restriction_type) {
+    case NO_LENGTH_RESTRICTION:
+      if (template_selection == ANY_VALUE) {
+        // ? => '*'
+        if (v.size() == 0 || v[v.size() - 1] != 3) {
+          // '**' == '*', so just ignore the second '*'
+          v.push_back(3);
+        }
+      }
+      else {
+        TTCN_error("Operand of bitstring template concatenation is an "
+          "AnyValueOrNone (*) matching mechanism with no length restriction");
+      }
+      break;
+    case RANGE_LENGTH_RESTRICTION:
+      if (!length_restriction.range_length.max_length ||
+          length_restriction.range_length.max_length != length_restriction.range_length.min_length) {
+        TTCN_error("Operand of bitstring template concatenation is an %s "
+          "matching mechanism with non-fixed length restriction",
+          template_selection == ANY_VALUE ? "AnyValue (?)" : "AnyValueOrNone (*)");
+      }
+      // else fall through (range length restriction is allowed if the minimum
+      // and maximum value are the same)
+    case SINGLE_LENGTH_RESTRICTION: {
+      // ? length(N) or * length(N) => '??...?' N times
+      int len = length_restriction_type == SINGLE_LENGTH_RESTRICTION ?
+        length_restriction.single_length : length_restriction.range_length.min_length;
+      for (int i = 0; i < len; ++i) {
+        v.push_back(2);
+      }
+      break; }
+    }
+    break;
+  case SPECIFIC_VALUE:
+    concat(v, single_value);
+    break;
+  case STRING_PATTERN:
+    for (unsigned int i = 0; i < pattern_value->n_elements; ++i) {
+      v.push_back(pattern_value->elements_ptr[i]);
+    }
+    break;
+  default:
+    TTCN_error("Operand of bitstring template concatenation is an "
+      "uninitialized or unsupported template.");
+  }
+}
+
+void BITSTRING_template::concat(Vector<unsigned char>& v, const BITSTRING& val)
+{
+  if (!val.is_bound()) {
+    TTCN_error("Operand of bitstring template concatenation is an "
+      "unbound value.");
+  }
+  for (int i = 0; i < val.val_ptr->n_bits; ++i) {
+    v.push_back(val.get_bit(i));
+  }
+}
+
+void BITSTRING_template::concat(Vector<unsigned char>& v, template_sel sel)
+{
+  if (sel == ANY_VALUE) {
+    // ? => '*'
+    if (v.size() == 0 || v[v.size() - 1] != 3) {
+      // '**' == '*', so just ignore the second '*'
+      v.push_back(3);
+    }
+  }
+  else {
+    TTCN_error("Operand of bitstring template concatenation is an "
+      "uninitialized or unsupported template.");
+  }
+}
+
+
+BITSTRING_template BITSTRING_template::operator+(
+  const BITSTRING_template& other_value) const
+{
+  if (template_selection == SPECIFIC_VALUE &&
+      other_value.template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return single_value + other_value.single_value;
+  }
+  if (template_selection == ANY_VALUE &&
+      other_value.template_selection == ANY_VALUE &&
+      length_restriction_type == NO_LENGTH_RESTRICTION &&
+      other_value.length_restriction_type == NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return BITSTRING_template(ANY_VALUE);
+  }
+  // otherwise the result is an bitstring pattern
+  Vector<unsigned char> v;
+  concat(v);
+  other_value.concat(v);
+  return BITSTRING_template(v.size(), v.data_ptr());
+}
+
+BITSTRING_template BITSTRING_template::operator+(
+  const BITSTRING& other_value) const
+{
+  if (template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return single_value + other_value;
+  }
+  // otherwise the result is an bitstring pattern
+  Vector<unsigned char> v;
+  concat(v);
+  concat(v, other_value);
+  return BITSTRING_template(v.size(), v.data_ptr());
+}
+
+BITSTRING_template BITSTRING_template::operator+(
+  const BITSTRING_ELEMENT& other_value) const
+{
+  return *this + BITSTRING(other_value);
+}
+
+BITSTRING_template BITSTRING_template::operator+(
+  const OPTIONAL<BITSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const BITSTRING&)other_value;
+  }
+  TTCN_error("Operand of bitstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+BITSTRING_template BITSTRING_template::operator+(
+  template_sel other_template_sel) const
+{
+  if (template_selection == ANY_VALUE && other_template_sel == ANY_VALUE &&
+      length_restriction_type == NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return BITSTRING_template(ANY_VALUE);
+  }
+  // the result is always an bitstring pattern
+  Vector<unsigned char> v;
+  concat(v);
+  concat(v, other_template_sel);
+  return BITSTRING_template(v.size(), v.data_ptr());
+}
+
+BITSTRING_template operator+(const BITSTRING& left_value,
+  const BITSTRING_template& right_template)
+{
+  if (right_template.template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return left_value + right_template.single_value;
+  }
+  // otherwise the result is an bitstring pattern
+  Vector<unsigned char> v;
+  BITSTRING_template::concat(v, left_value);
+  right_template.concat(v);
+  return BITSTRING_template(v.size(), v.data_ptr());
+}
+
+BITSTRING_template operator+(const BITSTRING_ELEMENT& left_value,
+  const BITSTRING_template& right_template)
+{
+  return BITSTRING(left_value) + right_template;
+}
+
+BITSTRING_template operator+(const OPTIONAL<BITSTRING>& left_value,
+  const BITSTRING_template& right_template)
+{
+  if (left_value.is_present()) {
+    return (const BITSTRING&)left_value + right_template;
+  }
+  TTCN_error("Operand of bitstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+BITSTRING_template operator+(template_sel left_template_sel,
+  const BITSTRING_template& right_template)
+{
+  if (left_template_sel == ANY_VALUE &&
+      right_template.template_selection == ANY_VALUE &&
+      right_template.length_restriction_type ==
+      Restricted_Length_Template::NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return BITSTRING_template(ANY_VALUE);
+  }
+  // the result is always an bitstring pattern
+  Vector<unsigned char> v;
+  BITSTRING_template::concat(v, left_template_sel);
+  right_template.concat(v);
+  return BITSTRING_template(v.size(), v.data_ptr());
+}
+
+BITSTRING_template operator+(const BITSTRING& left_value,
+  template_sel right_template_sel)
+{
+  // the result is always an bitstring pattern
+  Vector<unsigned char> v;
+  BITSTRING_template::concat(v, left_value);
+  BITSTRING_template::concat(v, right_template_sel);
+  return BITSTRING_template(v.size(), v.data_ptr());
+}
+
+BITSTRING_template operator+(const BITSTRING_ELEMENT& left_value,
+  template_sel right_template_sel)
+{
+  return BITSTRING(left_value) + right_template_sel;
+}
+
+BITSTRING_template operator+(const OPTIONAL<BITSTRING>& left_value,
+  template_sel right_template_sel)
+{
+  if (left_value.is_present()) {
+    return (const BITSTRING&)left_value + right_template_sel;
+  }
+  TTCN_error("Operand of bitstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+BITSTRING_template operator+(template_sel left_template_sel,
+  const BITSTRING& right_value)
+{
+  // the result is always an bitstring pattern
+  Vector<unsigned char> v;
+  BITSTRING_template::concat(v, left_template_sel);
+  BITSTRING_template::concat(v, right_value);
+  return BITSTRING_template(v.size(), v.data_ptr());
+}
+
+BITSTRING_template operator+(template_sel left_template_sel,
+  const BITSTRING_ELEMENT& right_value)
+{
+  return left_template_sel + BITSTRING(right_value);
+}
+
+BITSTRING_template operator+(template_sel left_template_sel,
+  const OPTIONAL<BITSTRING>& right_value)
+{
+  if (right_value.is_present()) {
+    return left_template_sel + (const BITSTRING&)right_value;
+  }
+  TTCN_error("Operand of bitstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+#endif // TITAN_RUNTIME_2
+
 BITSTRING_ELEMENT BITSTRING_template::operator[](int index_value)
 {
   if (template_selection != SPECIFIC_VALUE || is_ifpresent)
@@ -2004,9 +2291,10 @@ Module_Param* BITSTRING_template::get_param(Module_Param_Name& param_name) const
     mp = new Module_Param_Bitstring_Template(pattern_value->n_elements, val_cpy);
     break; }
   case DECODE_MATCH:
-    mp->error("Referencing a decoded content matching template is not supported.");
+    TTCN_error("Referencing a decoded content matching template is not supported.");
     break;
   default:
+    TTCN_error("Referencing an uninitialized/unsupported bitstring template.");
     break;
   }
   if (is_ifpresent) {
@@ -2133,8 +2421,8 @@ int BITSTRING::RAW_encode_negtest_raw(RAW_enc_tree& p_myleaf) const
 {
   if (p_myleaf.must_free)
     Free(p_myleaf.body.leaf.data_ptr);
-  p_myleaf.must_free = false;
-  p_myleaf.data_ptr_used = true;
+  p_myleaf.must_free = FALSE;
+  p_myleaf.data_ptr_used = TRUE;
   p_myleaf.body.leaf.data_ptr = val_ptr->bits_ptr;
   return p_myleaf.length = val_ptr->n_bits;
 }
