@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -36,6 +36,7 @@
 #include "TEXT.hh"
 #include "Charstring.hh"
 #include "XmlReader.hh"
+#include "Optional.hh"
 
 static const Token_Match boolean_true_match("^(true).*$",TRUE);
 static const Token_Match boolean_false_match("^(false).*$",TRUE);
@@ -237,7 +238,7 @@ void BOOLEAN::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
   case TTCN_EncDec::CT_XER: {
     TTCN_EncDec_ErrorContext ec("While XER-encoding type '%s': ", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0);
+    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0, 0);
     break;}
   case TTCN_EncDec::CT_JSON: {
     TTCN_EncDec_ErrorContext ec("While JSON-encoding type '%s': ", p_td.name);
@@ -326,7 +327,7 @@ void BOOLEAN::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -612,7 +613,7 @@ int BOOLEAN::RAW_decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
 }
 
 int BOOLEAN::XER_encode(const XERdescriptor_t& p_td,
-  TTCN_Buffer& p_buf, unsigned int flavor, int indent, embed_values_enc_struct_t*) const
+  TTCN_Buffer& p_buf, unsigned int flavor, unsigned int /*flavor2*/, int indent, embed_values_enc_struct_t*) const
 {
   if(!is_bound()) {
     TTCN_EncDec_ErrorContext::error
@@ -620,10 +621,10 @@ int BOOLEAN::XER_encode(const XERdescriptor_t& p_td,
   }
   int encoded_length=(int)p_buf.get_len();
 
-  int exer  = is_exer(flavor);
+  boolean exer  = is_exer(flavor);
 
   flavor |= (SIMPLE_TYPE | BXER_EMPTY_ELEM);
-  if (begin_xml(p_td, p_buf, flavor, indent, false) == -1) --encoded_length;
+  if (begin_xml(p_td, p_buf, flavor, indent, FALSE) == -1) --encoded_length;
 
   if (exer) {
     if (p_td.xer_bits & XER_TEXT) {
@@ -639,7 +640,7 @@ int BOOLEAN::XER_encode(const XERdescriptor_t& p_td,
     else               p_buf.put_s(8, (cbyte*)"<false/>");
   }
 
-  end_xml(p_td, p_buf, flavor, indent, false);
+  end_xml(p_td, p_buf, flavor, indent, FALSE);
 
   return (int)p_buf.get_len() - encoded_length;
 }
@@ -702,16 +703,16 @@ int BOOLEAN::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
     // extract the data
     if (value[1]=='\0' && (*value & 0x3E) == '0')
     {
-      bound_flag = true;
+      bound_flag = TRUE;
       boolean_value = *value == '1';
     }
     else if (!strcmp(value, "true")) {
-      boolean_value = true;
-      bound_flag    = true;
+      boolean_value = TRUE;
+      bound_flag    = TRUE;
     }
     else if (!strcmp(value, "false")) {
-      boolean_value = false;
-      bound_flag    = true;
+      boolean_value = FALSE;
+      bound_flag    = TRUE;
     }
 
   }
@@ -746,7 +747,7 @@ int BOOLEAN::JSON_encode(const TTCN_Typedescriptor_t&, JSON_Tokenizer& p_tok) co
 int BOOLEAN::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_tok, boolean p_silent)
 {
   json_token_t token = JSON_TOKEN_NONE;
-  int dec_len = 0;
+  size_t dec_len = 0;
   if (p_td.json->default_value && 0 == p_tok.get_buffer_length()) {
     // No JSON data in the buffer -> use default value
     if (strcmp(p_td.json->default_value, "true") == 0) {
@@ -763,18 +764,18 @@ int BOOLEAN::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_to
     return JSON_ERROR_FATAL;
   }
   else if (JSON_TOKEN_LITERAL_TRUE == token) {
-    bound_flag = true;
-    boolean_value = true;
+    bound_flag = TRUE;
+    boolean_value = TRUE;
   }
   else if (JSON_TOKEN_LITERAL_FALSE == token) {
-    bound_flag = true;
-    boolean_value = false;
+    bound_flag = TRUE;
+    boolean_value = FALSE;
   } 
   else {
-    bound_flag = false;
+    bound_flag = FALSE;
     return JSON_ERROR_INVALID_TOKEN;
   }
-  return dec_len;
+  return (int)dec_len;
 }
 
 
@@ -874,7 +875,7 @@ BOOLEAN_template::BOOLEAN_template(const OPTIONAL<BOOLEAN>& other_value)
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Creating a boolean template from an unbound optional field.");
   }
 }
@@ -928,7 +929,7 @@ BOOLEAN_template& BOOLEAN_template::operator=
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Assignment of an unbound optional field to a boolean "
       "template.");
   }
@@ -1113,6 +1114,7 @@ Module_Param* BOOLEAN_template::get_param(Module_Param_Name& param_name) const
     }
     break; }
   default:
+    TTCN_error("Referencing an uninitialized/unsupported boolean template.");
     break;
   }
   if (is_ifpresent) {
