@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -93,28 +93,28 @@ void COMPONENT::log() const
   else TTCN_Logger::log_event_unbound();
 }
 
-alt_status COMPONENT::done() const
+alt_status COMPONENT::done(Index_Redirect*) const
 {
   if (component_value == UNBOUND_COMPREF) TTCN_error("Performing done "
     "operation on an unbound component reference.");
   return TTCN_Runtime::component_done(component_value);
 }
 
-alt_status COMPONENT::killed() const
+alt_status COMPONENT::killed(Index_Redirect*) const
 {
   if (component_value == UNBOUND_COMPREF) TTCN_error("Performing killed "
     "operation on an unbound component reference.");
   return TTCN_Runtime::component_killed(component_value);
 }
 
-boolean COMPONENT::running() const
+boolean COMPONENT::running(Index_Redirect*) const
 {
   if (component_value == UNBOUND_COMPREF) TTCN_error("Performing running "
     "operation on an unbound component reference.");
   return TTCN_Runtime::component_running(component_value);
 }
 
-boolean COMPONENT::alive() const
+boolean COMPONENT::alive(Index_Redirect*) const
 {
   if (component_value == UNBOUND_COMPREF) TTCN_error("Performing alive "
     "operation on an unbound component reference.");
@@ -746,6 +746,7 @@ Module_Param* COMPONENT_template::get_param(Module_Param_Name& param_name) const
     }
     break; }
   default:
+    TTCN_error("Referencing an uninitialized/unsupported component reference template.");
     break;
   }
   if (is_ifpresent) {
