@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -27,6 +27,7 @@
 #include "RAW.hh"
 #include "Error.hh"
 #include "Logger.hh"
+#include "string.h"
 
 const TTCN_EncDec::error_behavior_t
 TTCN_EncDec::default_error_behavior[TTCN_EncDec::ET_ALL] = {
@@ -675,7 +676,7 @@ void TTCN_Buffer::put_b(size_t len, const unsigned char *s,
   unsigned char* st=NULL;
   unsigned char* st2=NULL;
   int loc_align=align<0?-align:align;
-  bool must_align=false;
+  boolean must_align=FALSE;
   raw_order_t local_bitorder=coding_par.bitorder;
   raw_order_t local_fieldorder=coding_par.fieldorder;
   if(current_bitorder) {
@@ -724,7 +725,7 @@ local_fieldorder==ORDER_MSB?"M":"L"
     else{
       if(coding_par.byteorder==ORDER_MSB) align=-align;
       if(align<0) put_zero(loc_align,local_fieldorder);
-      else must_align=true;
+      else must_align=TRUE;
     }
   }
   if(len==0) {
