@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -125,7 +125,7 @@ double FLOAT::operator-() const
   return -float_value;
 }
 
-bool FLOAT::is_special(double flt_val)
+boolean FLOAT::is_special(double flt_val)
 {
   return ( (flt_val!=flt_val) || (flt_val==INFINITY) || (flt_val==-INFINITY) );
 }
@@ -141,8 +141,6 @@ void FLOAT::check_numeric(double flt_val, const char *err_msg_begin)
 double FLOAT::operator+(double other_value) const
 {
   must_bound("Unbound left operand of float addition.");
-  check_numeric(float_value, "Left operand of float addition");
-  check_numeric(other_value, "Right operand of float addition");
   return float_value + other_value;
 }
 
@@ -150,16 +148,12 @@ double FLOAT::operator+(const FLOAT& other_value) const
 {
   must_bound("Unbound left operand of float addition.");
   other_value.must_bound("Unbound right operand of float addition.");
-  check_numeric(float_value, "Left operand of float addition");
-  check_numeric(other_value.float_value, "Right operand of float addition");
   return float_value + other_value.float_value;
 }
 
 double FLOAT::operator-(double other_value) const
 {
   must_bound("Unbound left operand of float subtraction.");
-  check_numeric(float_value, "Left operand of float subtraction");
-  check_numeric(other_value, "Right operand of float subtraction");
   return float_value - other_value;
 }
 
@@ -167,16 +161,12 @@ double FLOAT::operator-(const FLOAT& other_value) const
 {
   must_bound("Unbound left operand of float subtraction.");
   other_value.must_bound("Unbound right operand of float subtraction.");
-  check_numeric(float_value, "Left operand of float subtraction");
-  check_numeric(other_value.float_value, "Right operand of float subtraction");
   return float_value - other_value.float_value;
 }
 
 double FLOAT::operator*(double other_value) const
 {
   must_bound("Unbound left operand of float multiplication.");
-  check_numeric(float_value, "Left operand of float multiplication");
-  check_numeric(other_value, "Right operand of float multiplication");
   return float_value * other_value;
 }
 
@@ -184,16 +174,12 @@ double FLOAT::operator*(const FLOAT& other_value) const
 {
   must_bound("Unbound left operand of float multiplication.");
   other_value.must_bound("Unbound right operand of float multiplication.");
-  check_numeric(float_value, "Left operand of float multiplication");
-  check_numeric(other_value.float_value, "Right operand of float multiplication");
   return float_value * other_value.float_value;
 }
 
 double FLOAT::operator/(double other_value) const
 {
   must_bound("Unbound left operand of float division.");
-  check_numeric(float_value, "Left operand of float division");
-  check_numeric(other_value, "Right operand of float division");
   if (other_value == 0.0) TTCN_error("Float division by zero.");
   return float_value / other_value;
 }
@@ -202,8 +188,6 @@ double FLOAT::operator/(const FLOAT& other_value) const
 {
   must_bound("Unbound left operand of float division.");
   other_value.must_bound("Unbound right operand of float division.");
-  check_numeric(float_value, "Left operand of float division");
-  check_numeric(other_value.float_value, "Right operand of float division");
   if (other_value.float_value == 0.0) TTCN_error("Float division by zero.");
   return float_value / other_value.float_value;
 }
@@ -369,7 +353,7 @@ void FLOAT::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
   case TTCN_EncDec::CT_XER: {
     TTCN_EncDec_ErrorContext ec("While XER-encoding type '%s': ", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0);
+    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0, 0);
     break;}
   case TTCN_EncDec::CT_JSON: {
     TTCN_EncDec_ErrorContext ec("While JSON-encoding type '%s': ", p_td.name);
@@ -440,7 +424,7 @@ void FLOAT::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -766,7 +750,6 @@ int FLOAT::RAW_decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
   boolean /*first_call*/)
 {
   int prepaddlength = buff.increase_pos_padd(p_td.raw->prepadding);
-  unsigned char *dv;
   limit -= prepaddlength;
   int decode_length = p_td.raw->fieldlength;
   if ( p_td.raw->fieldlength > limit
@@ -796,7 +779,7 @@ int FLOAT::RAW_decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
   cp.hexorder = ORDER_LSB;
   buff.get_b((size_t) decode_length, data, cp, top_bit_ord);
   if (decode_length == 64) {
-    dv = (unsigned char *) &tmp;
+    unsigned char *dv = (unsigned char *) &tmp;
 #if defined __sparc__ || defined __sparc
     memcpy(dv,data,8);
 #else
@@ -848,7 +831,7 @@ const char* XER_NEG_INF_STR = "-INF";
 const char* XER_NAN_STR = "NaN";
 
 int FLOAT::XER_encode(const XERdescriptor_t& p_td,
-  TTCN_Buffer& p_buf, unsigned int flavor, int indent, embed_values_enc_struct_t*) const
+  TTCN_Buffer& p_buf, unsigned int flavor, unsigned int /*flavor2*/, int indent, embed_values_enc_struct_t*) const
 {
   if(!is_bound()) {
     TTCN_EncDec_ErrorContext::error(
@@ -859,7 +842,7 @@ int FLOAT::XER_encode(const XERdescriptor_t& p_td,
   int encoded_length=(int)p_buf.get_len();
   flavor &= ~XER_RECOF; // float doesn't care
 
-  begin_xml(p_td, p_buf, flavor, indent, false);
+  begin_xml(p_td, p_buf, flavor, indent, FALSE);
 
   if (exer && (p_td.xer_bits & XER_DECIMAL)) {
     char buf[312];
@@ -897,17 +880,17 @@ int FLOAT::XER_encode(const XERdescriptor_t& p_td,
     p_buf.put_string(value);
   }
 
-  end_xml(p_td, p_buf, flavor, indent, false);
+  end_xml(p_td, p_buf, flavor, indent, FALSE);
 
   return (int)p_buf.get_len() - encoded_length;
 }
 
 boolean FLOAT::is_float(const char* p_str)
 {
-  bool first_digit = false; // first digit reached
-  bool decimal_point = false; // decimal point (.) reached
-  bool exponent_mark = false; // exponential mark (e or E) reached
-  bool exponent_sign = false; // sign of the exponential (- or +) reached
+  boolean first_digit = FALSE; // first digit reached
+  boolean decimal_point = FALSE; // decimal point (.) reached
+  boolean exponent_mark = FALSE; // exponential mark (e or E) reached
+  boolean exponent_sign = FALSE; // sign of the exponential (- or +) reached
   
   if ('-' == *p_str || '+' == *p_str) {
     ++p_str;
@@ -917,18 +900,18 @@ boolean FLOAT::is_float(const char* p_str)
     switch(*p_str) {
     case '.':
       if (decimal_point || exponent_mark || !first_digit) {
-        return false;
+        return FALSE;
       }
-      decimal_point = true;
-      first_digit = false;
+      decimal_point = TRUE;
+      first_digit = FALSE;
       break;
     case 'e':
     case 'E':
       if (exponent_mark || !first_digit) {
-        return false;
+        return FALSE;
       }
-      exponent_mark = true;
-      first_digit = false;
+      exponent_mark = TRUE;
+      first_digit = FALSE;
       break;
     case '0':
     case '1':
@@ -940,17 +923,17 @@ boolean FLOAT::is_float(const char* p_str)
     case '7':
     case '8':
     case '9':
-      first_digit = true;
+      first_digit = TRUE;
       break;
     case '-':
     case '+':
       if (exponent_sign || !exponent_mark || first_digit) {
-        return false;
+        return FALSE;
       }
-      exponent_sign = true;
+      exponent_sign = TRUE;
       break;
     default:
-      return false;
+      return FALSE;
     }
     
     ++p_str; 
@@ -961,8 +944,8 @@ boolean FLOAT::is_float(const char* p_str)
 int FLOAT::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
   unsigned int flavor, unsigned int /*flavor2*/, embed_values_dec_struct_t*)
 {
-  bound_flag = false;
-  int exer  = is_exer(flavor);
+  bound_flag = FALSE;
+  boolean exer  = is_exer(flavor);
   int success = reader.Ok(), depth = -1;
   if (success <= 0) return 0;
   boolean own_tag = !(exer && (p_td.xer_bits & UNTAGGED)) && !is_exerlist(flavor);
@@ -975,7 +958,7 @@ tagless:
     if (value) {
       if (is_float(value)) {
         if (exer && (p_td.xer_bits & XER_DECIMAL) && p_td.fractionDigits != -1) {
-          char *p = strchr((char*)value, '.');
+          const char *p = strchr(value, '.');
           if (p != NULL) {
             unsigned int fraction_digits_pos = (int)(p - value) + 1 + p_td.fractionDigits;
             if (fraction_digits_pos < strlen(value)) {
@@ -986,20 +969,20 @@ tagless:
             }
           }
         }
-        bound_flag = true;
+        bound_flag = TRUE;
         sscanf(value, "%lf", &float_value);
       } else if (strcmp(XER_NAN_STR, value) == 0 ) {
-        bound_flag = true;
+        bound_flag = TRUE;
 #ifdef NAN
         float_value = NAN;
 #else
         float_value = INFINITY + (-INFINITY);
 #endif
       } else if (strcmp(XER_POS_INF_STR, value) == 0) {
-        bound_flag = true;
+        bound_flag = TRUE;
         float_value = (double)INFINITY;
       } else if (strcmp(XER_NEG_INF_STR, value) == 0) {
-        bound_flag = true;
+        bound_flag = TRUE;
         float_value = -(double)INFINITY;
       }
     }
@@ -1029,7 +1012,7 @@ tagless:
         if (value) {
           if (is_float(value)) {
             if (exer && (p_td.xer_bits & XER_DECIMAL) && p_td.fractionDigits != -1) {
-              char *p = strchr((char*)value, '.');
+              const char *p = strchr(value, '.');
               if (p != NULL) {
                 unsigned int fraction_digits_pos = (int)(p - value) + 1 + p_td.fractionDigits;
                 if (fraction_digits_pos < strlen(value)) {
@@ -1040,20 +1023,20 @@ tagless:
                 }
               }
             }
-            bound_flag = true;
+            bound_flag = TRUE;
             sscanf(value, "%lf", &float_value);
           } else if (strcmp("NaN", value) == 0 ) {
-            bound_flag = true;
+            bound_flag = TRUE;
 #ifdef NAN
             float_value = NAN;
 #else
             float_value = INFINITY + (-INFINITY);
 #endif
           } else if (strcmp("INF", value) == 0) {
-            bound_flag = true;
+            bound_flag = TRUE;
             float_value = (double)INFINITY;
           } else if (strcmp("-INF", value) == 0) {
-            bound_flag = true;
+            bound_flag = TRUE;
             float_value = -(double)INFINITY;
           }
         }
@@ -1095,7 +1078,7 @@ int FLOAT::JSON_encode(const TTCN_Typedescriptor_t&, JSON_Tokenizer& p_tok) cons
   }
   
   // true if decimal representation possible (use %f format)
-  bool decimal_repr = (value == 0.0)
+  boolean decimal_repr = (value == 0.0)
     || (value > -MAX_DECIMAL_FLOAT && value <= -MIN_DECIMAL_FLOAT)
     || (value >= MIN_DECIMAL_FLOAT && value < MAX_DECIMAL_FLOAT);
   
@@ -1107,15 +1090,15 @@ int FLOAT::JSON_encode(const TTCN_Typedescriptor_t&, JSON_Tokenizer& p_tok) cons
 
 int FLOAT::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_tok, boolean p_silent)
 {
-  bound_flag = false;
+  bound_flag = FALSE;
   json_token_t token = JSON_TOKEN_NONE;
   char* value = 0;
   size_t value_len = 0;
-  int dec_len = 0;
+  size_t dec_len = 0;
   boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
   if (use_default) {
     // No JSON data in the buffer -> use default value
-    value = (char*)p_td.json->default_value;
+    value = const_cast<char*>(p_td.json->default_value);
     value_len = strlen(value);
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
@@ -1126,15 +1109,15 @@ int FLOAT::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_tok,
   }
   else if (JSON_TOKEN_STRING == token || use_default) {
     if (0 == strncmp(value, POS_INF_STR + (use_default ? 1 : 0), value_len)) {
-      bound_flag = true;
+      bound_flag = TRUE;
       float_value = INFINITY;
     }
     else if (0 == strncmp(value, NEG_INF_STR + (use_default ? 1 : 0), value_len)) {
-      bound_flag = true;
+      bound_flag = TRUE;
       float_value = -INFINITY;
     }
     else if (0 == strncmp(value, NAN_STR + (use_default ? 1 : 0), value_len)) {
-      bound_flag = true;
+      bound_flag = TRUE;
 #ifdef NAN
       float_value = NAN;
 #else
@@ -1145,14 +1128,14 @@ int FLOAT::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_tok,
       char* spec_val = mprintf("float (%s, %s or %s)", POS_INF_STR, NEG_INF_STR, NAN_STR);
       JSON_ERROR(TTCN_EncDec::ET_INVAL_MSG, JSON_DEC_FORMAT_ERROR, "string", spec_val);
       Free(spec_val);
-      bound_flag = false;
+      bound_flag = FALSE;
       return JSON_ERROR_FATAL;
     }
   }
   else if (JSON_TOKEN_NUMBER == token) {
     char* value2 = mcopystrn(value, value_len);
     sscanf(value2, "%lf", &float_value);
-    bound_flag = true;
+    bound_flag = TRUE;
     Free(value2);
   } else {
     return JSON_ERROR_INVALID_TOKEN;
@@ -1162,10 +1145,10 @@ int FLOAT::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_tok,
     // check for a valid number
     char* value2 = mcopystrn(value, value_len);
     sscanf(value2, "%lf", &float_value);
-    bound_flag = true;
+    bound_flag = TRUE;
     Free(value2);
   }
-  return dec_len;
+  return (int)dec_len;
 }
 
 
@@ -1174,32 +1157,24 @@ int FLOAT::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_tok,
 double operator+(double double_value, const FLOAT& other_value)
 {
   other_value.must_bound("Unbound right operand of float addition.");
-  FLOAT::check_numeric(double_value, "Left operand of float addition");
-  FLOAT::check_numeric(other_value.float_value, "Right operand of float addition");
   return double_value + other_value.float_value;
 }
 
 double operator-(double double_value, const FLOAT& other_value)
 {
   other_value.must_bound("Unbound right operand of float subtraction.");
-  FLOAT::check_numeric(double_value, "Left operand of float subtraction");
-  FLOAT::check_numeric(other_value.float_value, "Right operand of float subtraction");
   return double_value - other_value.float_value;
 }
 
 double operator*(double double_value, const FLOAT& other_value)
 {
   other_value.must_bound("Unbound right operand of float multiplication.");
-  FLOAT::check_numeric(double_value, "Left operand of float multiplication");
-  FLOAT::check_numeric(other_value.float_value, "Right operand of float multiplication");
   return double_value * other_value.float_value;
 }
 
 double operator/(double double_value, const FLOAT& other_value)
 {
   other_value.must_bound("Unbound right operand of float division.");
-  FLOAT::check_numeric(double_value, "Left operand of float division");
-  FLOAT::check_numeric(other_value.float_value, "Right operand of float division");
   if (other_value.float_value == 0.0) TTCN_error("Float division by zero.");
   return double_value / other_value.float_value;
 }
@@ -1293,7 +1268,7 @@ FLOAT_template::FLOAT_template(const OPTIONAL<FLOAT>& other_value)
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Creating a float template from an unbound optional field.");
   }
 }
@@ -1346,7 +1321,7 @@ FLOAT_template& FLOAT_template::operator=(const OPTIONAL<FLOAT>& other_value)
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Assignment of an unbound optional field to a float template.");
   }
   return *this;
@@ -1379,10 +1354,21 @@ boolean FLOAT_template::match(double other_value, boolean /* legacy */) const
         return template_selection == VALUE_LIST;
     return template_selection == COMPLEMENTED_LIST;
   case VALUE_RANGE:
-    return (!value_range.min_is_present ||
-            value_range.min_value <= other_value) &&
-      (!value_range.max_is_present ||
-       value_range.max_value >= other_value);
+    
+    return (
+      // Min boundary is -infinity and (not min exclusive            or  the value is larger than -infinity)
+      (!value_range.min_is_present && (!value_range.min_is_exclusive || other_value != MINUS_INFINITY)) ||
+      // Min boundary is a number and not min exclusive           and it is less or equal than the value   or
+      (value_range.min_is_present && !value_range.min_is_exclusive && value_range.min_value <= other_value) ||
+      // Min boundary is a number and min exclusive              and it is less than the value
+      (value_range.min_is_present && value_range.min_is_exclusive && value_range.min_value < other_value))
+      &&
+      // Max boundary is infinity  and  (not min exclusive            or the value is smaller than infinity)
+      ((!value_range.max_is_present && (!value_range.max_is_exclusive || other_value != PLUS_INFINITY)) ||
+      // Max boundary is a number and not max exclusive           and it is more or equal than the value   or
+       (value_range.max_is_present && !value_range.max_is_exclusive && value_range.max_value >= other_value) ||
+      // Max boundary is a number and max exclusive               and it is more than the value
+       (value_range.max_is_present && value_range.max_is_exclusive && value_range.max_value > other_value));
   default:
     TTCN_error("Matching with an uninitialized/unsupported float template.");
   }
@@ -1411,6 +1397,8 @@ void FLOAT_template::set_type(template_sel template_type,
     set_selection(VALUE_RANGE);
     value_range.min_is_present = FALSE;
     value_range.max_is_present = FALSE;
+    value_range.min_is_exclusive = FALSE;
+    value_range.max_is_exclusive = FALSE;
     break;
   default:
     TTCN_error("Setting an invalid type for a float template.");
@@ -1435,6 +1423,7 @@ void FLOAT_template::set_min(double min_value)
     TTCN_error("The lower limit of the range is greater than the "
                "upper limit in a float template.");
   value_range.min_is_present = TRUE;
+  value_range.min_is_exclusive = FALSE;
   value_range.min_value = min_value;
 }
 
@@ -1453,6 +1442,7 @@ void FLOAT_template::set_max(double max_value)
     TTCN_error("The upper limit of the range is smaller than the "
                "lower limit in a float template.");
   value_range.max_is_present = TRUE;
+  value_range.max_is_exclusive = FALSE;
   value_range.max_value = max_value;
 }
 
@@ -1463,6 +1453,20 @@ void FLOAT_template::set_max(const FLOAT& max_value)
   set_max(max_value.float_value);
 }
 
+void FLOAT_template::set_min_exclusive(boolean min_exclusive)
+{
+  if (template_selection != VALUE_RANGE)
+    TTCN_error("Float template is not range when setting lower limit exclusiveness.");
+  value_range.min_is_exclusive = min_exclusive;
+}
+
+void FLOAT_template::set_max_exclusive(boolean max_exclusive)
+{
+  if (template_selection != VALUE_RANGE)
+    TTCN_error("Float template is not range when setting upper limit exclusiveness.");
+  value_range.max_is_exclusive = max_exclusive;
+}
+
 double FLOAT_template::valueof() const
 {
   if (template_selection != SPECIFIC_VALUE || is_ifpresent)
@@ -1490,9 +1494,11 @@ void FLOAT_template::log() const
     break;
   case VALUE_RANGE:
     TTCN_Logger::log_char('(');
+    if (value_range.min_is_exclusive) TTCN_Logger::log_char('!');
     if (value_range.min_is_present) log_float(value_range.min_value);
     else TTCN_Logger::log_event_str("-infinity");
     TTCN_Logger::log_event_str(" .. ");
+    if (value_range.max_is_exclusive) TTCN_Logger::log_char('!');
     if (value_range.max_is_present) log_float(value_range.max_value);
     else TTCN_Logger::log_event_str("infinity");
     TTCN_Logger::log_char(')');
@@ -1554,6 +1560,8 @@ void FLOAT_template::set_param(Module_Param& param) {
     set_type(VALUE_RANGE);
     if (mp->has_lower_float()) set_min(mp->get_lower_float());
     if (mp->has_upper_float()) set_max(mp->get_upper_float());
+    set_min_exclusive(mp->get_is_min_exclusive());
+    set_max_exclusive(mp->get_is_max_exclusive());
     break;
   case Module_Param::MP_Expression:
     switch (mp->get_expr_type()) {
@@ -1635,9 +1643,10 @@ Module_Param* FLOAT_template::get_param(Module_Param_Name& param_name) const
   case VALUE_RANGE:
     mp = new Module_Param_FloatRange(
       value_range.min_value, value_range.min_is_present,
-      value_range.max_value, value_range.max_is_present);
+      value_range.max_value, value_range.max_is_present, value_range.min_is_exclusive, value_range.max_is_exclusive);
     break;
   default:
+    TTCN_error("Referencing an uninitialized/unsupported float template.");
     break;
   }
   if (is_ifpresent) {
@@ -1704,6 +1713,8 @@ void FLOAT_template::decode_text(Text_Buf& text_buf)
     value_range.max_is_present = text_buf.pull_int() != 0;
     if (value_range.max_is_present)
       value_range.max_value = text_buf.pull_double();
+    value_range.min_is_exclusive = FALSE;
+    value_range.max_is_exclusive = FALSE;
     break;
   default:
     TTCN_error("Text decoder: An unknown/unsupported selection was "
