@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -32,10 +32,12 @@
 #include "Encdec.hh"
 #include "RAW.hh"
 #include "Addfunc.hh"
+#include "Optional.hh"
 
 #include "../common/dbgnew.hh"
 
 #include <string.h>
+#include <ctype.h>
 
 // hexstring value class
 
@@ -261,6 +263,16 @@ HEXSTRING HEXSTRING::operator+(const HEXSTRING_ELEMENT& other_value) const
   return ret_val;
 }
 
+#ifdef TITAN_RUNTIME_2
+HEXSTRING HEXSTRING::operator+(const OPTIONAL<HEXSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const HEXSTRING&)other_value;
+  }
+  TTCN_error("Unbound or omitted right operand of hexstring concatenation.");
+}
+#endif
+
 HEXSTRING HEXSTRING::operator~() const
 {
   must_bound("Unbound hexstring operand of operator not4b.");
@@ -674,14 +686,14 @@ void HEXSTRING::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
     RAW_enc_tr_pos rp;
     rp.level = 0;
     rp.pos = NULL;
-    RAW_enc_tree root(true, NULL, &rp, 1, p_td.raw);
+    RAW_enc_tree root(TRUE, NULL, &rp, 1, p_td.raw);
     RAW_encode(p_td, root);
     root.put_to_buf(p_buf);
     break;}
   case TTCN_EncDec::CT_XER: {
     TTCN_EncDec_ErrorContext ec("While XER-encoding type '%s': ", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0);
+    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0, 0);
     break;}
   case TTCN_EncDec::CT_JSON: {
     TTCN_EncDec_ErrorContext ec("While JSON-encoding type '%s': ", p_td.name);
@@ -742,7 +754,7 @@ void HEXSTRING::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -774,8 +786,8 @@ int HEXSTRING::RAW_encode(const TTCN_Typedescriptor_t& p_td,
 
   if (myleaf.must_free) Free(myleaf.body.leaf.data_ptr);
 
-  myleaf.must_free = false;
-  myleaf.data_ptr_used = true;
+  myleaf.must_free = FALSE;
+  myleaf.data_ptr_used = TRUE;
   myleaf.body.leaf.data_ptr = val_ptr->nibbles_ptr;
 
   if (p_td.raw->endianness == ORDER_MSB)
@@ -802,12 +814,12 @@ int HEXSTRING::RAW_decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
       ? (int)buff.unread_len_bit() : limit) / 4) * 4;
   }
   RAW_coding_par cp;
-  bool orders = false;
-  if (p_td.raw->bitorderinoctet == ORDER_MSB) orders = true;
+  boolean orders = FALSE;
+  if (p_td.raw->bitorderinoctet == ORDER_MSB) orders = TRUE;
   if (p_td.raw->bitorderinfield == ORDER_MSB) orders = !orders;
   cp.bitorder = orders ? ORDER_MSB : ORDER_LSB;
-  orders = false;
-  if (p_td.raw->byteorder == ORDER_MSB) orders = true;
+  orders = FALSE;
+  if (p_td.raw->byteorder == ORDER_MSB) orders = TRUE;
   if (p_td.raw->bitorderinfield == ORDER_MSB) orders = !orders;
   cp.byteorder = orders ? ORDER_MSB : ORDER_LSB;
   cp.fieldorder = p_td.raw->fieldorder;
@@ -848,7 +860,7 @@ extern char  base64_decoder_table[256];
 extern const char cb64[];
 
 int HEXSTRING::XER_encode(const XERdescriptor_t& p_td,
-  TTCN_Buffer& p_buf, unsigned int flavor, int indent, embed_values_enc_struct_t*) const
+  TTCN_Buffer& p_buf, unsigned int flavor, unsigned int /*flavor2*/, int indent, embed_values_enc_struct_t*) const
 {
   if(!is_bound()) {
     TTCN_EncDec_ErrorContext::error
@@ -894,7 +906,7 @@ int HEXSTRING::XER_encode(const XERdescriptor_t& p_td,
 
 }
 
-unsigned int xlate_hs(cbyte*in, int phase, unsigned char*dest) {
+unsigned int xlate_hs(xmlChar in[4], int phase, unsigned char*dest) {
   static unsigned char nbytes[4] = { 3,1,1,2 };
   unsigned char out[4];
 
@@ -924,11 +936,10 @@ Because of this, the bit shifting is different. The first three bytes
 int HEXSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
   unsigned int flavor, unsigned int /*flavor2*/, embed_values_dec_struct_t*)
 {
-  int exer  = is_exer(flavor);
+  boolean exer  = is_exer(flavor);
   int success = reader.Ok(), depth = -1, type;
   boolean own_tag = !is_exerlist(flavor) && !(exer && (p_td.xer_bits & UNTAGGED));
 
-  const char * value = 0;
   if (exer && (p_td.xer_bits & XER_ATTRIBUTE)) {
     const char * name = verify_name(reader, p_td, exer);
     (void)name;
@@ -968,7 +979,7 @@ int HEXSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
 
   type = reader.NodeType();
   if (success == 1 && (XML_READER_TYPE_TEXT == type || XML_READER_TYPE_ATTRIBUTE == type)) {
-    value = (const char *)reader.Value();
+    const char * value = (const char *)reader.Value();
     size_t len = value ? strlen(value) : 0;
 
     if (exer && (p_td.xer_bits & BASE_64)) {
@@ -1078,12 +1089,12 @@ int HEXSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_
   json_token_t token = JSON_TOKEN_NONE;
   char* value = 0;
   size_t value_len = 0;
-  boolean error = false;
-  int dec_len = 0;
+  boolean error = FALSE;
+  size_t dec_len = 0;
   boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
   if (use_default) {
     // No JSON data in the buffer -> use default value
-    value = (char*)p_td.json->default_value;
+    value = const_cast<char*>(p_td.json->default_value);
     value_len = strlen(value);
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
@@ -1093,23 +1104,45 @@ int HEXSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_
     return JSON_ERROR_FATAL;
   }
   else if (JSON_TOKEN_STRING == token || use_default) {
-    if (use_default || (value_len > 2 && value[0] == '\"' && value[value_len - 1] == '\"')) {
+    if (use_default || (value_len >= 2 && value[0] == '\"' && value[value_len - 1] == '\"')) {
       if (!use_default) {
         // The default value doesn't have quotes around it
         value_len -= 2;
         ++value;
       }
-      init_struct(value_len);
+      // White spaces are ignored, so the resulting hexstring might be shorter
+      // than the extracted JSON string
+      int nibbles = value_len;
       for (size_t i = 0; i < value_len; ++i) {
-        unsigned char nibble = char_to_hexdigit(value[i]);
-        if (nibble <= 0x0F) {
-          set_nibble(i, nibble);
-        } else {
-          error = true;
-        }      
+        if (value[i] == ' ') {
+          --nibbles;
+        }
+        else if (!isxdigit(value[i])) {
+          if (value[i] == '\\' && i + 1 < value_len &&
+              (value[i + 1] == 'n' || value[i + 1] == 'r' || value[i + 1] == 't')) {
+            // Escaped white space character
+            ++i;
+            nibbles -= 2;
+          }
+          else {
+            error = TRUE;
+            break;
+          }
+        }
+      }
+      if (!error) {
+        init_struct(nibbles);
+        int nibble_index = 0;
+        for (size_t i = 0; i < value_len; ++i) {
+          if (!isxdigit(value[i])) {
+            continue;
+          }
+          set_nibble(nibble_index, char_to_hexdigit(value[i]));
+          ++nibble_index;
+        }
       }
     } else {
-      error = true;
+      error = TRUE;
     }
   } else {
     return JSON_ERROR_INVALID_TOKEN;
@@ -1117,12 +1150,9 @@ int HEXSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_
   
   if (error) {
     JSON_ERROR(TTCN_EncDec::ET_INVAL_MSG, JSON_DEC_FORMAT_ERROR, "string", "hexstring");
-    if (p_silent) {
-      clean_up();
-    }
     return JSON_ERROR_FATAL;    
   }
-  return dec_len;
+  return (int)dec_len;
 }
 
 
@@ -1205,6 +1235,17 @@ HEXSTRING HEXSTRING_ELEMENT::operator+(const HEXSTRING_ELEMENT& other_value) con
   return HEXSTRING(2, &result);
 }
 
+#ifdef TITAN_RUNTIME_2
+HEXSTRING HEXSTRING_ELEMENT::operator+(
+  const OPTIONAL<HEXSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const HEXSTRING&)other_value;
+  }
+  TTCN_error("Unbound or omitted right operand of hexstring concatenation.");
+}
+#endif
+
 HEXSTRING HEXSTRING_ELEMENT::operator~() const
 {
   must_bound("Unbound hexstring element operand of operator not4b.");
@@ -1472,7 +1513,7 @@ HEXSTRING_template::HEXSTRING_template(const OPTIONAL<HEXSTRING>& other_value)
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Creating a hexstring template from an unbound optional field.");
   }
 }
@@ -1540,7 +1581,7 @@ HEXSTRING_template& HEXSTRING_template::operator=(
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Assignment of an unbound optional field to a hexstring "
       "template.");
   }
@@ -1557,6 +1598,253 @@ HEXSTRING_template& HEXSTRING_template::operator=(
   return *this;
 }
 
+#ifdef TITAN_RUNTIME_2
+void HEXSTRING_template::concat(Vector<unsigned char>& v) const
+{
+  switch (template_selection) {
+  case ANY_VALUE:
+  case ANY_OR_OMIT:
+    switch (length_restriction_type) {
+    case NO_LENGTH_RESTRICTION:
+      if (template_selection == ANY_VALUE) {
+        // ? => '*'
+        if (v.size() == 0 || v[v.size() - 1] != 17) {
+          // '**' == '*', so just ignore the second '*'
+          v.push_back(17);
+        }
+      }
+      else {
+        TTCN_error("Operand of hexstring template concatenation is an "
+          "AnyValueOrNone (*) matching mechanism with no length restriction");
+      }
+      break;
+    case RANGE_LENGTH_RESTRICTION:
+      if (!length_restriction.range_length.max_length ||
+          length_restriction.range_length.max_length != length_restriction.range_length.min_length) {
+        TTCN_error("Operand of hexstring template concatenation is an %s "
+          "matching mechanism with non-fixed length restriction",
+          template_selection == ANY_VALUE ? "AnyValue (?)" : "AnyValueOrNone (*)");
+      }
+      // else fall through (range length restriction is allowed if the minimum
+      // and maximum value are the same)
+    case SINGLE_LENGTH_RESTRICTION: {
+      // ? length(N) or * length(N) => '??...?' N times
+      int len = length_restriction_type == SINGLE_LENGTH_RESTRICTION ?
+        length_restriction.single_length : length_restriction.range_length.min_length;
+      for (int i = 0; i < len; ++i) {
+        v.push_back(16);
+      }
+      break; }
+    }
+    break;
+  case SPECIFIC_VALUE:
+    concat(v, single_value);
+    break;
+  case STRING_PATTERN:
+    for (unsigned int i = 0; i < pattern_value->n_elements; ++i) {
+      v.push_back(pattern_value->elements_ptr[i]);
+    }
+    break;
+  default:
+    TTCN_error("Operand of hexstring template concatenation is an "
+      "uninitialized or unsupported template.");
+  }
+}
+
+void HEXSTRING_template::concat(Vector<unsigned char>& v, const HEXSTRING& val)
+{
+  if (!val.is_bound()) {
+    TTCN_error("Operand of hexstring template concatenation is an "
+      "unbound value.");
+  }
+  for (int i = 0; i < val.val_ptr->n_nibbles; ++i) {
+    v.push_back(val.get_nibble(i));
+  }
+}
+
+void HEXSTRING_template::concat(Vector<unsigned char>& v, template_sel sel)
+{
+  if (sel == ANY_VALUE) {
+    // ? => '*'
+    if (v.size() == 0 || v[v.size() - 1] != 17) {
+      // '**' == '*', so just ignore the second '*'
+      v.push_back(17);
+    }
+  }
+  else {
+    TTCN_error("Operand of hexstring template concatenation is an "
+      "uninitialized or unsupported template.");
+  }
+}
+
+
+HEXSTRING_template HEXSTRING_template::operator+(
+  const HEXSTRING_template& other_value) const
+{
+  if (template_selection == SPECIFIC_VALUE &&
+      other_value.template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return single_value + other_value.single_value;
+  }
+  if (template_selection == ANY_VALUE &&
+      other_value.template_selection == ANY_VALUE &&
+      length_restriction_type == NO_LENGTH_RESTRICTION &&
+      other_value.length_restriction_type == NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return HEXSTRING_template(ANY_VALUE);
+  }
+  // otherwise the result is an hexstring pattern
+  Vector<unsigned char> v;
+  concat(v);
+  other_value.concat(v);
+  return HEXSTRING_template(v.size(), v.data_ptr());
+}
+
+HEXSTRING_template HEXSTRING_template::operator+(
+  const HEXSTRING& other_value) const
+{
+  if (template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return single_value + other_value;
+  }
+  // otherwise the result is an hexstring pattern
+  Vector<unsigned char> v;
+  concat(v);
+  concat(v, other_value);
+  return HEXSTRING_template(v.size(), v.data_ptr());
+}
+
+HEXSTRING_template HEXSTRING_template::operator+(
+  const HEXSTRING_ELEMENT& other_value) const
+{
+  return *this + HEXSTRING(other_value);
+}
+
+HEXSTRING_template HEXSTRING_template::operator+(
+  const OPTIONAL<HEXSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const HEXSTRING&)other_value;
+  }
+  TTCN_error("Operand of hexstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+HEXSTRING_template HEXSTRING_template::operator+(
+  template_sel other_template_sel) const
+{
+  if (template_selection == ANY_VALUE && other_template_sel == ANY_VALUE &&
+      length_restriction_type == NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return HEXSTRING_template(ANY_VALUE);
+  }
+  // the result is always an hexstring pattern
+  Vector<unsigned char> v;
+  concat(v);
+  concat(v, other_template_sel);
+  return HEXSTRING_template(v.size(), v.data_ptr());
+}
+
+HEXSTRING_template operator+(const HEXSTRING& left_value,
+  const HEXSTRING_template& right_template)
+{
+  if (right_template.template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return left_value + right_template.single_value;
+  }
+  // otherwise the result is an hexstring pattern
+  Vector<unsigned char> v;
+  HEXSTRING_template::concat(v, left_value);
+  right_template.concat(v);
+  return HEXSTRING_template(v.size(), v.data_ptr());
+}
+
+HEXSTRING_template operator+(const HEXSTRING_ELEMENT& left_value,
+  const HEXSTRING_template& right_template)
+{
+  return HEXSTRING(left_value) + right_template;
+}
+
+HEXSTRING_template operator+(const OPTIONAL<HEXSTRING>& left_value,
+  const HEXSTRING_template& right_template)
+{
+  if (left_value.is_present()) {
+    return (const HEXSTRING&)left_value + right_template;
+  }
+  TTCN_error("Operand of hexstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+HEXSTRING_template operator+(template_sel left_template_sel,
+  const HEXSTRING_template& right_template)
+{
+  if (left_template_sel == ANY_VALUE &&
+      right_template.template_selection == ANY_VALUE &&
+      right_template.length_restriction_type ==
+      Restricted_Length_Template::NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return HEXSTRING_template(ANY_VALUE);
+  }
+  // the result is always an hexstring pattern
+  Vector<unsigned char> v;
+  HEXSTRING_template::concat(v, left_template_sel);
+  right_template.concat(v);
+  return HEXSTRING_template(v.size(), v.data_ptr());
+}
+
+HEXSTRING_template operator+(const HEXSTRING& left_value,
+  template_sel right_template_sel)
+{
+  // the result is always an hexstring pattern
+  Vector<unsigned char> v;
+  HEXSTRING_template::concat(v, left_value);
+  HEXSTRING_template::concat(v, right_template_sel);
+  return HEXSTRING_template(v.size(), v.data_ptr());
+}
+
+HEXSTRING_template operator+(const HEXSTRING_ELEMENT& left_value,
+  template_sel right_template_sel)
+{
+  return HEXSTRING(left_value) + right_template_sel;
+}
+
+HEXSTRING_template operator+(const OPTIONAL<HEXSTRING>& left_value,
+  template_sel right_template_sel)
+{
+  if (left_value.is_present()) {
+    return (const HEXSTRING&)left_value + right_template_sel;
+  }
+  TTCN_error("Operand of hexstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+HEXSTRING_template operator+(template_sel left_template_sel,
+  const HEXSTRING& right_value)
+{
+  // the result is always an hexstring pattern
+  Vector<unsigned char> v;
+  HEXSTRING_template::concat(v, left_template_sel);
+  HEXSTRING_template::concat(v, right_value);
+  return HEXSTRING_template(v.size(), v.data_ptr());
+}
+
+HEXSTRING_template operator+(template_sel left_template_sel,
+  const HEXSTRING_ELEMENT& right_value)
+{
+  return left_template_sel + HEXSTRING(right_value);
+}
+
+HEXSTRING_template operator+(template_sel left_template_sel,
+  const OPTIONAL<HEXSTRING>& right_value)
+{
+  if (right_value.is_present()) {
+    return left_template_sel + (const HEXSTRING&)right_value;
+  }
+  TTCN_error("Operand of hexstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+#endif // TITAN_RUNTIME_2
+
 HEXSTRING_ELEMENT HEXSTRING_template::operator[](int index_value)
 {
   if (template_selection != SPECIFIC_VALUE || is_ifpresent)
@@ -1896,9 +2184,10 @@ Module_Param* HEXSTRING_template::get_param(Module_Param_Name& param_name) const
     mp = new Module_Param_Hexstring_Template(pattern_value->n_elements, val_cpy);
     break; }
   case DECODE_MATCH:
-    mp->error("Referencing a decoded content matching template is not supported.");
+    TTCN_error("Referencing a decoded content matching template is not supported.");
     break;
   default:
+    TTCN_error("Referencing an uninitialized/unsupported hexstring template.");
     break;
   }
   if (is_ifpresent) {
