@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -276,11 +276,11 @@ INTEGER INTEGER::operator+(const INTEGER& other_value) const
     : BN_is_negative(val.openssl);
   boolean other_value_neg = other_value.native_flag
     ? (other_value.val.native < 0) : BN_is_negative(other_value.val.openssl);
-  boolean result_neg = this_neg && other_value_neg;
   if (!this_neg && other_value_neg) return operator-(-other_value);
   if (this_neg && !other_value_neg) return other_value.operator-(-(*this));
   if (likely(native_flag)) {
     if (likely(other_value.native_flag)) {
+      boolean result_neg = this_neg && other_value_neg;
       unsigned int result_u = val.native + other_value.val.native;
       int result = val.native + other_value.val.native;
       if ((static_cast<int>(result_u) != result) || (!result_neg &&
@@ -592,7 +592,7 @@ long long int INTEGER::get_long_long_val() const
 {
   must_bound("Using the value of an unbound integer variable.");
   if (likely(native_flag)) return val.native;
-  bool is_negative = BN_is_negative(val.openssl);
+  boolean is_negative = BN_is_negative(val.openssl);
   long long int ret_val = 0;
   if (unlikely(BN_is_zero(val.openssl))) return 0;
   // It feels so bad accessing a BIGNUM directly, but faster than string
@@ -629,7 +629,7 @@ void INTEGER::set_long_long_val(long long int other_value)
   val.openssl = BN_new();
   // Make it 0.
   BN_zero(val.openssl);
-  bool is_negative = other_value < 0;
+  boolean is_negative = other_value < 0;
   unsigned long long int tmp = !is_negative ? other_value : -other_value;
   for (int i = sizeof(long long int) - 1; i >= 0; i--) {
     BN_add_word(val.openssl, (tmp >> 8 * i) & 0xff);
@@ -794,7 +794,7 @@ void INTEGER::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
   case TTCN_EncDec::CT_XER: {
     TTCN_EncDec_ErrorContext ec("While XER-encoding type '%s': ", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0);
+    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0, 0);
     break;}
   case TTCN_EncDec::CT_JSON: {
     TTCN_EncDec_ErrorContext ec("While JSON-encoding type '%s': ", p_td.name);
@@ -883,7 +883,7 @@ void INTEGER::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -1592,7 +1592,7 @@ int INTEGER::RAW_decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
 }
 
 int INTEGER::XER_encode(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
-  unsigned int flavor, int indent, embed_values_enc_struct_t*) const
+  unsigned int flavor, unsigned int /*flavor2*/, int indent, embed_values_enc_struct_t*) const
 {
   if (!is_bound()) {
     TTCN_EncDec_ErrorContext::error(TTCN_EncDec::ET_UNBOUND,
@@ -1602,7 +1602,7 @@ int INTEGER::XER_encode(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
 
   flavor |= SIMPLE_TYPE;
   flavor &= ~XER_RECOF; // integer doesn't care
-  if (begin_xml(p_td, p_buf, flavor, indent, false) == -1) --encoded_length;
+  if (begin_xml(p_td, p_buf, flavor, indent, FALSE) == -1) --encoded_length;
 
   char *tmp_str;
   if (native_flag)
@@ -1616,7 +1616,7 @@ int INTEGER::XER_encode(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
     OPENSSL_free(tmp_str);
   p_buf.put_string(value);
 
-  end_xml(p_td, p_buf, flavor, indent, false);
+  end_xml(p_td, p_buf, flavor, indent, FALSE);
 
   return (int) p_buf.get_len() - encoded_length;
 }
@@ -1625,7 +1625,6 @@ int INTEGER::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
   unsigned int flavor, unsigned int /*flavor2*/, embed_values_dec_struct_t*)
 {
   const boolean exer = is_exer(flavor);
-  int depth = -1, success = reader.Ok(), type;
   const char * value = 0;
 
   boolean own_tag = !(exer && (p_td.xer_bits & UNTAGGED)) && !is_exerlist(flavor);
@@ -1644,6 +1643,7 @@ tagless:
     // Let the caller do reader.AdvanceAttribute();
   }
   else {
+    int depth = -1, success = reader.Ok(), type;
     for (; success == 1; success = reader.Read()) {
       type = reader.NodeType();
       if (XML_READER_TYPE_ELEMENT == type) {
@@ -1738,11 +1738,11 @@ int INTEGER::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_to
   json_token_t token = JSON_TOKEN_NONE;
   char* value = 0;
   size_t value_len = 0;
-  int dec_len = 0;
+  size_t dec_len = 0;
   boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
   if (use_default) {
     // No JSON data in the buffer -> use default value
-    value = (char*)p_td.json->default_value;
+    value = const_cast<char*>(p_td.json->default_value);
     value_len = strlen(value);
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
@@ -1754,18 +1754,18 @@ int INTEGER::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_to
   else if (JSON_TOKEN_NUMBER == token || use_default) {
     char* number = mcopystrn(value, value_len);
     if (from_string(number) && (int)value_len == get_nof_digits() + ('-' == value[0] ? 1 : 0)) {
-      bound_flag = true;
+      bound_flag = TRUE;
     } else {
       JSON_ERROR(TTCN_EncDec::ET_INVAL_MSG, JSON_DEC_FORMAT_ERROR, "number", "integer");
-      bound_flag = false;
+      bound_flag = FALSE;
       dec_len = JSON_ERROR_FATAL;
     }
     Free(number);
   } else {
-    bound_flag = false;
+    bound_flag = FALSE;
     return JSON_ERROR_INVALID_TOKEN;
   }
-  return dec_len;
+  return (int)dec_len;
 }
 
 
@@ -1930,6 +1930,7 @@ void INTEGER_template::copy_template(const INTEGER_template& other_value)
     break;
   case VALUE_RANGE:
     value_range.min_is_present = other_value.value_range.min_is_present;
+    value_range.min_is_exclusive = other_value.value_range.min_is_exclusive;
     if (value_range.min_is_present) {
       value_range.min_value.native_flag = other_value.value_range.min_value.native_flag;
       if (likely(value_range.min_value.native_flag))
@@ -1940,6 +1941,7 @@ void INTEGER_template::copy_template(const INTEGER_template& other_value)
           BN_dup(other_value.value_range.min_value.val.openssl);
     }
     value_range.max_is_present = other_value.value_range.max_is_present;
+    value_range.max_is_exclusive = other_value.value_range.max_is_exclusive;
     if (value_range.max_is_present) {
       value_range.max_value.native_flag = other_value.value_range.max_value.native_flag;
       if (likely(value_range.max_value.native_flag))
@@ -1999,7 +2001,7 @@ INTEGER_template::INTEGER_template(const OPTIONAL<INTEGER>& other_value)
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Creating an integer template from an unbound optional field.");
   }
 }
@@ -2062,7 +2064,7 @@ INTEGER_template& INTEGER_template::operator=
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Assignment of an unbound optional field to an integer "
       "template.");
   }
@@ -2101,15 +2103,27 @@ boolean INTEGER_template::match(int other_value, boolean /* legacy */) const
     boolean upper_boundary = !value_range.max_is_present;
     // Lower boundary is set.
     if (!lower_boundary) {
+      if (!value_range.min_is_exclusive) {
       lower_boundary = (likely(value_range.min_value.native_flag) ?
         int_val_t(value_range.min_value.val.native) :
           int_val_t(BN_dup(value_range.min_value.val.openssl))) <= other_value;
+      } else {
+        lower_boundary = (likely(value_range.min_value.native_flag) ?
+        int_val_t(value_range.min_value.val.native) :
+          int_val_t(BN_dup(value_range.min_value.val.openssl))) < other_value;
+      }
     }
     // Upper boundary is set.
     if (!upper_boundary) {
+      if (!value_range.max_is_exclusive) {
       upper_boundary = (likely(value_range.max_value.native_flag) ?
         int_val_t(value_range.max_value.val.native) :
           int_val_t(BN_dup(value_range.max_value.val.openssl))) >= other_value;
+      } else {
+        upper_boundary = (likely(value_range.max_value.native_flag) ?
+        int_val_t(value_range.max_value.val.native) :
+          int_val_t(BN_dup(value_range.max_value.val.openssl))) > other_value;
+      }
     }
     return lower_boundary && upper_boundary; }
   default:
@@ -2144,16 +2158,28 @@ boolean INTEGER_template::match(const INTEGER& other_value,
       boolean upper_boundary = !value_range.max_is_present;
       // Lower boundary is set.
       if (!lower_boundary) {
-        lower_boundary = (likely(value_range.min_value.native_flag) ?
-          int_val_t(value_range.min_value.val.native) :
-            int_val_t(BN_dup(value_range.min_value.val.openssl))) <= other_value.get_val();
+        if (!value_range.min_is_exclusive) {
+          lower_boundary = (likely(value_range.min_value.native_flag) ?
+            int_val_t(value_range.min_value.val.native) :
+              int_val_t(BN_dup(value_range.min_value.val.openssl))) <= other_value.get_val();
+        } else {
+          lower_boundary = (likely(value_range.min_value.native_flag) ?
+            int_val_t(value_range.min_value.val.native) :
+              int_val_t(BN_dup(value_range.min_value.val.openssl))) < other_value.get_val();
+        }
       }
       // Upper boundary is set.
       if (!upper_boundary) {
+        if (value_range.max_is_exclusive) {
         upper_boundary = (likely(value_range.max_value.native_flag) ?
           int_val_t(value_range.max_value.val.native) :
+            int_val_t(BN_dup(value_range.max_value.val.openssl))) > other_value.get_val();
+        } else {
+          upper_boundary = (likely(value_range.max_value.native_flag) ?
+            int_val_t(value_range.max_value.val.native) :
             int_val_t(BN_dup(value_range.max_value.val.openssl))) >= other_value.get_val();
       }
+      }
       return lower_boundary && upper_boundary; }
     default:
       TTCN_error("Matching with an uninitialized/unsupported integer "
@@ -2186,6 +2212,8 @@ void INTEGER_template::set_type(template_sel template_type,
     set_selection(VALUE_RANGE);
     value_range.min_is_present = FALSE;
     value_range.max_is_present = FALSE;
+    value_range.min_is_exclusive = FALSE;
+    value_range.max_is_exclusive = FALSE;
     break;
   default:
     TTCN_error("Setting an invalid type for an integer template.");
@@ -2215,6 +2243,7 @@ void INTEGER_template::set_min(int min_value)
         "limit in an integer template.");
   }
   value_range.min_is_present = TRUE;
+  value_range.min_is_exclusive = FALSE;
   value_range.min_value.native_flag = TRUE;
   value_range.min_value.val.native = min_value;
 }
@@ -2236,6 +2265,7 @@ void INTEGER_template::set_min(const INTEGER& min_value)
         "limit in an integer template.");
   }
   value_range.min_is_present = TRUE;
+  value_range.min_is_exclusive = FALSE;
   value_range.min_value.native_flag = min_value_int.native_flag;
   if (likely(value_range.min_value.native_flag))
     value_range.min_value.val.native = min_value_int.val.native;
@@ -2255,10 +2285,25 @@ void INTEGER_template::set_max(int max_value)
         "limit in an integer template.");
   }
   value_range.max_is_present = TRUE;
+  value_range.max_is_exclusive = FALSE;
   value_range.max_value.native_flag = TRUE;
   value_range.max_value.val.native = max_value;
 }
 
+void INTEGER_template::set_min_exclusive(boolean min_exclusive)
+{
+  if (template_selection != VALUE_RANGE)
+    TTCN_error("Integer template is not range when setting lower limit exclusiveness.");
+  value_range.min_is_exclusive = min_exclusive;
+}
+
+void INTEGER_template::set_max_exclusive(boolean max_exclusive)
+{
+  if (template_selection != VALUE_RANGE)
+    TTCN_error("Integer template is not range when setting upper limit exclusiveness.");
+  value_range.max_is_exclusive = max_exclusive;
+}
+
 void INTEGER_template::set_max(const INTEGER& max_value)
 {
   max_value.must_bound("Using an unbound value when setting the upper bound "
@@ -2275,6 +2320,7 @@ void INTEGER_template::set_max(const INTEGER& max_value)
         "limit in an integer template.");
   }
   value_range.max_is_present = TRUE;
+  value_range.max_is_exclusive = FALSE;
   value_range.max_value.native_flag = max_value_int.native_flag;
   if (likely(value_range.max_value.native_flag))
     value_range.max_value.val.native = max_value_int.val.native;
@@ -2304,6 +2350,7 @@ void INTEGER_template::log() const
     break;
   case VALUE_RANGE:
     TTCN_Logger::log_char('(');
+    if (value_range.min_is_exclusive) TTCN_Logger::log_char('!');
     if (value_range.min_is_present) {
       int_val_t min_value_int = likely(value_range.min_value.native_flag) ?
         int_val_t(value_range.min_value.val.native) :
@@ -2315,6 +2362,7 @@ void INTEGER_template::log() const
       TTCN_Logger::log_event_str("-infinity");
     }
     TTCN_Logger::log_event_str(" .. ");
+    if (value_range.max_is_exclusive) TTCN_Logger::log_char('!');
     if (value_range.max_is_present) {
       int_val_t max_value_int = likely(value_range.max_value.native_flag) ?
         int_val_t(value_range.max_value.val.native) :
@@ -2389,11 +2437,13 @@ void INTEGER_template::set_param(Module_Param& param) {
       tmp.set_val(*mp->get_lower_int());
       set_min(tmp);
     }
+    set_min_exclusive(mp->get_is_min_exclusive());
     if (mp->get_upper_int()!=NULL) {
       INTEGER tmp;
       tmp.set_val(*mp->get_upper_int());
       set_max(tmp);
     }
+    set_max_exclusive(mp->get_is_max_exclusive());
   } break;
   case Module_Param::MP_Expression:
     switch (mp->get_expr_type()) {
@@ -2496,9 +2546,10 @@ Module_Param* INTEGER_template::get_param(Module_Param_Name& param_name) const
         upper_bound = new int_val_t(BN_dup(value_range.max_value.val.openssl));
       }
     }
-    mp = new Module_Param_IntRange(lower_bound, upper_bound);
+    mp = new Module_Param_IntRange(lower_bound, upper_bound, value_range.min_is_exclusive, value_range.max_is_exclusive);
     break; }
   default:
+    TTCN_error("Referencing an uninitialized/unsupported integer template.");
     break;
   }
   if (is_ifpresent) {
@@ -2583,6 +2634,8 @@ void INTEGER_template::decode_text(Text_Buf& text_buf)
         value_range.max_value.val.native = max_value_int.val.native;
       else value_range.max_value.val.openssl = BN_dup(max_value_int.val.openssl);
     }
+    value_range.min_is_exclusive = FALSE;
+    value_range.max_is_exclusive = FALSE;
     break;
   default:
     TTCN_error("Text decoder: An unknown/unsupported selection was received "
