@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -27,6 +27,7 @@
 #include <limits.h>
 #include "../common/static_check.h"
 #include "Integer.hh"
+#include "Optional.hh"
 
 static const size_t MIN_COMPONENTS = 2;
 
@@ -297,7 +298,7 @@ void OBJID::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
   case TTCN_EncDec::CT_XER: {
     TTCN_EncDec_ErrorContext ec("While XER-encoding type '%s': ", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0);
+    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0, 0);
     break;}
   case TTCN_EncDec::CT_JSON: {
     TTCN_EncDec_ErrorContext ec("While JSON-encoding type '%s': ", p_td.name);
@@ -354,7 +355,7 @@ void OBJID::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -501,7 +502,7 @@ boolean OBJID::BER_decode_TLV(const TTCN_Typedescriptor_t& p_td,
 
 
 int OBJID::XER_encode(const XERdescriptor_t& p_td,
-		  TTCN_Buffer& p_buf, unsigned int flavor, int indent, embed_values_enc_struct_t*) const
+		  TTCN_Buffer& p_buf, unsigned int flavor, unsigned int /*flavor2*/, int indent, embed_values_enc_struct_t*) const
 {
   if(!is_bound()) {
     TTCN_EncDec_ErrorContext::error(TTCN_EncDec::ET_UNBOUND,
@@ -511,7 +512,7 @@ int OBJID::XER_encode(const XERdescriptor_t& p_td,
 
   flavor |= SIMPLE_TYPE;
   flavor &= ~XER_RECOF; // object identifier doesn't care
-  begin_xml(p_td, p_buf, flavor, indent, false);
+  begin_xml(p_td, p_buf, flavor, indent, FALSE);
 
   static char str_buf[64];
   for (int i = 0; i < val_ptr->n_components; ++i ) {
@@ -528,7 +529,7 @@ int OBJID::XER_encode(const XERdescriptor_t& p_td,
     else p_buf.put_s(str_len, (const unsigned char*)str_buf);
   }
 
-  end_xml(p_td, p_buf, flavor, indent, false);
+  end_xml(p_td, p_buf, flavor, indent, FALSE);
 
   return (int)p_buf.get_len() - encoded_length;
 }
@@ -561,7 +562,7 @@ void OBJID::from_string(char* p_str)
 int OBJID::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
 		  unsigned int flavor, unsigned int /*flavor2*/, embed_values_dec_struct_t*)
 {
-  int exer  = is_exer(flavor);
+  boolean exer  = is_exer(flavor);
   int success = reader.Ok(), depth = -1;
   for (; success == 1; success = reader.Read()) {
     int type = reader.NodeType();
@@ -621,12 +622,12 @@ int OBJID::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_tok,
   json_token_t token = JSON_TOKEN_NONE;
   char* value = 0;
   size_t value_len = 0;
-  boolean error = false;
-  int dec_len = 0;
+  boolean error = FALSE;
+  size_t dec_len = 0;
   boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
   if (use_default) {
     // No JSON data in the buffer -> use default value
-    value = (char*)p_td.json->default_value;
+    value = const_cast<char*>(p_td.json->default_value);
     value_len = strlen(value);
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
@@ -659,7 +660,7 @@ int OBJID::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_tok,
     }
     return JSON_ERROR_FATAL;    
   }
-  return dec_len;
+  return (int)dec_len;
 }
 
 void OBJID_template::clean_up()
@@ -720,7 +721,7 @@ OBJID_template::OBJID_template(const OPTIONAL<OBJID>& other_value)
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Creating an objid template from an unbound optional field.");
   }
 }
@@ -765,7 +766,7 @@ OBJID_template& OBJID_template::operator=(const OPTIONAL<OBJID>& other_value)
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Assignment of an unbound optional field to an objid template.");
   }
   return *this;
@@ -981,6 +982,7 @@ Module_Param* OBJID_template::get_param(Module_Param_Name& param_name) const
     }
     break; }
   default:
+    TTCN_error("Referencing an uninitialized/unsupported objid template.");
     break;
   }
   if (is_ifpresent) {
