@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -34,9 +34,11 @@
 #include "BER.hh"
 #include "TEXT.hh"
 #include "Addfunc.hh"
+#include "Optional.hh"
 
 #include "../common/dbgnew.hh"
 #include <string.h>
+#include <ctype.h>
 
 // octetstring value class
 
@@ -183,6 +185,16 @@ OCTETSTRING OCTETSTRING::operator+(const OCTETSTRING_ELEMENT& other_value) const
   return ret_val;
 }
 
+#ifdef TITAN_RUNTIME_2
+OCTETSTRING OCTETSTRING::operator+(const OPTIONAL<OCTETSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const OCTETSTRING&)other_value;
+  }
+  TTCN_error("Unbound or omitted right operand of octetstring concatenation.");
+}
+#endif
+
 OCTETSTRING& OCTETSTRING::operator+=(const OCTETSTRING& other_value)
 {
   must_bound("Appending an octetstring value to an unbound octetstring value.");
@@ -616,7 +628,7 @@ void OCTETSTRING::encode(const TTCN_Typedescriptor_t& p_td,
     RAW_enc_tr_pos rp;
     rp.level=0;
     rp.pos=NULL;
-    RAW_enc_tree root(true,NULL,&rp,1,p_td.raw);
+    RAW_enc_tree root(TRUE,NULL,&rp,1,p_td.raw);
     RAW_encode(p_td, root);
     root.put_to_buf(p_buf);
     break;}
@@ -630,7 +642,7 @@ void OCTETSTRING::encode(const TTCN_Typedescriptor_t& p_td,
   case TTCN_EncDec::CT_XER: {
     TTCN_EncDec_ErrorContext ec("While XER-encoding type '%s': ", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0);
+    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0, 0);
     break;}
   case TTCN_EncDec::CT_JSON: {
     TTCN_EncDec_ErrorContext ec("While TEXT-encoding type '%s': ", p_td.name);
@@ -721,7 +733,7 @@ void OCTETSTRING::decode(const TTCN_Typedescriptor_t& p_td,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -768,8 +780,8 @@ int OCTETSTRING::RAW_encode_negtest_raw(RAW_enc_tree& p_myleaf) const
 {
   if (p_myleaf.must_free)
     Free(p_myleaf.body.leaf.data_ptr);
-  p_myleaf.must_free = false;
-  p_myleaf.data_ptr_used = true;
+  p_myleaf.must_free = FALSE;
+  p_myleaf.data_ptr_used = TRUE;
   p_myleaf.body.leaf.data_ptr = val_ptr->octets_ptr;
   return p_myleaf.length = val_ptr->n_octets * 8;
 }
@@ -915,7 +927,7 @@ extern unsigned int xlate(cbyte* in, int phase, unsigned char* dest);
 extern const char cb64[];
 
 int OCTETSTRING::XER_encode(const XERdescriptor_t& p_td,
-    TTCN_Buffer& p_buf, unsigned int flavor, int indent, embed_values_enc_struct_t*) const
+    TTCN_Buffer& p_buf, unsigned int flavor, unsigned int /*flavor2*/, int indent, embed_values_enc_struct_t*) const
 {
   if(!is_bound()) {
     TTCN_EncDec_ErrorContext::error
@@ -962,11 +974,10 @@ int OCTETSTRING::XER_encode(const XERdescriptor_t& p_td,
 int OCTETSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
     unsigned int flavor, unsigned int /*flavor2*/, embed_values_dec_struct_t*)
 {
-  int exer  = is_exer(flavor);
+  boolean exer  = is_exer(flavor);
   int success = reader.Ok(), depth = -1, type;
   boolean own_tag = !is_exerlist(flavor) && !(exer && (p_td.xer_bits & UNTAGGED));
 
-  const char * value = 0;
   if (exer && (p_td.xer_bits & XER_ATTRIBUTE)) {
     const char * name = verify_name(reader, p_td, exer);
     (void)name;
@@ -1006,7 +1017,7 @@ int OCTETSTRING::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& reader,
 
   type = reader.NodeType();
   if (success == 1 && (XML_READER_TYPE_TEXT == type || XML_READER_TYPE_ATTRIBUTE == type)) {
-    value = (const char *)reader.Value();
+    const char * value = (const char *)reader.Value();
     size_t len = value ? strlen(value) : 0;
 
     if (exer && (p_td.xer_bits & BASE_64)) {
@@ -1185,18 +1196,18 @@ int OCTETSTRING::RAW_encode(const TTCN_Typedescriptor_t& p_td,
     align_length = 0;
   }
   if (myleaf.must_free) Free(myleaf.body.leaf.data_ptr);
-  myleaf.must_free = false;
-  myleaf.data_ptr_used = true;
+  myleaf.must_free = FALSE;
+  myleaf.data_ptr_used = TRUE;
   if (p_td.raw->extension_bit != EXT_BIT_NO
     && myleaf.coding_par.bitorder == ORDER_MSB) {
     if (blength > RAW_INT_ENC_LENGTH) {
       myleaf.body.leaf.data_ptr = bc = (unsigned char*)Malloc(blength*sizeof(*bc));
-      myleaf.must_free = true;
-      myleaf.data_ptr_used = true;
+      myleaf.must_free = TRUE;
+      myleaf.data_ptr_used = TRUE;
     }
     else {
       bc = myleaf.body.leaf.data_array;
-      myleaf.data_ptr_used = false;
+      myleaf.data_ptr_used = FALSE;
     }
     for (int a = 0; a < blength; a++) bc[a] = val_ptr->octets_ptr[a] << 1;
   }
@@ -1222,12 +1233,12 @@ int OCTETSTRING::RAW_decode(const TTCN_Typedescriptor_t& p_td,
       ? (int)buff.unread_len_bit() : limit) / 8) * 8;
   }
   RAW_coding_par cp;
-  bool orders = false;
-  if (p_td.raw->bitorderinoctet == ORDER_MSB) orders = true;
+  boolean orders = FALSE;
+  if (p_td.raw->bitorderinoctet == ORDER_MSB) orders = TRUE;
   if (p_td.raw->bitorderinfield == ORDER_MSB) orders = !orders;
   cp.bitorder = orders ? ORDER_MSB : ORDER_LSB;
-  orders = false;
-  if (p_td.raw->byteorder       == ORDER_MSB) orders = true;
+  orders = FALSE;
+  if (p_td.raw->byteorder       == ORDER_MSB) orders = TRUE;
   if (p_td.raw->bitorderinfield == ORDER_MSB) orders = !orders;
   cp.byteorder = orders ? ORDER_MSB : ORDER_LSB;
   cp.fieldorder = p_td.raw->fieldorder;
@@ -1301,12 +1312,12 @@ int OCTETSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer&
   json_token_t token = JSON_TOKEN_NONE;
   char* value = 0;
   size_t value_len = 0;
-  boolean error = false;
-  int dec_len = 0;
+  boolean error = FALSE;
+  size_t dec_len = 0;
   boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
   if (use_default) {
     // No JSON data in the buffer -> use default value
-    value = (char*)p_td.json->default_value;
+    value = const_cast<char*>(p_td.json->default_value);
     value_len = strlen(value);
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
@@ -1316,26 +1327,55 @@ int OCTETSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer&
     return JSON_ERROR_FATAL;
   }
   else if (JSON_TOKEN_STRING == token || use_default) {
-    if (0 == value_len % 2 && (use_default ||
-        (value_len > 2 && value[0] == '\"' && value[value_len - 1] == '\"'))) {
+    if (use_default || (value_len >= 2 && value[0] == '\"' && value[value_len - 1] == '\"')) {
       if (!use_default) {
         // The default value doesn't have quotes around it
         value_len -= 2;
         ++value;
       }
-      size_t octets = value_len / 2;
-      init_struct(octets);
-      for (size_t i = 0; i < octets; ++i) {
-        unsigned char upper_nibble = char_to_hexdigit(value[2 * i]);
-        unsigned char lower_nibble = char_to_hexdigit(value[2 * i + 1]);
-        if (upper_nibble <= 0x0F && lower_nibble <= 0x0F) {
-          val_ptr->octets_ptr[i] = (upper_nibble << 4) | lower_nibble;
-        } else {
-          error = true;
-        }      
+      // White spaces are ignored, so the resulting octetstring might be shorter
+      // than the extracted JSON string
+      int nibbles = value_len;
+      for (size_t i = 0; i < value_len; ++i) {
+        if (value[i] == ' ') {
+          --nibbles;
+        }
+        else if (!isxdigit(value[i]) || i + 1 == value_len ||
+                 !isxdigit(value[i + 1])) {
+          if (value[i] == '\\' && i + 1 < value_len &&
+              (value[i + 1] == 'n' || value[i + 1] == 'r' || value[i + 1] == 't')) {
+            // Escaped white space character
+            ++i;
+            nibbles -= 2;
+          }
+          else {
+            error = TRUE;
+            break;
+          }
+        }
+        else {
+          // It's a valid octet (jump through its second nibble)
+          ++i;
+        }
+      }
+      if (!error) {
+        init_struct(nibbles / 2);
+        if (value_len != 0) {
+          int octet_index = 0;
+          for (size_t i = 0; i < value_len - 1; ++i) {
+            if (!isxdigit(value[i]) || !isxdigit(value[i + 1])) {
+              continue;
+            }
+            unsigned char upper_nibble = char_to_hexdigit(value[i]);
+            unsigned char lower_nibble = char_to_hexdigit(value[i + 1]);
+            val_ptr->octets_ptr[octet_index] = (upper_nibble << 4) | lower_nibble;
+            ++octet_index;
+            ++i;
+          }
+        }
       }
     } else {
-      error = true;
+      error = TRUE;
     }
   } else {
     return JSON_ERROR_INVALID_TOKEN;
@@ -1343,12 +1383,9 @@ int OCTETSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer&
   
   if (error) {
     JSON_ERROR(TTCN_EncDec::ET_INVAL_MSG, JSON_DEC_FORMAT_ERROR, "string", "octetstring");
-    if (p_silent) {
-      clean_up();
-    }
     return JSON_ERROR_FATAL;    
   }
-  return dec_len;
+  return (int)dec_len;
 }
 
 // octetstring element class
@@ -1427,6 +1464,17 @@ OCTETSTRING OCTETSTRING_ELEMENT::operator+
   return OCTETSTRING(2, result);
 }
 
+#ifdef TITAN_RUNTIME_2
+OCTETSTRING OCTETSTRING_ELEMENT::operator+(
+  const OPTIONAL<OCTETSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const OCTETSTRING&)other_value;
+  }
+  TTCN_error("Unbound or omitted right operand of octetstring concatenation.");
+}
+#endif
+
 OCTETSTRING OCTETSTRING_ELEMENT::operator~() const
 {
   must_bound("Unbound octetstring element operand of operator not4b.");
@@ -1702,7 +1750,7 @@ OCTETSTRING_template::OCTETSTRING_template
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Creating an octetstring template from an unbound optional "
       "field.");
   }
@@ -1775,7 +1823,7 @@ OCTETSTRING_template& OCTETSTRING_template::operator=
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Assignment of an unbound optional field to an octetstring "
       "template.");
   }
@@ -1792,6 +1840,253 @@ OCTETSTRING_template& OCTETSTRING_template::operator=
   return *this;
 }
 
+#ifdef TITAN_RUNTIME_2
+void OCTETSTRING_template::concat(Vector<unsigned short>& v) const
+{
+  switch (template_selection) {
+  case ANY_VALUE:
+  case ANY_OR_OMIT:
+    switch (length_restriction_type) {
+    case NO_LENGTH_RESTRICTION:
+      if (template_selection == ANY_VALUE) {
+        // ? => '*'
+        if (v.size() == 0 || v[v.size() - 1] != 257) {
+          // '**' == '*', so just ignore the second '*'
+          v.push_back(257);
+        }
+      }
+      else {
+        TTCN_error("Operand of octetstring template concatenation is an "
+          "AnyValueOrNone (*) matching mechanism with no length restriction");
+      }
+      break;
+    case RANGE_LENGTH_RESTRICTION:
+      if (!length_restriction.range_length.max_length ||
+          length_restriction.range_length.max_length != length_restriction.range_length.min_length) {
+        TTCN_error("Operand of octetstring template concatenation is an %s "
+          "matching mechanism with non-fixed length restriction",
+          template_selection == ANY_VALUE ? "AnyValue (?)" : "AnyValueOrNone (*)");
+      }
+      // else fall through (range length restriction is allowed if the minimum
+      // and maximum value are the same)
+    case SINGLE_LENGTH_RESTRICTION: {
+      // ? length(N) or * length(N) => '??...?' N times
+      int len = length_restriction_type == SINGLE_LENGTH_RESTRICTION ?
+        length_restriction.single_length : length_restriction.range_length.min_length;
+      for (int i = 0; i < len; ++i) {
+        v.push_back(256);
+      }
+      break; }
+    }
+    break;
+  case SPECIFIC_VALUE:
+    concat(v, single_value);
+    break;
+  case STRING_PATTERN:
+    for (unsigned int i = 0; i < pattern_value->n_elements; ++i) {
+      v.push_back(pattern_value->elements_ptr[i]);
+    }
+    break;
+  default:
+    TTCN_error("Operand of octetstring template concatenation is an "
+      "uninitialized or unsupported template.");
+  }
+}
+
+void OCTETSTRING_template::concat(Vector<unsigned short>& v, const OCTETSTRING& val)
+{
+  if (!val.is_bound()) {
+    TTCN_error("Operand of octetstring template concatenation is an "
+      "unbound value.");
+  }
+  for (int i = 0; i < val.val_ptr->n_octets; ++i) {
+    v.push_back(val.val_ptr->octets_ptr[i]);
+  }
+}
+
+void OCTETSTRING_template::concat(Vector<unsigned short>& v, template_sel sel)
+{
+  if (sel == ANY_VALUE) {
+    // ? => '*'
+    if (v.size() == 0 || v[v.size() - 1] != 257) {
+      // '**' == '*', so just ignore the second '*'
+      v.push_back(257);
+    }
+  }
+  else {
+    TTCN_error("Operand of octetstring template concatenation is an "
+      "uninitialized or unsupported template.");
+  }
+}
+
+
+OCTETSTRING_template OCTETSTRING_template::operator+(
+  const OCTETSTRING_template& other_value) const
+{
+  if (template_selection == SPECIFIC_VALUE &&
+      other_value.template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return single_value + other_value.single_value;
+  }
+  if (template_selection == ANY_VALUE &&
+      other_value.template_selection == ANY_VALUE &&
+      length_restriction_type == NO_LENGTH_RESTRICTION &&
+      other_value.length_restriction_type == NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return OCTETSTRING_template(ANY_VALUE);
+  }
+  // otherwise the result is an octetstring pattern
+  Vector<unsigned short> v;
+  concat(v);
+  other_value.concat(v);
+  return OCTETSTRING_template(v.size(), v.data_ptr());
+}
+
+OCTETSTRING_template OCTETSTRING_template::operator+(
+  const OCTETSTRING& other_value) const
+{
+  if (template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return single_value + other_value;
+  }
+  // otherwise the result is an octetstring pattern
+  Vector<unsigned short> v;
+  concat(v);
+  concat(v, other_value);
+  return OCTETSTRING_template(v.size(), v.data_ptr());
+}
+
+OCTETSTRING_template OCTETSTRING_template::operator+(
+  const OCTETSTRING_ELEMENT& other_value) const
+{
+  return *this + OCTETSTRING(other_value);
+}
+
+OCTETSTRING_template OCTETSTRING_template::operator+(
+  const OPTIONAL<OCTETSTRING>& other_value) const
+{
+  if (other_value.is_present()) {
+    return *this + (const OCTETSTRING&)other_value;
+  }
+  TTCN_error("Operand of octetstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+OCTETSTRING_template OCTETSTRING_template::operator+(
+  template_sel other_template_sel) const
+{
+  if (template_selection == ANY_VALUE && other_template_sel == ANY_VALUE &&
+      length_restriction_type == NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return OCTETSTRING_template(ANY_VALUE);
+  }
+  // the result is always an octetstring pattern
+  Vector<unsigned short> v;
+  concat(v);
+  concat(v, other_template_sel);
+  return OCTETSTRING_template(v.size(), v.data_ptr());
+}
+
+OCTETSTRING_template operator+(const OCTETSTRING& left_value,
+  const OCTETSTRING_template& right_template)
+{
+  if (right_template.template_selection == SPECIFIC_VALUE) {
+    // result is a specific value template
+    return left_value + right_template.single_value;
+  }
+  // otherwise the result is an octetstring pattern
+  Vector<unsigned short> v;
+  OCTETSTRING_template::concat(v, left_value);
+  right_template.concat(v);
+  return OCTETSTRING_template(v.size(), v.data_ptr());
+}
+
+OCTETSTRING_template operator+(const OCTETSTRING_ELEMENT& left_value,
+  const OCTETSTRING_template& right_template)
+{
+  return OCTETSTRING(left_value) + right_template;
+}
+
+OCTETSTRING_template operator+(const OPTIONAL<OCTETSTRING>& left_value,
+  const OCTETSTRING_template& right_template)
+{
+  if (left_value.is_present()) {
+    return (const OCTETSTRING&)left_value + right_template;
+  }
+  TTCN_error("Operand of octetstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+OCTETSTRING_template operator+(template_sel left_template_sel,
+  const OCTETSTRING_template& right_template)
+{
+  if (left_template_sel == ANY_VALUE &&
+      right_template.template_selection == ANY_VALUE &&
+      right_template.length_restriction_type ==
+      Restricted_Length_Template::NO_LENGTH_RESTRICTION) {
+    // special case: ? & ? => ?
+    return OCTETSTRING_template(ANY_VALUE);
+  }
+  // the result is always an octetstring pattern
+  Vector<unsigned short> v;
+  OCTETSTRING_template::concat(v, left_template_sel);
+  right_template.concat(v);
+  return OCTETSTRING_template(v.size(), v.data_ptr());
+}
+
+OCTETSTRING_template operator+(const OCTETSTRING& left_value,
+  template_sel right_template_sel)
+{
+  // the result is always an octetstring pattern
+  Vector<unsigned short> v;
+  OCTETSTRING_template::concat(v, left_value);
+  OCTETSTRING_template::concat(v, right_template_sel);
+  return OCTETSTRING_template(v.size(), v.data_ptr());
+}
+
+OCTETSTRING_template operator+(const OCTETSTRING_ELEMENT& left_value,
+  template_sel right_template_sel)
+{
+  return OCTETSTRING(left_value) + right_template_sel;
+}
+
+OCTETSTRING_template operator+(const OPTIONAL<OCTETSTRING>& left_value,
+  template_sel right_template_sel)
+{
+  if (left_value.is_present()) {
+    return (const OCTETSTRING&)left_value + right_template_sel;
+  }
+  TTCN_error("Operand of octetstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+
+OCTETSTRING_template operator+(template_sel left_template_sel,
+  const OCTETSTRING& right_value)
+{
+  // the result is always an octetstring pattern
+  Vector<unsigned short> v;
+  OCTETSTRING_template::concat(v, left_template_sel);
+  OCTETSTRING_template::concat(v, right_value);
+  return OCTETSTRING_template(v.size(), v.data_ptr());
+}
+
+OCTETSTRING_template operator+(template_sel left_template_sel,
+  const OCTETSTRING_ELEMENT& right_value)
+{
+  return left_template_sel + OCTETSTRING(right_value);
+}
+
+OCTETSTRING_template operator+(template_sel left_template_sel,
+  const OPTIONAL<OCTETSTRING>& right_value)
+{
+  if (right_value.is_present()) {
+    return left_template_sel + (const OCTETSTRING&)right_value;
+  }
+  TTCN_error("Operand of octetstring template concatenation is an "
+    "unbound or omitted record/set field.");
+}
+#endif // TITAN_RUNTIME_2
+
 OCTETSTRING_ELEMENT OCTETSTRING_template::operator[](int index_value)
 {
   if (template_selection != SPECIFIC_VALUE || is_ifpresent)
@@ -2128,9 +2423,10 @@ Module_Param* OCTETSTRING_template::get_param(Module_Param_Name& param_name) con
     mp = new Module_Param_Octetstring_Template(pattern_value->n_elements, val_cpy);
     break; }
   case DECODE_MATCH:
-    mp->error("Referencing a decoded content matching template is not supported.");
+    TTCN_error("Referencing a decoded content matching template is not supported.");
     break;
   default:
+    TTCN_error("Referencing an uninitialized/unsupported octetstring template.");
     break;
   }
   if (is_ifpresent) {
