@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -8,6 +8,7 @@
  * Contributors:
  *   Baji, Laszlo
  *   Balasko, Jeno
+ *   Baranyi, Botond
  *   Delic, Adam
  *   Feher, Csaba
  *   Forstner, Matyas
@@ -53,11 +54,14 @@
 #include "../common/dbgnew.hh"
 
 PORT *PORT::list_head = NULL, *PORT::list_tail = NULL;
+PORT *PORT::system_list_head = NULL, *PORT::system_list_tail = NULL;
 
-void PORT::add_to_list()
+void PORT::add_to_list(boolean system)
 {
+  PORT **head = system == FALSE ? &list_head : &system_list_head;
+  PORT **tail = system == FALSE ? &list_tail : &system_list_tail;
   // check for duplicate names
-  for (PORT *p = list_head; p != NULL; p = p->list_next) {
+  for (PORT *p = *head; p != NULL; p = p->list_next) {
     // do nothing if this is already a member of the list
     if (p == this) return;
     else if (!strcmp(p->port_name, port_name))
@@ -65,28 +69,37 @@ void PORT::add_to_list()
         "name %s.", port_name);
   }
   // append this to the list
-  if (list_head == NULL) list_head = this;
-  else if (list_tail != NULL) list_tail->list_next = this;
-  list_prev = list_tail;
+  if (*head == NULL) *head = this;
+  else if (*tail != NULL) (*tail)->list_next = this;
+  list_prev = *tail;
   list_next = NULL;
-  list_tail = this;
+  *tail = this;
 }
 
-void PORT::remove_from_list()
+void PORT::remove_from_list(boolean system)
 {
+  PORT **head = system == FALSE ? &list_head : &system_list_head;
+  PORT **tail = system == FALSE ? &list_tail : &system_list_tail;
+  
   if (list_prev != NULL) list_prev->list_next = list_next;
-  else if (list_head == this) list_head = list_next;
+  else if (*head == this) *head = list_next;
   if (list_next != NULL) list_next->list_prev = list_prev;
-  else if (list_tail == this) list_tail = list_prev;
+  else if (*tail == this) *tail = list_prev;
   list_prev = NULL;
   list_next = NULL;
 }
 
-PORT *PORT::lookup_by_name(const char *par_port_name)
+PORT *PORT::lookup_by_name(const char *par_port_name, boolean system)
 {
-  for (PORT *port = list_head; port != NULL; port = port->list_next)
-    if (!strcmp(par_port_name, port->port_name)) return port;
-  return NULL;
+  if (system == FALSE) {
+    for (PORT *port = list_head; port != NULL; port = port->list_next)
+      if (!strcmp(par_port_name, port->port_name)) return port;
+    return NULL;
+  } else {
+    for (PORT *port = system_list_head; port != NULL; port = port->list_next)
+      if (!strcmp(par_port_name, port->port_name)) return port;
+    return NULL;
+  }
 }
 
 struct PORT::port_parameter {
@@ -114,10 +127,12 @@ void PORT::apply_parameter(port_parameter *par_ptr)
 
 void PORT::set_system_parameters(const char *system_port)
 {
-  for (port_parameter *par = parameter_head; par != NULL; par = par->next_par)
+  for (port_parameter *par = parameter_head; par != NULL; par = par->next_par) {
     if (par->component_id.id_selector == COMPONENT_ID_SYSTEM &&
-      (par->port_name == NULL || !strcmp(par->port_name, system_port)))
+      (par->port_name == NULL || !strcmp(par->port_name, system_port))) {
       set_parameter(par->parameter_name, par->parameter_value);
+    }
+  }
 }
 
 void PORT::add_parameter(const component_id_t& component_id,
@@ -277,7 +292,7 @@ PORT::PORT(const char *par_port_name)
 
 PORT::~PORT()
 {
-  if (is_active) deactivate_port();
+  if (is_active) deactivate_port(FALSE); //TODO false or true
 }
 
 void PORT::set_name(const char * name)
@@ -292,19 +307,25 @@ void PORT::log() const
   TTCN_Logger::log_event("port %s", port_name);
   }
 
-void PORT::activate_port()
+void PORT::activate_port(boolean system)
 {
   if (!is_active) {
-    add_to_list();
+    add_to_list(system);
     is_active = TRUE;
     msg_head_count = 0;
     msg_tail_count = 0;
     proc_head_count = 0;
     proc_tail_count = 0;
+    
+    // Only has effect when the translation port has port variables with
+    // default values. Only call when it is activated.
+    if (n_system_mappings == 0) {
+      init_port_variables();
+    }
   }
 }
 
-void PORT::deactivate_port()
+void PORT::deactivate_port(boolean system)
 {
   if (is_active) {
     /* In order to proceed with the deactivation we must ignore the
@@ -336,11 +357,11 @@ void PORT::deactivate_port()
         TitanLoggerApi::Port__Misc_reason::removing__unterminated__mapping,
         port_name, NULL_COMPREF, system_port);
       try {
-        unmap(system_port);
+        unmap(system_port, system);
       } catch (const TC_Error&) { }
       if (is_parallel) {
         try {
-          TTCN_Communication::send_unmapped(port_name, system_port);
+          TTCN_Communication::send_unmapped(port_name, system_port, system);
         } catch (const TC_Error&) { }
       }
       Free(system_port);
@@ -358,14 +379,15 @@ void PORT::deactivate_port()
     Fd_And_Timeout_User::set_timer(this, 0.0);
     // File descriptor events of port connections are removed
     // in remove_connection
-    remove_from_list();
+    remove_from_list(system);
     is_active = FALSE;
   }
 }
 
 void PORT::deactivate_all()
 {
-  while (list_head != NULL) list_head->deactivate_port();
+  while (list_head != NULL) list_head->deactivate_port(FALSE);
+  while (system_list_head != NULL) system_list_head->deactivate_port(TRUE);
 }
 
 void PORT::clear()
@@ -385,6 +407,8 @@ void PORT::all_clear()
 {
   for (PORT *port = list_head; port != NULL; port = port->list_next)
     port->clear();
+  for (PORT *port = system_list_head; port != NULL; port = port->list_next)
+    port->clear();
 }
 
 void PORT::start()
@@ -413,6 +437,8 @@ void PORT::all_start()
 {
   for (PORT *port = list_head; port != NULL; port = port->list_next)
     port->start();
+  for (PORT *port = system_list_head; port != NULL; port = port->list_next)
+    port->start();
 }
 
 void PORT::stop()
@@ -441,6 +467,8 @@ void PORT::all_stop()
 {
   for (PORT *port = list_head; port != NULL; port = port->list_next)
     port->stop();
+  for (PORT *port = system_list_head; port != NULL; port = port->list_next)
+    port->stop();
 }
 
 void PORT::halt()
@@ -467,14 +495,24 @@ void PORT::all_halt()
 {
   for (PORT *port = list_head; port != NULL; port = port->list_next)
     port->halt();
+  for (PORT *port = system_list_head; port != NULL; port = port->list_next)
+    port->halt();
 }
 
-alt_status PORT::receive(const COMPONENT_template&, COMPONENT *)
+boolean PORT::port_is_started() {
+  return is_started;
+}
+
+PORT* PORT::get_provider_port() {
+  return NULL;
+}
+
+alt_status PORT::receive(const COMPONENT_template&, COMPONENT *, Index_Redirect*)
 {
   TTCN_Logger::log_matching_problem(
     TitanLoggerApi::MatchingProblemType_reason::no__incoming__types,
     TitanLoggerApi::MatchingProblemType_operation::receive__,
-    false, false, port_name);
+    FALSE, FALSE, port_name);
   return ALT_NO;
 }
 
@@ -503,17 +541,17 @@ alt_status PORT::any_receive(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::receive__,
-      true, false);
+      TRUE, FALSE);
     return ALT_NO;
   }
 }
 
-alt_status PORT::check_receive(const COMPONENT_template&, COMPONENT *)
+alt_status PORT::check_receive(const COMPONENT_template&, COMPONENT *, Index_Redirect*)
 {
   TTCN_Logger::log_matching_problem(
     TitanLoggerApi::MatchingProblemType_reason::no__incoming__types,
     TitanLoggerApi::MatchingProblemType_operation::receive__,
-    false, true, port_name);
+    FALSE, TRUE, port_name);
   return ALT_NO;
 }
 
@@ -542,17 +580,17 @@ alt_status PORT::any_check_receive(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::receive__,
-      true, true);
+      TRUE, TRUE);
     return ALT_NO;
   }
 }
 
-alt_status PORT::trigger(const COMPONENT_template&, COMPONENT *)
+alt_status PORT::trigger(const COMPONENT_template&, COMPONENT *, Index_Redirect*)
 {
   TTCN_Logger::log_matching_problem(
     TitanLoggerApi::MatchingProblemType_reason::no__incoming__types,
     TitanLoggerApi::MatchingProblemType_operation::trigger__,
-    false, false, port_name);
+    FALSE, FALSE, port_name);
   return ALT_NO;
 }
 
@@ -583,12 +621,12 @@ alt_status PORT::any_trigger(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::trigger__,
-      true, false);
+      TRUE, FALSE);
     return ALT_NO;
   }
 }
 
-alt_status PORT::getcall(const COMPONENT_template&, COMPONENT *)
+alt_status PORT::getcall(const COMPONENT_template&, COMPONENT *, Index_Redirect*)
 {
 //  ToDo:Unnecessary log matching problem warning removed.
 //  Question: does it unnecessary?
@@ -624,12 +662,12 @@ alt_status PORT::any_getcall(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::getcall__,
-      true, false);
+      TRUE, FALSE);
     return ALT_NO;
   }
 }
 
-alt_status PORT::check_getcall(const COMPONENT_template&, COMPONENT *)
+alt_status PORT::check_getcall(const COMPONENT_template&, COMPONENT *, Index_Redirect*)
 {
 //  ToDo:Unnecessary log matching problem warning removed.
 //  Question: does it unnecessary
@@ -665,12 +703,12 @@ alt_status PORT::any_check_getcall(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::getcall__,
-      true, true);
+      TRUE, TRUE);
     return ALT_NO;
   }
 }
 
-alt_status PORT::getreply(const COMPONENT_template&, COMPONENT *)
+alt_status PORT::getreply(const COMPONENT_template&, COMPONENT *, Index_Redirect*)
 {
 //  ToDo:Unnecessary log matching problem warning removed.
 //  Question: does it unnecessary
@@ -706,12 +744,12 @@ alt_status PORT::any_getreply(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::getreply__,
-      true, false);
+      TRUE, FALSE);
     return ALT_NO;
   }
 }
 
-alt_status PORT::check_getreply(const COMPONENT_template&, COMPONENT *)
+alt_status PORT::check_getreply(const COMPONENT_template&, COMPONENT *, Index_Redirect*)
 {
 //  ToDo:Unnecessary log matching problem warning removed.
 //  Question: does it unnecessary
@@ -747,12 +785,12 @@ alt_status PORT::any_check_getreply(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::getreply__,
-      true, true);
+      TRUE, TRUE);
     return ALT_NO;
   }
 }
 
-alt_status PORT::get_exception(const COMPONENT_template&, COMPONENT *)
+alt_status PORT::get_exception(const COMPONENT_template&, COMPONENT *, Index_Redirect*)
 {
 //  ToDo:Unnecessary log matching problem warning removed.
 //  Question: does it unnecessary
@@ -788,20 +826,20 @@ alt_status PORT::any_catch(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::catch__,
-      true, false);
+      TRUE, FALSE);
     return ALT_NO;
   }
 }
 
 alt_status PORT::check_catch(const COMPONENT_template& ,
-  COMPONENT *)
+  COMPONENT *, Index_Redirect*)
 {
 //  ToDo:Unnecessary log matching problem warning removed.
 //  Question: does it unnecessary
 //  TTCN_Logger::log_matching_problem(
 //    TitanLoggerApi::MatchingProblemType_reason::no__outgoing__blocking__signatures__that__support__exceptions,
 //    TitanLoggerApi::MatchingProblemType_operation::catch__,
-//    false, true, port_name);
+//    false, TRUE, port_name);
   return ALT_NO;
 }
 
@@ -830,13 +868,13 @@ alt_status PORT::any_check_catch(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::catch__,
-      true, true);
+      TRUE, TRUE);
     return ALT_NO;
   }
 }
 
 alt_status PORT::check(const COMPONENT_template& sender_template,
-  COMPONENT *sender_ptr)
+  COMPONENT *sender_ptr, Index_Redirect*)
 {
   alt_status ret_val = ALT_NO;
   // the procedure-based queue must have the higher priority
@@ -924,7 +962,7 @@ alt_status PORT::any_check(const COMPONENT_template& sender_template,
     TTCN_Logger::log_matching_problem(
       TitanLoggerApi::MatchingProblemType_reason::component__has__no__ports,
       TitanLoggerApi::MatchingProblemType_operation::check__,
-      true, false);
+      TRUE, FALSE);
     return ALT_NO;
   }
 }
@@ -1057,7 +1095,7 @@ void PORT::Install_Handler(const fd_set *read_fds, const fd_set *write_fds,
     "inactive port %s.", port_name);
 
   if ((long) FdMap::getFdLimit() > (long) FD_SETSIZE) {
-    static bool once = true;
+    static boolean once = TRUE;
     if (once) {
       TTCN_warning("The maximum number of open file descriptors (%i)"
         " is greater than FD_SETSIZE (%li)."
@@ -1068,7 +1106,7 @@ void PORT::Install_Handler(const fd_set *read_fds, const fd_set *write_fds,
         FdMap::getFdLimit(), (long) FD_SETSIZE, (long) FD_SETSIZE,
         port_name);
     }
-    once = false;
+    once = FALSE;
   }
 
   Fd_And_Timeout_User::set_fds_with_fd_sets(this, read_fds, write_fds,
@@ -1296,7 +1334,11 @@ port_connection *PORT::add_connection(component remote_component,
       else if (ret_val > 0) break;
     } else if (conn_ptr->remote_component > remote_component) break;
   }
-
+  
+  if (n_system_mappings > 0) {
+    TTCN_error("Connect operation cannot be performed on a mapped port (%s).", port_name);
+  }
+  
   port_connection *new_conn = new port_connection;
   new_conn->owner_port = this;
 
@@ -1871,7 +1913,7 @@ void PORT::send_data_local(port_connection *conn_ptr, Text_Buf& outgoing_data)
 boolean PORT::send_data_stream(port_connection *conn_ptr,
   Text_Buf& outgoing_data, boolean ignore_peer_disconnect)
 {
-  bool would_block_warning=false;
+  boolean would_block_warning=FALSE;
   outgoing_data.calculate_length();
   const char *msg_ptr = outgoing_data.get_data();
   size_t msg_len = outgoing_data.get_len(), sent_len = 0;
@@ -1906,7 +1948,7 @@ boolean PORT::send_data_stream(port_connection *conn_ptr,
               "outgoing buffer. Trying to process incoming data to "
               "avoid deadlock.", conn_ptr->remote_port);
             TTCN_warning_end();
-            would_block_warning=true;
+            would_block_warning=TRUE;
           }
           TTCN_Snapshot::block_for_sending(conn_ptr->stream.comm_fd);
         }
@@ -2122,7 +2164,7 @@ void PORT::process_last_message(port_connection *conn_ptr)
   }
 }
 
-void PORT::map(const char *system_port)
+void PORT::map(const char *system_port, boolean translation)
 {
   if (!is_active) TTCN_error("Inactive port %s cannot be mapped.", port_name);
 
@@ -2131,19 +2173,33 @@ void PORT::map(const char *system_port)
     int str_diff = strcmp(system_port, system_mappings[new_posn]);
     if (str_diff < 0) break;
     else if (str_diff == 0) {
-      TTCN_warning("Port %s is already mapped to system:%s."
-        " Map operation was ignored.", port_name, system_port);
+      if (translation == FALSE) {
+        TTCN_warning("Port %s is already mapped to system:%s."
+          " Map operation was ignored.", port_name, system_port);
+      } else {
+        TTCN_warning("System:%s is already mapped to port %s."
+          " Map operation was ignored.", system_port, port_name);
+      }
       return;
     }
   }
-
-  set_system_parameters(system_port);
+  if (translation == FALSE) {
+    set_system_parameters(system_port);
+  } else {
+    set_system_parameters(port_name);
+  }
 
   user_map(system_port);
 
-  TTCN_Logger::log_port_misc(
-    TitanLoggerApi::Port__Misc_reason::port__was__mapped__to__system,
-    port_name, SYSTEM_COMPREF, system_port);
+  if (translation == FALSE) {
+    TTCN_Logger::log_port_misc(
+      TitanLoggerApi::Port__Misc_reason::port__was__mapped__to__system,
+      port_name, SYSTEM_COMPREF, system_port);
+  } else {
+    TTCN_Logger::log_port_misc(
+      TitanLoggerApi::Port__Misc_reason::port__was__mapped__to__system,
+      system_port, SYSTEM_COMPREF, port_name);
+  }
 
   // the mapping shall be registered in the table only if user_map() was
   // successful
@@ -2159,7 +2215,7 @@ void PORT::map(const char *system_port)
     "addressing.", port_name);
 }
 
-void PORT::unmap(const char *system_port)
+void PORT::unmap(const char *system_port, boolean translation)
 {
   int del_posn;
   for (del_posn = 0; del_posn < n_system_mappings; del_posn++) {
@@ -2171,8 +2227,13 @@ void PORT::unmap(const char *system_port)
     }
   }
   if (del_posn >= n_system_mappings) {
-    TTCN_warning("Port %s is not mapped to system:%s. "
-      "Unmap operation was ignored.", port_name, system_port);
+    if (translation == FALSE) {
+      TTCN_warning("Port %s is not mapped to system:%s. "
+        "Unmap operation was ignored.", port_name, system_port);
+    } else {
+      TTCN_warning("System:%s is not mapped to port %s. "
+        "Unmap operation was ignored.", system_port, port_name);
+    }
     return;
   }
 
@@ -2192,6 +2253,16 @@ void PORT::unmap(const char *system_port)
     Free(unmapped_port);
     throw;
   }
+  
+  // Only valid for provider ports and standard like translation (user) ports.
+  // Currently the requirement is that the port needs to map only when mapped 
+  // to one port. If it would be mapped to more ports then this call would
+  // remove all translation capability.
+  // This does not resets the 'port' variables in the port type definition, but
+  // the internal c++ port variables of the port type.
+  if (n_system_mappings == 0) {
+    reset_port_variables();
+  }
 
   TTCN_Logger::log_port_misc(
     TitanLoggerApi::Port__Misc_reason::port__was__unmapped__from__system,
@@ -2200,6 +2271,18 @@ void PORT::unmap(const char *system_port)
   Free(unmapped_port);
 }
 
+void PORT::reset_port_variables() {
+  
+}
+
+void PORT::init_port_variables() {
+  
+}
+
+void PORT::change_port_state(translation_port_state /*state*/) {
+  
+}
+
 void PORT::process_connect_listen(const char *local_port,
   component remote_component, const char *remote_port,
   transport_type_enum transport_type)
@@ -2408,27 +2491,45 @@ void PORT::terminate_local_connection(const char *src_port,
   }
 }
 
-void PORT::map_port(const char *component_port, const char *system_port)
+void PORT::map_port(const char *component_port, const char *system_port, boolean translation)
 {
-  PORT *port_ptr = lookup_by_name(component_port);
+  const char *port_name = translation == FALSE ? component_port : system_port;
+  PORT *port_ptr = lookup_by_name(port_name, translation);
   if (port_ptr == NULL) TTCN_error("Map operation refers to "
-    "non-existent port %s.", component_port);
-  port_ptr->map(system_port);
-  if (!TTCN_Runtime::is_single())
-    TTCN_Communication::send_mapped(component_port, system_port);
+    "non-existent port %s.", port_name);
+  if (port_ptr->connection_list_head != NULL) {
+    TTCN_error("Map operation is not allowed on a connected port (%s).", port_name);
+  }
+  if (translation == FALSE) {
+    port_ptr->map(system_port, translation);
+  } else {
+    port_ptr->map(component_port, translation);
+  }
+  if (!TTCN_Runtime::is_single()) {
+    if (translation == FALSE) {
+      TTCN_Communication::send_mapped(component_port, system_port, translation);
+    } else {
+      TTCN_Communication::send_mapped(system_port, component_port, translation);
+    }
+  }
 }
 
-void PORT::unmap_port(const char *component_port, const char *system_port)
+void PORT::unmap_port(const char *component_port, const char *system_port, boolean translation)
 {
-  PORT *port_ptr = lookup_by_name(component_port);
+  const char *port_name = translation == FALSE ? component_port : system_port;
+  PORT *port_ptr = lookup_by_name(port_name, translation);
   if (port_ptr == NULL) TTCN_error("Unmap operation refers to "
-    "non-existent port %s.", component_port);
-  port_ptr->unmap(system_port);
+    "non-existent port %s.", port_name);
+  if (translation == FALSE) {
+    port_ptr->unmap(system_port, translation);
+  } else {
+    port_ptr->unmap(component_port, translation);
+  }
   if (!TTCN_Runtime::is_single())
-    TTCN_Communication::send_unmapped(component_port, system_port);
+    TTCN_Communication::send_unmapped(component_port, system_port, translation);
 }
 
-bool PORT::check_port_state(const CHARSTRING& type) const
+boolean PORT::check_port_state(const CHARSTRING& type) const
 {
   if (type == "Started") {
     return is_started;
@@ -2446,23 +2547,32 @@ bool PORT::check_port_state(const CHARSTRING& type) const
   TTCN_error("%s is not an allowed parameter of checkstate().", (const char*)type);
 }
 
-bool PORT::any_check_port_state(const CHARSTRING& type)
+boolean PORT::any_check_port_state(const CHARSTRING& type)
 {
-  bool result = false;
+  boolean result = FALSE;
   for (PORT *port = list_head; port != NULL; port = port->list_next) {
     result = port->check_port_state(type);
     if (result) {
-      return true;
+      return TRUE;
     }
   }
-  return false;
+  for (PORT *port = system_list_head; port != NULL; port = port->list_next) {
+    result = port->check_port_state(type);
+    if (result) {
+      return TRUE;
+    }
+  }
+  return FALSE;
 }
 
-bool PORT::all_check_port_state(const CHARSTRING& type)
+boolean PORT::all_check_port_state(const CHARSTRING& type)
 {
-  bool result = true;
+  boolean result = TRUE;
   for (PORT *port = list_head; port != NULL && result; port = port->list_next) {
     result = port->check_port_state(type);
   }
+  for (PORT *port = system_list_head; port != NULL && result; port = port->list_next) {
+    result = port->check_port_state(type);
+  }
   return result;
-}
\ No newline at end of file
+}
