@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -86,11 +86,11 @@ const unsigned char BitMaskTable[9]={
 RAW_enc_tree::RAW_enc_tree(boolean is_leaf, RAW_enc_tree *par,
   RAW_enc_tr_pos *par_pos, int my_pos, const TTCN_RAWdescriptor_t *raw_attr)
 {
-  bool orders = false;
+  boolean orders = FALSE;
   isleaf = is_leaf;
-  must_free = false;
-  data_ptr_used = false;
-  rec_of = false;
+  must_free = FALSE;
+  data_ptr_used = FALSE;
+  rec_of = FALSE;
   parent = par;
   curr_pos.pos = (int*) Malloc((par_pos->level+1)*sizeof(int));
   if (par_pos->level) memcpy((void*) curr_pos.pos, (void*) par_pos->pos,
@@ -111,11 +111,11 @@ RAW_enc_tree::RAW_enc_tree(boolean is_leaf, RAW_enc_tree *par,
   ext_bit = raw_attr->extension_bit;
   top_bit_order = raw_attr->top_bit_order;
   calc = CALC_NO;
-  if (raw_attr->byteorder == ORDER_MSB) orders = true;
+  if (raw_attr->byteorder == ORDER_MSB) orders = TRUE;
   if (raw_attr->bitorderinfield == ORDER_MSB) orders = !orders;
   coding_par.byteorder = orders ? ORDER_MSB : ORDER_LSB;
-  orders = false;
-  if (raw_attr->bitorderinoctet == ORDER_MSB) orders = true;
+  orders = FALSE;
+  if (raw_attr->bitorderinoctet == ORDER_MSB) orders = TRUE;
   if (raw_attr->bitorderinfield == ORDER_MSB) orders = !orders;
   coding_par.bitorder = orders ? ORDER_MSB : ORDER_LSB;
   coding_par.hexorder = raw_attr->hexorder;
@@ -361,7 +361,7 @@ int RAW_encode_enum_type(const TTCN_Typedescriptor_t& p_td,
 // Called from code generated by enum.c (defEnumClass)
 int RAW_decode_enum_type(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& buff,
   int limit, raw_order_t top_bit_ord, int& value, int min_bits_enum,
-  bool no_err)
+  boolean no_err)
 {
   int fl = p_td.raw->fieldlength ? p_td.raw->fieldlength : min_bits_enum;
   TTCN_RAWdescriptor_t my_raw;
