@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -26,13 +26,14 @@
 #include "Error.hh"
 #include "Logger.hh"
 #include "Textbuf.hh"
+#include "Optional.hh"
 
 #include "../common/dbgnew.hh"
 
 #define UNBOUND_VERDICT ((verdicttype)(ERROR + 1))
 #define IS_VALID(verdict_value) (verdict_value >= NONE && verdict_value <= ERROR)
 
-const char * const verdict_name[] = { "none", "pass", "inconc", "fail", "error" };
+const char * const verdict_name[5] = { "none", "pass", "inconc", "fail", "error" };
 
 VERDICTTYPE::VERDICTTYPE()
 {
@@ -185,7 +186,7 @@ void VERDICTTYPE::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
   case TTCN_EncDec::CT_XER: {
     TTCN_EncDec_ErrorContext ec("While XER-encoding type '%s': ", p_td.name);
     unsigned XER_coding=va_arg(pvar, unsigned);
-    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0);
+    XER_encode(*p_td.xer, p_buf, XER_coding, 0, 0, 0);
     break;}
   case TTCN_EncDec::CT_JSON: {
     TTCN_EncDec_ErrorContext ec("While JSON-encoding type '%s': ", p_td.name);
@@ -268,7 +269,7 @@ void VERDICTTYPE::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
       TTCN_EncDec_ErrorContext::error_internal
         ("No JSON descriptor available for type '%s'.", p_td.name);
     JSON_Tokenizer tok((const char*)p_buf.get_data(), p_buf.get_len());
-    if(JSON_decode(p_td, tok, false)<0)
+    if(JSON_decode(p_td, tok, FALSE)<0)
       ec.error(TTCN_EncDec::ET_INCOMPL_MSG,
                "Can not decode type '%s', because invalid or incomplete"
                " message was received"
@@ -284,19 +285,19 @@ void VERDICTTYPE::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
 
 
 int VERDICTTYPE::XER_encode(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf,
-  unsigned int p_flavor, int p_indent, embed_values_enc_struct_t*) const
+  unsigned int p_flavor, unsigned int /*p_flavor2*/, int p_indent, embed_values_enc_struct_t*) const
 {
   int encoded_length=(int)p_buf.get_len();
   //const boolean e_xer = is_exer(p_flavor);
   p_flavor |= (SIMPLE_TYPE | BXER_EMPTY_ELEM);
-  if (begin_xml(p_td, p_buf, p_flavor, p_indent, false) == -1) --encoded_length;
+  if (begin_xml(p_td, p_buf, p_flavor, p_indent, FALSE) == -1) --encoded_length;
   //if (!e_xer) p_buf.put_c('<');
   {
     const char * enumval = verdict_name[verdict_value];
     p_buf.put_s(strlen(enumval), (const unsigned char*)enumval);
   }
   //if (!e_xer) p_buf.put_s(2, (const unsigned char*)"/>");
-  end_xml(p_td, p_buf, p_flavor, p_indent, false);
+  end_xml(p_td, p_buf, p_flavor, p_indent, FALSE);
   return (int)p_buf.get_len() - encoded_length;
 }
 
@@ -318,18 +319,19 @@ verdicttype VERDICTTYPE::str_to_verdict(const char *v, boolean silent)
 int VERDICTTYPE::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& p_reader,
   unsigned int p_flavor, unsigned int /*flavor2*/, embed_values_dec_struct_t*)
 {
-  int rd_ok = 1, type;
-  const int e_xer = is_exer(p_flavor);
-  const boolean name_tag = !((!e_xer && is_record_of(p_flavor)) || (e_xer && ((p_td.xer_bits & UNTAGGED) ||(is_record_of(p_flavor) && is_exerlist(p_flavor)))));
+  const boolean e_xer = is_exer(p_flavor);
   if (e_xer && ((p_td.xer_bits & XER_ATTRIBUTE) || is_exerlist(p_flavor))) {
     if ((p_td.xer_bits & XER_ATTRIBUTE)) verify_name(p_reader, p_td, e_xer);
     const char * value = (const char *)p_reader.Value();
     if (value) {
-        verdict_value = str_to_verdict(value, (p_flavor & EXIT_ON_ERROR) ? true : false);
+        verdict_value = str_to_verdict(value, (p_flavor & EXIT_ON_ERROR) ? TRUE : FALSE);
     }
   }
   else {
-    if (name_tag)      for (; rd_ok == 1; rd_ok = p_reader.Read()) {
+    int rd_ok = 1, type;
+    const boolean name_tag = !((!e_xer && is_record_of(p_flavor)) || (e_xer && ((p_td.xer_bits & UNTAGGED) ||(is_record_of(p_flavor) && is_exerlist(p_flavor)))));
+    if (name_tag)
+      for (; rd_ok == 1; rd_ok = p_reader.Read()) {
         type = p_reader.NodeType();
         if (XML_READER_TYPE_ELEMENT == type) {
           // If our parent is optional and there is an unexpected tag then return and
@@ -350,7 +352,7 @@ int VERDICTTYPE::XER_decode(const XERdescriptor_t& p_td, XmlReaderWrap& p_reader
     const char *local_name = /*e_xer ?*/ (const char *)p_reader.Value() /*: (const char *)p_reader.Name()*/;
     if (!local_name) ; else    {
       for (; '\t'==*local_name || '\n'==*local_name; ++local_name) ;
-      verdict_value = str_to_verdict(local_name, (p_flavor & EXIT_ON_ERROR) ? true : false);
+      verdict_value = str_to_verdict(local_name, (p_flavor & EXIT_ON_ERROR) ? TRUE : FALSE);
     }
     if (name_tag)
       for (rd_ok = p_reader.Read(); rd_ok == 1; rd_ok = p_reader.Read()) {
@@ -385,16 +387,16 @@ int VERDICTTYPE::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer&
   json_token_t token = JSON_TOKEN_NONE;
   char* value = 0;
   size_t value_len = 0;
-  int dec_len = 0;
+  size_t dec_len = 0;
   boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
   if (use_default) {
     // No JSON data in the buffer -> use default value
-    value = (char*)p_td.json->default_value;
+    value = const_cast<char*>(p_td.json->default_value);
     value_len = strlen(value);
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
   }
-  boolean error = true;
+  boolean error = TRUE;
   if (JSON_TOKEN_ERROR == token) {
     JSON_ERROR(TTCN_EncDec::ET_INVAL_MSG, JSON_DEC_BAD_TOKEN_ERROR, "");
     dec_len = JSON_ERROR_FATAL;
@@ -409,22 +411,21 @@ int VERDICTTYPE::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer&
       for (int i = NONE; i <= ERROR; ++i) {
         if (0 == strncmp(value, verdict_name[i], value_len)) {
           verdict_value = (verdicttype)i;
-          error = false;
+          error = FALSE;
           break;
         }
       }
     }
   } else {
-    error = false;
     verdict_value = UNBOUND_VERDICT;
-    dec_len = JSON_ERROR_INVALID_TOKEN;
+    return JSON_ERROR_INVALID_TOKEN;
   }
   if (error) {
     JSON_ERROR(TTCN_EncDec::ET_INVAL_MSG, JSON_DEC_FORMAT_ERROR, "string", "verdicttype");
     verdict_value = UNBOUND_VERDICT;
-    dec_len = JSON_ERROR_FATAL;
+    return JSON_ERROR_FATAL;
   }
-  return dec_len;
+  return (int)dec_len;
 }
 
 
@@ -513,7 +514,7 @@ VERDICTTYPE_template::VERDICTTYPE_template
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Creating a verdict template from an unbound optional field.");
   }
 }
@@ -567,7 +568,7 @@ VERDICTTYPE_template& VERDICTTYPE_template::operator=
   case OPTIONAL_OMIT:
     set_selection(OMIT_VALUE);
     break;
-  default:
+  case OPTIONAL_UNBOUND:
     TTCN_error("Assignment of an unbound optional field to a verdict "
       "template.");
   }
@@ -759,6 +760,7 @@ Module_Param* VERDICTTYPE_template::get_param(Module_Param_Name& param_name) con
     }
     break; }
   default:
+    TTCN_error("Referencing an uninitialized/unsupported verdict template.");
     break;
   }
   if (is_ifpresent) {
