@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -49,14 +49,14 @@ int do_indent(TTCN_Buffer& buf, int level)
   return level;
 }
 
-const char* verify_name(XmlReaderWrap& reader, const XERdescriptor_t& p_td, int exer)
+const char* verify_name(XmlReaderWrap& reader, const XERdescriptor_t& p_td, boolean exer)
 {
   const char *name = (const char*)reader.LocalName();
   const char *nsuri= (const char*)reader.NamespaceUri(); // NULL if no ns
 
   const namespace_t *expected_ns = 0;
   if (p_td.my_module != 0 && p_td.ns_index != -1) {
-    expected_ns = p_td.my_module->get_ns(p_td.ns_index);
+    expected_ns = p_td.my_module->get_ns((size_t)p_td.ns_index);
   }
 
   if (0 == name) {
@@ -85,7 +85,7 @@ const char* verify_name(XmlReaderWrap& reader, const XERdescriptor_t& p_td, int
       }
       else {
         if (nsuri == 0) { // XML node has no namespace
-          if (p_td.my_module->get_ns(p_td.ns_index)->px[0] != 0) { // and module has namespace prefix
+          if (p_td.my_module->get_ns((size_t)p_td.ns_index)->px[0] != 0) { // and module has namespace prefix
             TTCN_EncDec_ErrorContext::error(TTCN_EncDec::ET_TAG,
               "Missing namespace '%s'", expected_ns->ns);
           }
@@ -103,7 +103,7 @@ const char* verify_name(XmlReaderWrap& reader, const XERdescriptor_t& p_td, int
   return name;
 }
 
-void verify_end(XmlReaderWrap& reader, const XERdescriptor_t& p_td, const int depth, int exer)
+void verify_end(XmlReaderWrap& reader, const XERdescriptor_t& p_td, const int depth, boolean exer)
 {
   TTCN_EncDec_ErrorContext endcontext("While checking end tag: ");
   verify_name(reader, p_td, exer);
@@ -115,15 +115,15 @@ void verify_end(XmlReaderWrap& reader, const XERdescriptor_t& p_td, const int de
 }
 
 // This should be called for EXER only
-bool check_namespace(const char *ns_uri, const XERdescriptor_t& p_td)
+boolean check_namespace(const char *ns_uri, const XERdescriptor_t& p_td)
 {
   if (p_td.my_module==0 || p_td.ns_index==-1) {// no namespace in XER descriptor
     return ns_uri==0 || *ns_uri=='\0'; // there should be no ns
   }
   else {
-    const namespace_t *expected_ns = p_td.my_module->get_ns(p_td.ns_index);
+    const namespace_t *expected_ns = p_td.my_module->get_ns((size_t)p_td.ns_index);
     if (ns_uri!=0) return strcmp(ns_uri, expected_ns->ns)==0;
-    else return true; // if no namespace we presume it's the expected one
+    else return TRUE; // if no namespace we presume it's the expected one
   }
 }
 
@@ -131,7 +131,7 @@ void write_ns_prefix(const XERdescriptor_t& p_td, TTCN_Buffer& p_buf)
 {
   if (p_td.my_module != 0 && p_td.ns_index != -1
     && !(p_td.xer_bits & FORM_UNQUALIFIED)) {
-    const namespace_t *my_ns = p_td.my_module->get_ns(p_td.ns_index);
+    const namespace_t *my_ns = p_td.my_module->get_ns((size_t)p_td.ns_index);
     if (my_ns->px[0] != 0) { // not an empty prefix
       p_buf.put_s(strlen(my_ns->px), (cbyte*)my_ns->px);
       p_buf.put_c(':');
@@ -156,9 +156,9 @@ void check_namespace_restrictions(const XERdescriptor_t& p_td, const char* p_xml
 {
   // In case of "anyElement from ..." matching namespaces is good
   // in case of "anyElement except ..." matching namespaces is bad
-  bool ns_match_allowed = (p_td.xer_bits & ANY_FROM) ? true : false;
+  boolean ns_match_allowed = (p_td.xer_bits & ANY_FROM) ? TRUE : FALSE;
 
-  bool ns_error = ns_match_allowed;
+  boolean ns_error = ns_match_allowed;
   for (unsigned short idx = 0; idx < p_td.nof_ns_uris; ++idx) {
     if ((p_xmlns == 0 && strlen(p_td.ns_uris[idx]) == 0) ||
         (p_xmlns != 0 && strcmp(p_td.ns_uris[idx], p_xmlns) == 0))
@@ -178,4 +178,4 @@ void check_namespace_restrictions(const XERdescriptor_t& p_td, const char* p_xml
         "The unqualified XML namespace is %s namespace list.", ns_match_allowed ? "not in the allowed" : "in the excluded");
     }
   }
-}
\ No newline at end of file
+}
