@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -26,7 +26,7 @@
 // Author:                Janos Zoltan Szabo
 // mail:                  tmpjsz@eth.ericsson.se
 //
-// Copyright (c) 2000-2016 Ericsson Telecom AB
+// Copyright (c) 2000-2017 Ericsson Telecom AB
 //
 #ifndef MCTR_MAINCONTROLLER_H
 #define MCTR_MAINCONTROLLER_H
@@ -523,10 +523,10 @@ private:
     const char *local_port, component remote_comp, const char *remote_port);
   static void send_disconnect_ack(component_struct *tc);
   static void send_map(component_struct *tc,
-    const char *local_port, const char *system_port);
+    const char *local_port, const char *system_port, boolean translate);
   static void send_map_ack(component_struct *tc);
   static void send_unmap(component_struct *tc,
-    const char *local_port, const char *system_port);
+    const char *local_port, const char *system_port, boolean translate);
   static void send_unmap_ack(component_struct *tc);
   static void send_debug_command(int fd, int commandID, const char* arguments);
   static void send_debug_setup(host_struct *hc);
