@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -33,6 +33,9 @@ ComplexType::ComplexType(XMLParser * a_parser, TTCN3Module * a_module, Construct
 , with_union(false)
 , first_child(false)
 , fromAll(false)
+, listPrint(false)
+, listMinOccurs(1)
+, listMaxOccurs(1)
 , max_alt(0)
 , skipback(0)
 , lastType()
@@ -60,6 +63,9 @@ ComplexType::ComplexType(ComplexType & other)
 , with_union(other.with_union)
 , first_child(other.first_child)
 , fromAll(other.fromAll)
+, listPrint(other.listPrint)
+, listMinOccurs(other.listMinOccurs)
+, listMaxOccurs(other.listMaxOccurs)
 , max_alt(other.max_alt)
 , skipback(other.skipback)
 , lastType(other.lastType)
@@ -107,6 +113,9 @@ ComplexType::ComplexType(ComplexType * other)
 , with_union(false)
 , first_child(false)
 , fromAll(false)
+, listPrint(false)
+, listMinOccurs(1)
+, listMaxOccurs(1)
 , max_alt(0)
 , skipback(0)
 , lastType()
@@ -136,6 +145,9 @@ ComplexType::ComplexType(const SimpleType & other, CT_fromST c)
 , with_union(false)
 , first_child(false)
 , fromAll(false)
+, listPrint(false)
+, listMinOccurs(1)
+, listMaxOccurs(1)
 , max_alt(0)
 , skipback(0)
 , lastType()
@@ -283,7 +295,7 @@ void ComplexType::loadWithValues() {
     case n_restriction:
       mode = restrictionMode;
       //If it is an xsd:union then call SimpleType::loadWithValues
-      if (parent != NULL && parent->with_union) {
+      if (parent != NULL && parent->with_union && parent->hasVariant(Mstring("useUnion"))) {
         SimpleType::loadWithValues();
         break;
       }
@@ -294,6 +306,7 @@ void ComplexType::loadWithValues() {
         f->type.upload(atts.base);
         f->setReference(atts.base);
         f->addVariant(V_untagged);
+        f->mode = restrictionMode;
         complexfields.push_back(f);
         basefield = f;
         actfield = f;
@@ -317,6 +330,7 @@ void ComplexType::loadWithValues() {
         f->type.upload(atts.base);
         f->setReference(atts.base);
         f->addVariant(V_untagged);
+        f->mode = extensionMode;
         complexfields.push_back(f);
         basefield = f;
         actfield = f;
@@ -419,8 +433,6 @@ void ComplexType::loadWithValues() {
       attribute->addVariant(V_attribute);
       attribute->applyMinMaxOccursAttribute(0, 1);
       attribute->setXsdtype(n_attribute);
-      attribute->applyDefaultAttribute(atts.default_);
-      attribute->applyFixedAttribute(atts.fixed);
       attribute->setUseVal(atts.use);
       attribute->setAttributeFormAs(atts.form);
       lastType = n_attribute;
@@ -431,6 +443,8 @@ void ComplexType::loadWithValues() {
       } else {
         attribute->applyRefAttribute(atts.ref);
       }
+      attribute->applyDefaultAttribute(atts.default_);
+      attribute->applyFixedAttribute(atts.fixed);
       actfield = attribute;
       
       //In case of nillable parent it is difficult...
@@ -548,12 +562,14 @@ void ComplexType::loadWithValues() {
     {
       xsdtype = parser->getActualTagName();
       cmode = CT_simpletype_mode;
-      Mstring fieldname;
-      if (with_union) {
+      if (with_union && hasVariant(Mstring("useUnion"))) {
+        Mstring fieldname;
         if (max_alt == 0) {
           fieldname = Mstring("alt_");
         } else {
-          fieldname = mprintf("alt_%d", max_alt);
+          expstring_t new_name = mprintf("alt_%d", max_alt);
+          fieldname = new_name;
+          Free(new_name);
         }
         max_alt++;
         ComplexType * field = new ComplexType(this);
@@ -587,10 +603,38 @@ void ComplexType::loadWithValues() {
       break;
     case n_list:
       if (parent != NULL && parent->basefield == this) {
-        parent->SimpleType::loadWithValues();
+        if (parent->getMaxOccurs() == 1) { // optional or minOccurs = maxOccurs = 1
+          if (parent->getMinOccurs() == 0) {
+            parent->isOptional = true;
+            if (parent->parent != NULL && parent->parent->getXsdtype() == n_choice) {
+              parent->listPrint = true;
+              parent->listMinOccurs = parent->getMinOccurs();
+              parent->listMaxOccurs = parent->getMaxOccurs();
+            }
+          }
+        } else if (parent->parent != NULL){
+          parent->listPrint = true;
+          parent->listMinOccurs = parent->getMinOccurs();
+          parent->listMaxOccurs = parent->getMaxOccurs();
+        }
         parent->basefield = NULL;
+        parent->SimpleType::loadWithValues();
         setInvisible();
       } else if(parent != NULL) {
+        if (getMaxOccurs() == 1) { // optional or minOccurs = maxOccurs = 1
+          if (getMinOccurs() == 0) {
+            isOptional = true;
+            if (parent->getXsdtype() == n_choice) {
+              listPrint = true;
+              listMinOccurs = getMinOccurs();
+              listMaxOccurs = getMaxOccurs();
+            }
+          }
+        } else {
+          listPrint = true;
+          listMinOccurs = getMinOccurs();
+          listMaxOccurs = getMaxOccurs();
+        }
         SimpleType::loadWithValues();
       }
       break;
@@ -628,7 +672,6 @@ void ComplexType::modifyValues() {
   if (xsdtype == n_sequence) {
     skipback = skipback - 1;
   }
-  
 
   if ( parent != NULL && 
       (xsdtype == n_element || 
@@ -782,7 +825,7 @@ void ComplexType::applyReference(const SimpleType & other, const bool on_attribu
   }
 
   builtInBase = other.getBuiltInBase();
-
+  
   length.applyReference(other.getLength());
   pattern.applyReference(other.getPattern());
   enumeration.applyReference(other.getEnumeration());
@@ -819,7 +862,7 @@ void ComplexType::nameConversion_names(const List<NamespaceType> &) {
   if (!found) {
     addVariant(V_onlyValue, var);
   }
-  for (List<SimpleType*>::iterator dep = nameDepList.begin(); dep; dep = dep->Next) {
+  for (List<RootType*>::iterator dep = nameDepList.begin(); dep; dep = dep->Next) {
     dep->Data->setTypeValue(res);
   }
 }
@@ -966,10 +1009,10 @@ void ComplexType::setFieldPaths(Mstring path) {
   }
 }
 
-void ComplexType::finalModification2() {  
+void ComplexType::subFinalModification() {  
   //Call SimpleType finalModification
   SimpleType::finalModification();
-
+  
   //Set isOptional field
   isOptional = isOptional || (getMinOccurs() == 0 && getMaxOccurs() == 1);
   
@@ -984,7 +1027,7 @@ void ComplexType::finalModification2() {
       complexfields.remove(field);
     } else {
       //Recursive call
-      field->Data->finalModification2();
+      field->Data->subFinalModification();
       //collect <xsd:all> elements
       if (field->Data->fromAll) {
         enumNames.push_back(field->Data->getName().convertedValue);
@@ -1034,7 +1077,9 @@ void ComplexType::finalModification2() {
         if (number == 0) {
           field->Data->name.upload(Mstring("alt_"));
         } else {
-          field->Data->name.upload(Mstring(mprintf("alt_%d", number)));
+          expstring_t new_name = mprintf("alt_%d", number);
+          field->Data->name.upload(Mstring(new_name));
+          Free(new_name);
         }
         number++;
       }
@@ -1081,14 +1126,29 @@ void ComplexType::finalModification2() {
 }
 
 void ComplexType::finalModification() {
-  finalModification2();
+  subFinalModification();
   setFieldPaths(empty_string);
+}
+
+void ComplexType::finalModification2() {
+  subFinalModification2();
   List<Mstring> container;
   collectVariants(container);
   variant.clear();
   variant = container;
 }
 
+void ComplexType::subFinalModification2() {
+  SimpleType::finalModification2();
+  for (List<ComplexType*>::iterator field = complexfields.begin(); field; field = field->Next) {
+    //Recursive call
+    field->Data->subFinalModification2();
+  }
+  for (List<AttributeType*>::iterator field = attribfields.begin(); field; field = field->Next) {
+    field->Data->SimpleType::finalModification2();
+  }
+}
+
 void ComplexType::printToFile(FILE * file) {
   printToFile(file, 0, false);
 }
@@ -1106,11 +1166,12 @@ void ComplexType::printToFile(FILE * file, const unsigned level, const bool is_u
     }else {
       fprintf(file, "%s %s", type.convertedValue.c_str(), name.convertedValue.c_str());
     }
-    fprintf(file, "\n{\n");
-
-    if (attribfields.empty() && complexfields.empty()) {
-      fprintf(file, "\n");
-    }
+    if(type.convertedValue == "record" || type.convertedValue == "union"){
+      fprintf(file, "\n{\n");
+      if (attribfields.empty() && complexfields.empty()) {
+        fprintf(file, "\n");
+      }
+    } 
 
     for (List<ComplexType*>::iterator c = complexfields.begin(), nextField; c; c = nextField) {
       nextField = c->Next;
@@ -1191,24 +1252,39 @@ void ComplexType::printToFile(FILE * file, const unsigned level, const bool is_u
         if ((multiplicity > 1) && getReference().get_ref()) {
           fprintf(file, "%s.", getReference().get_ref()->getModule()->getModulename().c_str());
         }
-        if (field_is_record || field_is_union) {
-          printMinOccursMaxOccurs(file, with_union, !first_child || parent->getXsdtype() != n_choice);
-          fprintf(file, "%s {\n", getType().convertedValue.c_str());
-          for (List<AttributeType*>::iterator f = attribfields.begin(); f; f = f->Next) {
-            f->Data->printToFile(file, level + 1);
-            if (f->Next != NULL || !complexfields.empty()) {
-              fprintf(file, ",\n");
-            } else {
-              fprintf(file, "\n");
-            }
+        if (field_is_record || field_is_union || listPrint) {
+          unsigned long long int tempMin = getMinOccurs();
+          unsigned long long int tempMax = getMaxOccurs();
+          if (listPrint) {
+            setMinOccurs(listMinOccurs);
+            setMaxOccurs(listMaxOccurs);
           }
+          printMinOccursMaxOccurs(file, with_union, !first_child || parent->getXsdtype() != n_choice);
+          if (listPrint) {
+            setMinOccurs(tempMin);
+            setMaxOccurs(tempMax);
+          }
+          if (listPrint && complexfields.size() == 0) {
+            printMinOccursMaxOccurs(file, false);
+            fprintf(file, "%s ",getType().convertedValue.c_str());
+          } else {
+            fprintf(file, "%s {\n", getType().convertedValue.c_str());
+            for (List<AttributeType*>::iterator f = attribfields.begin(); f; f = f->Next) {
+              f->Data->printToFile(file, level + 1);
+              if (f->Next != NULL || !complexfields.empty()) {
+                fprintf(file, ",\n");
+              } else {
+                fprintf(file, "\n");
+              }
+            }
 
-          for (List<ComplexType*>::iterator c = complexfields.begin(); c; c = c->Next) {
-            c->Data->printToFile(file, level + 1, is_union);
-            if (c->Next != NULL) {
-              fprintf(file, ",\n");
-            } else {
-              fprintf(file, "\n");
+            for (List<ComplexType*>::iterator c = complexfields.begin(); c; c = c->Next) {
+              c->Data->printToFile(file, level + 1, is_union);
+              if (c->Next != NULL) {
+                fprintf(file, ",\n");
+              } else {
+                fprintf(file, "\n");
+              }
             }
           }
         } else {
@@ -1230,7 +1306,7 @@ void ComplexType::printToFile(FILE * file, const unsigned level, const bool is_u
           }
         }
       }
-      if (field_is_record || field_is_union) {
+      if ((field_is_record || field_is_union) && !listPrint) {
         indent(file, level);
         fprintf(file, "} ");
       }
@@ -1246,7 +1322,9 @@ void ComplexType::printToFile(FILE * file, const unsigned level, const bool is_u
   }
 
   if (top) {
-    fprintf(file, "}");
+    if(type.convertedValue == "record" || type.convertedValue == "union"){
+      fprintf(file, "}");
+    }
     if(mode == listMode){
       fprintf(file, " %s", name.convertedValue.c_str());
     }
@@ -1443,16 +1521,14 @@ void ComplexType::setMinMaxOccurs(const unsigned long long min, const unsigned l
       }
       if (parent != NULL && parent->getXsdtype() == n_choice) {
         name.list_extension = true;
-        if ((parent != NULL && parent->getXsdtype() == n_choice)) {
-          if (parent->first_child == false && getMinOccurs() == 0) {
-            parent->first_child = true;
-            with_union = true;
-            first_child = false;
-          } else {
-            with_union = true;
-            first_child = true;
-          }
-        }
+        if (parent->first_child == false && getMinOccurs() == 0) {
+          parent->first_child = true;
+          with_union = true;
+          first_child = false;
+        } else {
+          with_union = true;
+          first_child = true;
+        }       
       }
     }
   }
@@ -1481,8 +1557,8 @@ void ComplexType::applyNamespaceAttribute(VariantMode varLabel, const Mstring& n
   // of XML Schema. It is either ##any, ##other, ##local, ##targetNamespace,
   // or a list of (namespace reference | ##local | ##targetNamespace).
   for (List<Mstring>::iterator ns = namespaces.begin(); ns; ns = ns->Next) {
-    static const Mstring xxany("##any"), xxother("##other"), xxlocal("##local"),
-      xxtargetNamespace("##targetNamespace");
+    static char xxany[] = "##any", xxother[] = "##other", xxlocal[] = "##local",
+      xxtargetNamespace[] = "##targetNamespace";
     if (!first) any_ns += ',';
 
     if (ns->Data == xxany) {
@@ -1564,8 +1640,11 @@ void ComplexType::applyAttributeExtension(ComplexType * found_CT, AttributeType
 }
 
 //Attribute restriction logic when restricting complextypes
-void ComplexType::applyAttributeRestriction(ComplexType * found_CT) {
-  for (List<AttributeType*>::iterator attr = attribfields.begin(), nextAttr; attr; attr = nextAttr) {
+bool ComplexType::applyAttributeRestriction(ComplexType * found_CT) {
+  bool isModifiedAttr = false;
+  // when the element is nillable the attributes are in the parent
+  ComplexType * me = parent != NULL && nillable ? parent : this;
+  for (List<AttributeType*>::iterator attr = me->attribfields.begin(), nextAttr; attr; attr = nextAttr) {
     nextAttr = attr->Next;
     bool l = false;
     for (List<AttributeType*>::iterator attr2 = found_CT->attribfields.begin(); attr2; attr2 = attr2->Next) {
@@ -1578,31 +1657,35 @@ void ComplexType::applyAttributeRestriction(ComplexType * found_CT) {
     if (!l) {
       delete attr->Data;
       attr->Data = NULL;
-      attribfields.remove(attr);
+      me->attribfields.remove(attr);
     }
   }
   size_t size = found_CT->attribfields.size();
-  size_t size2 = attribfields.size();
+  size_t size2 = me->attribfields.size();
   size_t i = 0;
   List<AttributeType*>::iterator attr = found_CT->attribfields.begin();
   for (; i < size; attr = attr->Next, i = i + 1) {
     bool l = false;
     size_t j = 0;
-    List<AttributeType*>::iterator attr2 = attribfields.begin();
+    List<AttributeType*>::iterator attr2 = me->attribfields.begin();
     for (; j < size2; attr2 = attr2->Next, j = j + 1) {
       if (attr->Data->getName().convertedValue == attr2->Data->getName().convertedValue &&
         attr->Data->getType().convertedValue == attr2->Data->getType().convertedValue && !attr2->Data->getUsed()) {
         l = true;
+        if(attr->Data->getUseVal() != attr2->Data->getUseVal()){
+          isModifiedAttr = true;
+        }
         attr2->Data->setUsed(true);
         break;
       }
     }
     if (!l) {
       AttributeType * newAttrib = new AttributeType(*attr->Data);
-      attribfields.push_back(newAttrib);
-      setParent(this, newAttrib);
+      me->attribfields.push_back(newAttrib);
+      setParent(me, newAttrib);
     }
   }
+  return isModifiedAttr;
 }
 
 void ComplexType::addNameSpaceAsVariant(RootType * root, RootType * other) {
@@ -1621,10 +1704,19 @@ void ComplexType::resolveAttribute(AttributeType* attr) {
         attr->setTypeOfField(st->getName().convertedValue);
         attr->setNameOfField(st->getName().originalValueWoPrefix);
         attr->setOrigModule(st->getModule());
+        st->addToNameDepList(attr);
+        if (attr->getConstantDefaultForEmpty() != NULL) {
+          st->addToNameDepList((RootType*)attr->getConstantDefaultForEmpty());
+        }
       } else {
         attr->setTypeOfField(st->getName().convertedValue);
-        if (st->getType().convertedValue == "record" || st->getType().convertedValue == "union") {
+        if (st->getType().convertedValue == "record" || st->getType().convertedValue == "union"
+            || st->getXsdtype() == n_NOTSET) // It really is a simpleType
+          {
             st->addToNameDepList(attr);
+            if (attr->getConstantDefaultForEmpty() != NULL) {
+              st->addToNameDepList((RootType*)attr->getConstantDefaultForEmpty());
+            }
           }
       }
       attr->getReference().set_resolved(st);
@@ -1662,14 +1754,14 @@ void ComplexType::resolveAttributeGroup(SimpleType * st) {
       }
     }
     for (List<AttributeType*>::iterator attr = ct->attribfields.begin(); attr; attr = attr->Next) {
-      AttributeType * attrib = new AttributeType(*attr->Data);
-      attr->Data->setOrigModule(ct->getModule());
-      if (addNameSpaceas) {
-        attrib->addVariant(V_namespaceAs, ct->getModule()->getTargetNamespace());
-      }
       if (anyAttrib != NULL && attr->Data->isAnyAttribute()) {
         anyAttrib->Data->addNameSpaceAttribute(attr->Data->getNameSpaceAttribute());
       } else {
+        AttributeType * attrib = new AttributeType(*attr->Data);
+        attr->Data->setOrigModule(ct->getModule());
+        if (addNameSpaceas) {
+          attrib->addVariant(V_namespaceAs, ct->getModule()->getTargetNamespace());
+        }
         //Nillable attribute placement is hard...
         if (parent->nillable && parent->parent != NULL) {
           parent->parent->attribfields.push_back(attrib);
@@ -1762,12 +1854,21 @@ void ComplexType::resolveElement(SimpleType *st) {
     if(st->getSubstitution() != NULL){
       st->getSubstitution()->addToNameDepList(this);
       nameDep = st->getSubstitution();
+      if (defaultForEmptyConstant != NULL) {
+        st->getSubstitution()->addToNameDepList((RootType*)defaultForEmptyConstant);
+      }
     }if(st->getTypeSubstitution() != NULL){
       st->getTypeSubstitution()->addToNameDepList(this);
       nameDep = st->getTypeSubstitution();
+      if (defaultForEmptyConstant != NULL) {
+        st->getTypeSubstitution()->addToNameDepList((RootType*)defaultForEmptyConstant);
+      }
     }else {
       st->addToNameDepList(this);
       nameDep = st;
+      if (defaultForEmptyConstant != NULL) {
+        st->addToNameDepList((RootType*)defaultForEmptyConstant);
+      }
     }
   }
 }
@@ -1781,28 +1882,73 @@ void ComplexType::resolveSimpleTypeExtension() {
         if (ct->resolved == No) {
           ct->referenceResolving();
         }
-        basefield->outside_reference.set_resolved(ct);
-        ct->basefield->addToNameDepList(basefield);
-        basefield->nameDep = ct->basefield;
-        basefield->mode = extensionMode;
-        basefield->applyReference(*ct->basefield, true);
-        addNameSpaceAsVariant(basefield, ct->basefield);
-        applyAttributeExtension(ct);
+        // If an alias
+        if (attribfields.empty()) {
+          for (List<ComplexType*>::iterator field = complexfields.begin(); field; field = field->Next) {
+            field->Data->setInvisible();
+          }
+
+          for (List<AttributeType*>::iterator field = attribfields.begin(); field; field = field->Next) {
+            field->Data->setInvisible();
+          }
+          
+          basefield->setInvisible();
+          type.upload(st->getName().originalValueWoPrefix);
+          ct->addToNameDepList(this);
+          nameDep = ct;
+          addNameSpaceAsVariant(this, st);
+          alias = ct;
+        } else {
+          
+          while (ct != NULL && ct->getAlias() != NULL) {
+            ct = (ComplexType*)ct->getAlias();
+          }
+          
+          basefield->outside_reference.set_resolved(ct);
+          ct->basefield->addToNameDepList(basefield);
+          basefield->nameDep = ct->basefield;
+          basefield->mode = extensionMode;
+          basefield->applyReference(*ct->basefield, true);
+          addNameSpaceAsVariant(basefield, ct->basefield);
+          applyAttributeExtension(ct);
+        }
       } else {
         if (!st->getReference().empty() && !st->getReference().is_resolved()) {
           st->referenceResolving();
         }
-        st->addToNameDepList(basefield);
-        basefield->nameDep = st;
-        addNameSpaceAsVariant(basefield, st);
-        const Mstring old_type = basefield->getType().originalValueWoPrefix;
-        basefield->applyReference(*st);
-        // If st has enumeration then the type is restored to the original value
-        // because enumerations cannot be extended here and this way we just
-        // create an alias.
-        if (st->getEnumeration().modified) {
-          basefield->setTypeValue(old_type);
-          basefield->getEnumeration().modified = false;
+        
+        bool hasRestrOrExt = basefield->hasRestrictionOrExtension();
+        // If an alias
+        if(!hasRestrOrExt && attribfields.empty()){
+          // Set the fields and attributes invisible
+          for (List<ComplexType*>::iterator field = complexfields.begin(); field; field = field->Next) {
+            field->Data->setInvisible();
+          }
+
+          for (List<AttributeType*>::iterator field = attribfields.begin(); field; field = field->Next) {
+            field->Data->setInvisible();
+          }
+          basefield->setInvisible();
+          type.upload(st->getName().originalValueWoPrefix);
+          st->addToNameDepList(this);
+          nameDep = st;
+          addNameSpaceAsVariant(this, st);
+          alias = st;
+        } else {
+          while(st != NULL && st->getAlias() != NULL) {
+            st = st->getAlias();
+          }
+          basefield->builtInBase = st->getBuiltInBase();
+          addNameSpaceAsVariant(basefield, st);
+          const Mstring old_type = basefield->getType().originalValueWoPrefix;
+          basefield->applyReference(*st);
+          // If st has enumeration then the type is restored to the original value
+          // because enumerations cannot be extended here and this way we just
+          // create an alias.
+          if (st->getEnumeration().modified) {
+            basefield->setTypeValue(old_type);
+            basefield->getEnumeration().modified = false;
+          }
         }
       }
     } else if(!isBuiltInType(basefield->getType().convertedValue)){
@@ -1824,6 +1970,28 @@ void ComplexType::resolveSimpleTypeRestriction() {
       TTCN3ModuleInventory::getInstance().incrNumErrors();
       return;
     }
+    bool hasRestrOrExt = basefield->hasRestrictionOrExtension();
+    // If an alias
+    if(!hasRestrOrExt && attribfields.empty()){
+      type.upload(st->getName().convertedValue);
+      st->addToNameDepList(this);
+      nameDep = st;
+      alias = st;
+      basefield->setInvisible();
+      for (List<ComplexType*>::iterator field = complexfields.begin(); field; field = field->Next) {
+        field->Data->setInvisible();
+      }
+
+      for (List<AttributeType*>::iterator field = attribfields.begin(); field; field = field->Next) {
+        field->Data->setInvisible();
+      }
+      addNameSpaceAsVariant(this, st);
+      return;
+    }
+    
+    while (st != NULL && st->getAlias() != NULL) {
+      st = st->getAlias();
+    }
     basefield->outside_reference.set_resolved(st);
     if (st->getXsdtype() != n_NOTSET) {
       ComplexType * ct = (ComplexType*) st;
@@ -1868,6 +2036,7 @@ void ComplexType::resolveSimpleTypeRestriction() {
       return;
     }
     
+    // Alias not possible here, because basefield->outside_reference is empty
     basefield->outside_reference.set_resolved(ct);
     if (ct != NULL) {
       if (ct->resolved == No) {
@@ -1907,6 +2076,25 @@ void ComplexType::resolveComplexTypeExtension() {
       if (ct->resolved == No) {
         ct->referenceResolving();
       }
+      // it is an alias
+      if (complexfields.empty() && attribfields.empty()) {
+        type.upload(ct->getName().convertedValue);
+        
+        for (List<ComplexType*>::iterator field = complexfields.begin(); field; field = field->Next) {
+          field->Data->setInvisible();
+        }
+        
+        for (List<AttributeType*>::iterator field = attribfields.begin(); field; field = field->Next) {
+          field->Data->setInvisible();
+        }
+        ct->addToNameDepList(this);
+        nameDep = ct;
+        alias = ct;
+        return;
+      }
+      while (ct != NULL && ct->getAlias() != NULL) {
+        ct = (ComplexType*)ct->getAlias();
+      }
       List<AttributeType*>::iterator anyAttr = attribfields.begin();
       for (; anyAttr; anyAttr = anyAttr->Next) {
         if (anyAttr->Data->isAnyAttribute()) {
@@ -1956,12 +2144,38 @@ void ComplexType::resolveComplexTypeExtension() {
 void ComplexType::resolveComplexTypeRestriction() {
   if (mode == restrictionMode && cmode == CT_complextype_mode && !outside_reference.empty()) {
     ComplexType * ct = (ComplexType*) TTCN3ModuleInventory::getInstance().lookup(this, want_CT);
-    if(ct->getXsdtype() != n_NOTSET){
+    if (ct == NULL) {
+      printError(module->getSchemaname(), name.convertedValue,
+        "Reference for a non-defined type: " + getReference().repr());
+      TTCN3ModuleInventory::getInstance().incrNumErrors();
+      return;
+    }
+    if(ct->getXsdtype() != n_NOTSET) {
       if (ct->resolved == No) {
         ct->referenceResolving();
       }
+      ComplexType * maybeAlias = ct;
+      while (maybeAlias != NULL && maybeAlias->getAlias() != NULL) {
+        maybeAlias = (ComplexType*)maybeAlias->getAlias();
+      }
+      bool isModifiedAttr = applyAttributeRestriction(maybeAlias);
+      if(!isModifiedAttr && !hasComplexRestriction(maybeAlias)){
+        type.upload(ct->getName().convertedValue);
+        
+        for (List<ComplexType*>::iterator field = complexfields.begin(); field; field = field->Next) {
+          field->Data->setInvisible();
+        }
+        
+        for (List<AttributeType*>::iterator field = attribfields.begin(); field; field = field->Next) {
+          field->Data->setInvisible();
+        }
+        ct->addToNameDepList(this);
+        nameDep = ct;
+        alias = ct;
+        return;
+      }
+      
       outside_reference.set_resolved(ct);
-      applyAttributeRestriction(ct);
 
       size_t size = complexfields.size();
       size_t i = 0;
@@ -1970,7 +2184,7 @@ void ComplexType::resolveComplexTypeRestriction() {
         List<ComplexType*>::iterator field2 = ct->complexfields.begin();
         for (; field2; field2 = field2->Next) {
           if (field->Data->getName().convertedValue == field2->Data->getName().convertedValue &&
-            field->Data->getType().convertedValue == field2->Data->getType().convertedValue &&
+            field->Data->getType().convertedValue.getValueWithoutPrefix(':') == field2->Data->getType().convertedValue.getValueWithoutPrefix(':') &&
             field->Data->complexfields.size() <= field2->Data->complexfields.size() &&
             hasMatchingFields(field->Data->complexfields, field2->Data->complexfields)) {
             // TODO: better algorithm to find matching fields
@@ -2005,6 +2219,28 @@ bool ComplexType::hasMatchingFields(const List<ComplexType*>& mainList, const Li
     return true;
 }
 
+bool ComplexType::hasComplexRestriction(ComplexType* ct) const {
+  if(complexfields.size() != ct->complexfields.size() || hasRestrictionOrExtension()) {
+    return true;
+  }
+  
+  for(List<ComplexType*>::iterator field = complexfields.begin(); field; field = field->Next) {
+    for(List<ComplexType*>::iterator field2 = ct->complexfields.begin(); field2; field2 = field2->Next) {
+      if(field->Data->getName().convertedValue.getValueWithoutPrefix(':') == field2->Data->getName().convertedValue.getValueWithoutPrefix(':') &&
+         field->Data->getType().convertedValue.getValueWithoutPrefix(':') == field2->Data->getType().convertedValue.getValueWithoutPrefix(':')) {
+        if(field->Data->getMinOccurs() == field2->Data->getMinOccurs() && field->Data->getMaxOccurs() == field2->Data->getMaxOccurs()){
+          if(field->Data->hasComplexRestriction(field2->Data)) {
+            return true;
+          }
+        } else {
+          return true;
+        }
+      }
+    }
+  }
+  return false;
+}
+
 void ComplexType::resolveUnion(SimpleType *st) {
   if (parent != NULL && parent->with_union && xsdtype == n_simpleType && !outside_reference.empty()) {
     if (st->getXsdtype() != n_NOTSET) {
