@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -20,11 +20,11 @@
 #include "SimpleType.hh"
 #include "TTCN3Module.hh"
 #include "ImportStatement.hh"
+#include "converter.hh"
 
 #include <cctype> // for using "toupper" function
 #include <cstring>
 #include <cstdio>
-#include <cmath>
 #include <regex.h>
 
 #include "../common/version_internal.h"
@@ -136,7 +136,7 @@ void XSDName2TTCN3Name(const Mstring& in, QualifiedNames & used_names, modeType
 
     if (type_of_the_name == type_reference_name) {
       if (isBuiltInType(res)) {
-        res[0] = toupper(res[0]);
+        res[0] = (char)toupper(res[0]);
         res = "XSD." + res;
         return;
       }
@@ -616,12 +616,30 @@ void printWarning(const Mstring& filename, const Mstring& typeName, const Mstrin
     text.c_str());
 }
 
-void indent(FILE* file, const int x) {
-  for (int l = 0; l < x; ++l) {
+void indent(FILE* file, const unsigned int x) {
+  for (unsigned int l = 0; l < x; ++l) {
     fprintf(file, "\t");
   }
 }
 
+Mstring xmlFloat2TTCN3FloatStr(const Mstring& xmlFloat) {
+  Mstring result;
+  if (xmlFloat == "INF") {
+    result = "infinity";
+  } else if (xmlFloat == "-INF") {
+    result = "-infinity";
+  } else if (xmlFloat == "NaN") {
+    result = "not_a_number";
+  } else if (!xmlFloat.isFound('.') && !xmlFloat.isFound('e') && !xmlFloat.isFound('E')) {
+    // Float types need the .0 if it is a single integer or 
+    // translate special float values to TTCN3.
+    result = xmlFloat + Mstring(".0");
+  } else {
+    result = xmlFloat;
+  }
+  return result;
+}
+
 long double stringToLongDouble(const char *input) {
   long double result = 0.0;
   // `strtold()' is not available on older platforms.
