@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -103,7 +103,9 @@ void RootType::addVariant(const VariantMode var, const Mstring& var_value, const
       variantstring = "\"controlNamespace" + var_value + "\"";
       break;
     case V_defaultForEmpty:
-      variantstring = "\"defaultForEmpty as \'" + var_value + "\'\""; // chapter 7.1.5
+      // The apostrophes are not needed because the var_value will be a constant
+      // reference.
+      variantstring = "\"defaultForEmpty as " + var_value + "\""; // chapter 7.1.5
       break;
     case V_element:
       variantstring = "\"element\"";
@@ -203,7 +205,7 @@ void RootType::addComment(const Mstring& text) {
   comment.push_back(Mstring("/* " + text + " */\n"));
 }
 
-void RootType::printComment(FILE * file, int level) {
+void RootType::printComment(FILE * file, unsigned int level) {
   if (!c_flag_used && !comment.empty()) {
     for (List<Mstring>::iterator c = comment.begin(); c; c = c->Next) {
       indent(file, level);
@@ -295,4 +297,4 @@ void TypeType::checkBuintInTypeReference() {
     printError(t_module->getSchemaname(), t_parser->getActualLineNumber(), Mstring("Cannot find the namespace of type: ") + originalValueWoPrefix);
     t_parser->incrNumErrors();
   }
-}
\ No newline at end of file
+}
