@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -21,10 +21,27 @@
 #include "TTCN3ModuleInventory.hh"
 #include "TTCN3Module.hh"
 #include "ComplexType.hh"
+#include "Constant.hh"
+#include "converter.hh"
+
+#include <math.h>
+#include <cfloat>
 
 extern bool g_flag_used;
 extern bool h_flag_used;
 
+#ifndef INFINITY
+#define INFINITY (DBL_MAX*DBL_MAX)
+#endif
+const double PLUS_INFINITY(INFINITY);
+const double MINUS_INFINITY(-INFINITY);
+
+#ifdef NAN
+const double NOT_A_NUMBER(NAN);
+#else
+const double NOT_A_NUMBER((double)PLUS_INFINITY+(double)MINUS_INFINITY);
+#endif
+
 SimpleType::SimpleType(XMLParser * a_parser, TTCN3Module * a_module, ConstructType a_construct)
 : RootType(a_parser, a_module, a_construct)
 , builtInBase()
@@ -47,6 +64,8 @@ SimpleType::SimpleType(XMLParser * a_parser, TTCN3Module * a_module, ConstructTy
 , addedToTypeSubstitution(false)
 , block(not_set)
 , inList(false)
+, alias(NULL)
+, defaultForEmptyConstant(NULL)
 , parent(NULL) {
 }
 
@@ -72,6 +91,8 @@ SimpleType::SimpleType(const SimpleType& other)
 , addedToTypeSubstitution(other.addedToTypeSubstitution)
 , block(other.block)
 , inList(other.inList)
+, alias(other.alias)
+, defaultForEmptyConstant(other.defaultForEmptyConstant)
 , parent(NULL) {
   length.parent = this;
   pattern.parent = this;
@@ -243,10 +264,14 @@ void SimpleType::loadWithValues() {
 void SimpleType::applyDefaultAttribute(const Mstring& default_value) {
   if (!default_value.empty()) {
     value.default_value = default_value;
-  const Mstring nameT = type.originalValueWoPrefix.getValueWithoutPrefix(':');
+    const Mstring typeT = type.originalValueWoPrefix.getValueWithoutPrefix(':');
     //Not supported for hexBinary and base64Binary
-    if (nameT != "hexBinary" && nameT != "base64Binary") {
-      addVariant(V_defaultForEmpty, default_value);
+    if (typeT != "hexBinary" && typeT != "base64Binary") {
+      Constant * c = new Constant(this, type.convertedValue, default_value);
+      module->addConstant(c);
+      defaultForEmptyConstant = c;
+      // Delay adding the defaultForEmpty variant after the nameconversion
+      // happened on the constants
     }
   }
 }
@@ -255,10 +280,14 @@ void SimpleType::applyFixedAttribute(const Mstring& fixed_value) {
   if (!fixed_value.empty()) {
     value.fixed_value = fixed_value;
     value.modified = true;
-    const Mstring nameT = type.originalValueWoPrefix.getValueWithoutPrefix(':');
+    const Mstring typeT = type.originalValueWoPrefix.getValueWithoutPrefix(':');
     //Not supported for hexBinary and base64Binary
-    if (nameT != "hexBinary" && nameT != "base64Binary") {
-      addVariant(V_defaultForEmpty, fixed_value);
+    if (typeT != "hexBinary" && typeT != "base64Binary") {
+      Constant * c = new Constant(this, type.convertedValue, fixed_value);
+      module->addConstant(c);
+      defaultForEmptyConstant = c;
+      // Delay adding the defaultForEmpty variant after the nameconversion
+      // happened on the constants
     }
   }
 }
@@ -311,7 +340,7 @@ void SimpleType::addToSubstitutions(){
   //Simpletype
   if(st->subsGroup == NULL){
     ComplexType * head_element = new ComplexType(*st, ComplexType::fromTagSubstitution);
-    for(List<SimpleType*>::iterator simpletype = st->nameDepList.begin(); simpletype; simpletype = simpletype->Next){
+    for(List<RootType*>::iterator simpletype = st->nameDepList.begin(); simpletype; simpletype = simpletype->Next){
       head_element->getNameDepList().push_back(simpletype->Data);
     }
     st->nameDepList.clear();
@@ -345,10 +374,11 @@ void SimpleType::addToTypeSubstitutions() {
   //It would be nice if here outside_reference.resolved to everything
   bool newST = false;
   SimpleType * st = (SimpleType*)outside_reference.get_ref();
-  if(st == NULL && !isBuiltInType(type.convertedValue)){
+  bool isConvertedBuiltIn = isBuiltInType(type.convertedValue);
+  if(st == NULL && !isConvertedBuiltIn){
     //Not even a reference, and not a built in type
     return;
-  }else if(st == NULL && isBuiltInType(type.convertedValue)){
+  }else if(st == NULL && isConvertedBuiltIn){
     st = new SimpleType(parser, module, construct);
     st->type.upload(type.convertedValue);
     st->name.upload(type.convertedValue);
@@ -366,12 +396,14 @@ void SimpleType::addToTypeSubstitutions() {
   if(st->getTypeSubstitution() == NULL){
     ComplexType * head_element = new ComplexType(*st, ComplexType::fromTypeSubstitution);
     head_element->getNameDepList().clear();
-    for(List<SimpleType*>::iterator simpletype = st->nameDepList.begin(), nextST; simpletype; simpletype = nextST){
-      nextST = simpletype->Next;
+    for(List<RootType*>::iterator roottype = st->nameDepList.begin(), nextST; roottype; roottype = nextST){
+      nextST = roottype->Next;
       //Don't add if it is in a type substitution 
-      if(simpletype->Data->getTypeSubstitution() == NULL){
-        head_element->getNameDepList().push_back(simpletype->Data);
-        st->getNameDepList().remove(simpletype);
+      SimpleType* simpletype = dynamic_cast<SimpleType*>(roottype->Data);
+      if (simpletype == NULL) continue;
+      if(simpletype->getTypeSubstitution() == NULL){
+        head_element->getNameDepList().push_back(roottype->Data);
+        st->getNameDepList().remove(roottype);
       }
     }
     
@@ -595,6 +627,9 @@ void SimpleType::referenceResolving() {
 }
 
 void SimpleType::referenceForST(SimpleType * found_ST) {
+  while(found_ST != NULL && found_ST->alias != NULL){
+    found_ST = found_ST->alias;
+  }
   outside_reference.set_resolved(found_ST);
   if (in_name_only)
     return;
@@ -608,15 +643,27 @@ void SimpleType::referenceForST(SimpleType * found_ST) {
 
   if (mode == listMode || mode == restrictionAfterListMode)
     return;
-
+  
+  bool hasRestrOrExt = hasRestrictionOrExtension();
+  
   length.applyReference(found_ST->length);
   pattern.applyReference(found_ST->pattern);
   enumeration.applyReference(found_ST->enumeration);
   whitespace.applyReference(found_ST->whitespace);
   value.applyReference(found_ST->value);
+  
+  if(!hasRestrOrExt){
+    length.modified = false;
+    pattern.modified = false;
+    enumeration.modified = false;
+    whitespace.modified = false;
+    value.modified = false;
+    alias = found_ST;
+  }
 
   mode = found_ST->mode;
-  if (found_ST->mode != listMode && found_ST->mode != restrictionAfterListMode) {
+  if (found_ST->mode != listMode && found_ST->mode != restrictionAfterListMode
+      && hasRestrOrExt) {
     type.upload(found_ST->getType().convertedValue);
   }
 }
@@ -646,6 +693,7 @@ void SimpleType::referenceForCT(ComplexType * found_CT) {
           expstring_t tmp_string = mprintf("{%s:=%d}",
             field->Data->getName().convertedValue.c_str(), val);
           value.items_with_value.push_back(Mstring(tmp_string));
+          Free(tmp_string);
           break;
         }
       }
@@ -658,6 +706,7 @@ void SimpleType::referenceForCT(ComplexType * found_CT) {
           expstring_t tmp_string = mprintf("{%s:=%f}",
             field->Data->getName().convertedValue.c_str(), val);
           value.items_with_value.push_back(Mstring(tmp_string));
+          Free(tmp_string);
           break;
         }
       }
@@ -667,6 +716,7 @@ void SimpleType::referenceForCT(ComplexType * found_CT) {
           expstring_t tmp_string = mprintf("{%s:=\"%s\"}",
             field->Data->getName().convertedValue.c_str(), itemMisc->Data.c_str());
           value.items_with_value.push_back(Mstring(tmp_string));
+          Free(tmp_string);
           break;
         }
       }
@@ -675,6 +725,7 @@ void SimpleType::referenceForCT(ComplexType * found_CT) {
         expstring_t tmp_string = mprintf("{%s:=\"%s\"}",
           field->Data->getName().convertedValue.c_str(), itemMisc->Data.c_str());
         value.items_with_value.push_back(Mstring(tmp_string));
+        Free(tmp_string);
         break;
       }
     }
@@ -710,9 +761,15 @@ void SimpleType::nameConversion_names() {
   XSDName2TTCN3Name(name.convertedValue, TTCN3ModuleInventory::getInstance().getTypenames(), type_name, res, var);
   name.convertedValue = res;
   addVariant(V_onlyValue, var);
-  for (List<SimpleType*>::iterator st = nameDepList.begin(); st; st = st->Next) {
+  for (List<RootType*>::iterator st = nameDepList.begin(); st; st = st->Next) {
     st->Data->setTypeValue(res);
   }
+  if (outside_reference.get_ref() != NULL && defaultForEmptyConstant!= NULL) {
+    // We don't know if the name conversion already happened on the get_ref()
+    // so we push defaultForEmptyConstant to it's namedeplist.
+    defaultForEmptyConstant->setTypeValue(outside_reference.get_ref()->getType().convertedValue);
+    outside_reference.get_ref()->getNameDepList().push_back(defaultForEmptyConstant);
+  }
 }
 
 void SimpleType::nameConversion_types(const List<NamespaceType> & ns) {
@@ -783,6 +840,17 @@ void SimpleType::finalModification() {
   }
 
   isOptional = isOptional || (getMinOccurs() == 0 && getMaxOccurs() == 0);
+  
+
+}
+
+void SimpleType::finalModification2() {
+  // Delayed adding the defaultForEmpty variant after the nameconversion
+  // happened on the constants
+  if (defaultForEmptyConstant != NULL) {
+    TTCN3Module * mod = parent != NULL ? parent->getModule() : getModule();
+    addVariant(V_defaultForEmpty, defaultForEmptyConstant->getAlterego()->getConstantName(mod));
+  }
 }
 
 bool SimpleType::hasUnresolvedReference() {
@@ -804,6 +872,15 @@ void SimpleType::applyRefAttribute(const Mstring& ref_value) {
   }
 }
 
+bool SimpleType::hasRestrictionOrExtension() const {
+  return
+    enumeration.modified ||
+    length.modified ||
+    value.modified ||
+    pattern.modified ||
+    whitespace.modified;
+}
+
 void SimpleType::printToFile(FILE * file) {
   if (!visible) {
     return;
@@ -853,7 +930,7 @@ void SimpleType::printToFile(FILE * file) {
 
 void SimpleType::dump(unsigned int depth) const {
   static const char *modes[] = {
-    "", "restriction", "extension", "list"
+    "", "restriction", "extension", "list", "restrictionAfterList"
   };
   fprintf(stderr, "%*s SimpleType '%s' -> '%s' at %p\n", depth * 2, "",
     name.originalValueWoPrefix.c_str(), name.convertedValue.c_str(),
@@ -1148,7 +1225,8 @@ EnumerationType::EnumerationType(SimpleType * a_simpleType)
 , items_float()
 , items_time()
 , items_misc()
-, variants() {
+, variants()
+, converted_facets() {
 }
 
 void EnumerationType::applyReference(const EnumerationType & other) {
@@ -1178,6 +1256,7 @@ void EnumerationType::applyFacets() // string types, integer types, float types,
         Mstring res, var;
         XSDName2TTCN3Name(facet->Data, items_string, enum_id_name, res, var);
         text_variants.push_back(var);
+        converted_facets.push_back(res);
       }
     }
     text_variants.sort();
@@ -1197,7 +1276,17 @@ void EnumerationType::applyFacets() // string types, integer types, float types,
             break;
           }
         }
-        if (!found) items_int.push_back(int_value);
+        if (!found) {
+          items_int.push_back(int_value);
+          expstring_t tmp = NULL;
+          if (int_value < 0) {
+            tmp = mputprintf(tmp, "int_%d", abs(int_value));
+          } else {
+            tmp = mputprintf(tmp, "int%d", int_value);
+          }
+          converted_facets.push_back(Mstring(tmp));
+          Free(tmp);
+        }
 
         if (variants.empty() || variants.back() != "\"useNumber\"") {
           variants.push_back(Mstring("\"useNumber\""));
@@ -1209,7 +1298,11 @@ void EnumerationType::applyFacets() // string types, integer types, float types,
     for (List<Mstring>::iterator facet = facets.begin(); facet; facet = facet->Next) {
       double float_value = atof(facet->Data.c_str());
       const ValueType & value = parent->getValue();
-      if (value.lower <= float_value && float_value <= value.upper) {
+      // Value uses DBL_MAX as infinity or -DBL_MAX as -infinity
+      if ((value.lower <= float_value && float_value <= value.upper)
+        || (facet->Data == "-INF" && value.lower == -DBL_MAX)
+        || (facet->Data == "INF" && value.upper == DBL_MAX)
+        || facet->Data == "NaN") {
         bool found = false;
         for (List<double>::iterator itemFloat = items_float.begin(); itemFloat; itemFloat = itemFloat->Next) {
           if (float_value == itemFloat->Data) {
@@ -1219,6 +1312,8 @@ void EnumerationType::applyFacets() // string types, integer types, float types,
         }
         if (!found) {
           items_float.push_back(float_value);
+          Mstring res = xmlFloat2TTCN3FloatStr(facet->Data);
+          converted_facets.push_back(res);
         }
       }
     }
@@ -1228,6 +1323,7 @@ void EnumerationType::applyFacets() // string types, integer types, float types,
       Mstring res, var;
       XSDName2TTCN3Name(facet->Data, items_time, enum_id_name, res, var);
       text_variants.push_back(var);
+      converted_facets.push_back(res);
     }
     text_variants.sort();
     for (List<Mstring>::iterator var = text_variants.end(); var; var = var->Prev) {
@@ -1274,13 +1370,21 @@ void EnumerationType::printToFile(FILE * file, unsigned int indent_level) const
     for (List<double>::iterator itemFloat = items_float.begin(); itemFloat; itemFloat = itemFloat->Next) {
       if (itemFloat != items_float.begin()) fputs(", ", file);
 
-      double intpart = 0;
-      double fracpart = 0;
-      fracpart = modf(itemFloat->Data, &intpart);
-      if (fracpart == 0) {
-        fprintf(file, "%lld.0", (long long int) (itemFloat->Data));
+      if (itemFloat->Data == PLUS_INFINITY) {
+        fprintf(file, "infinity");
+      } else if (itemFloat->Data == MINUS_INFINITY) {
+        fprintf(file, "-infinity");
+      } else if (isnan(itemFloat->Data)) {
+        fprintf(file, "not_a_number");
       } else {
-        fprintf(file, "%g", itemFloat->Data);
+        double intpart = 0;
+        double fracpart = 0;
+        fracpart = modf(itemFloat->Data, &intpart);
+        if (fracpart == 0) {
+          fprintf(file, "%lld.0", (long long int) (itemFloat->Data));
+        } else {
+          fprintf(file, "%g", itemFloat->Data);
+        }
       }
     }
   } else if (isTimeType(base)) {
@@ -1448,20 +1552,20 @@ void ValueType::applyFacets() // only for integer and float types
     double r = pow(10.0, facet_totalDigits);
 
     if (base == "integer") {
-      lower = (int) -(r - 1);
-      upper = (int) (r - 1);
+      lower = static_cast<int>( -(r - 1) );
+      upper = static_cast<int>(r - 1);
     } else if (base == "positiveInteger") {
       lower = 1;
-      upper = (int) (r - 1);
+      upper = static_cast<int>(r - 1);
     } else if (base == "nonPositiveInteger") {
-      lower = (int) -(r - 1);
+      lower = static_cast<int>( -(r - 1) );
       upper = 0;
     } else if (base == "negativeInteger") {
-      lower = (int) -(r - 1);
+      lower = static_cast<int>( -(r - 1) );
       upper = -1;
     } else if (base == "nonNegativeInteger") {
       lower = 0;
-      upper = (int) (r - 1);
+      upper = static_cast<int>(r - 1);
     } else if (base == "long" ||
       base == "unsignedLong" ||
       base == "int" ||
@@ -1470,11 +1574,11 @@ void ValueType::applyFacets() // only for integer and float types
       base == "unsignedShort" ||
       base == "byte" ||
       base == "unsignedByte") {
-      lower = (int) -(r - 1);
-      upper = (int) (r - 1);
+      lower = static_cast<int>( -(r - 1) );
+      upper = static_cast<int>(r - 1);
     } else if (base == "decimal") {
-      lower = (int) -(r - 1);
-      upper = (int) (r - 1);
+      lower = static_cast<int>( -(r - 1) );
+      upper = static_cast<int>(r - 1);
     } else {
       printWarning(parent->getModule()->getSchemaname(), parent->getName().convertedValue,
         Mstring("Facet 'totalDigits' is not applicable for type '") + base + Mstring("'."));
@@ -1493,33 +1597,44 @@ void ValueType::printToFile(FILE * file) const {
     return;
   }
   if (!fixed_value.empty()) {
-    //Base64binary and hexbyte does not supported
-    Mstring type;
-    if(isBuiltInType(parent->getType().originalValueWoPrefix)){
-      type = parent->getType().originalValueWoPrefix; 
-    }else {
-      type = getPrefixByNameSpace(parent, parent->getReference().get_uri()) + Mstring(":") + parent->getReference().get_val();
-    }
-    if(!isBuiltInType(type)){
-      type = findBuiltInType(parent, type);
-    }
-    if (isStringType(type) || isTimeType(type) || isQNameType(type) || isAnyType(type)) {
-      const Mstring& name = type.getValueWithoutPrefix(':');
-      if (name != "hexBinary" && name != "base64Binary") {
-        fprintf(file, " (\"%s\")", fixed_value.c_str());
-      }
-    } else if (isBooleanType(type)) {
-      Mstring val;
-      if (fixed_value == "1") {
-        val = "true";
-      } else if (fixed_value == "0") {
-        val = "false";
-      } else {
-        val = fixed_value;
-      }
+    if (parent->getConstantDefaultForEmpty() != NULL) {
+      // Insert the constant's name in the TTCN type restriction.
+      TTCN3Module * mod = parent->parent != NULL ? parent->parent->getModule() :
+                          parent->getModule();
+      Mstring val = parent->getConstantDefaultForEmpty()->getAlterego()->getConstantName(mod);
       fprintf(file, " (%s)", val.c_str());
     } else {
-      fprintf(file, " (%s)", fixed_value.c_str());
+      //Base64binary and hexbyte does not supported
+      Mstring type;
+      if(isBuiltInType(parent->getType().originalValueWoPrefix)){
+        type = parent->getType().originalValueWoPrefix; 
+      }else {
+        type = getPrefixByNameSpace(parent, parent->getReference().get_uri()) + Mstring(":") + parent->getReference().get_val();
+      }
+      if(!isBuiltInType(type)){
+        type = findBuiltInType(parent, type);
+      }
+      if (isStringType(type) || isTimeType(type) || isQNameType(type) || isAnyType(type)) {
+        const Mstring& name = type.getValueWithoutPrefix(':');
+        if (name != "hexBinary" && name != "base64Binary") {
+          fprintf(file, " (\"%s\")", fixed_value.c_str());
+        }
+      } else if (isBooleanType(type)) {
+        Mstring val;
+        if (fixed_value == "1") {
+          val = "true";
+        } else if (fixed_value == "0") {
+          val = "false";
+        } else {
+          val = fixed_value;
+        }
+        fprintf(file, " (%s)", val.c_str());
+      } else if (isFloatType(type)) {
+        Mstring val = xmlFloat2TTCN3FloatStr(fixed_value);
+        fprintf(file, " (%s)", val.c_str());
+      } else {
+        fprintf(file, " (%s)", fixed_value.c_str());
+      }
     }
     return;
   }
