@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -22,9 +22,13 @@
 #include "ComplexType.hh"
 #include "ImportStatement.hh"
 #include "Annotation.hh"
+#include "Constant.hh"
+#include "converter.hh"
 
 #include "../common/version_internal.h"
 
+#include <cctype>
+
 #if defined(WIN32) && !defined(MINGW)
 #include <cygwin/version.h>
 #include <sys/cygwin.h>
@@ -39,6 +43,7 @@ TTCN3Module::TTCN3Module(const char * a_filename, XMLParser * a_parser)
 , schemaname()
 , modulename()
 , definedTypes()
+, constantDefs()
 , actualXsdConstruct(c_unknown)
 , xsdVersion()
 , xsdEncoding()
@@ -55,7 +60,8 @@ TTCN3Module::TTCN3Module(const char * a_filename, XMLParser * a_parser)
 , variant()
 , xmlSchemaPrefixes()
 , moduleNotIntoFile(false)
-, moduleNotIntoNameConversion(false) {
+, moduleNotIntoNameConversion(false)
+, const_counter(1) {
 #if defined(WIN32) && !defined(MINGW)
   // Transform a Windows style path: "C:\cygwin\tmp\a.xsd"
   // into a POSIX style path: "/home/a/xsd", so getValueWithoutPrefix('/')
@@ -64,7 +70,7 @@ TTCN3Module::TTCN3Module(const char * a_filename, XMLParser * a_parser)
   char *posix = NULL;
   ssize_t needed = cygwin_conv_path(CCP_WIN_A_TO_POSIX | CCP_RELATIVE, a_filename, NULL, 0);
   if (needed >= 0) {
-    posix = (char*) Malloc(needed);
+    posix = static_cast<char*>( Malloc(needed) );
     if (cygwin_conv_path(CCP_WIN_A_TO_POSIX | CCP_RELATIVE, a_filename, posix, needed)) {
       posix = NULL; // conversion failed
     }
@@ -87,6 +93,10 @@ TTCN3Module::~TTCN3Module() {
   for (List<RootType*>::iterator type = definedTypes.begin(); type; type = type->Next) {
     delete type->Data;
   }
+  
+  for (List<RootType*>::iterator type = constantDefs.begin(); type; type = type->Next) {
+    delete type->Data;
+  }
 }
 
 void TTCN3Module::loadValuesFromXMLDeclaration(const char *a_version,
@@ -284,6 +294,9 @@ void TTCN3Module::generate_TTCN3_included_types(FILE * file) {
 }
 
 void TTCN3Module::generate_TTCN3_types(FILE * file) {
+  for (List<RootType*>::iterator type = constantDefs.begin(); type; type = type->Next) {
+    type->Data->printToFile(file);
+  }
   for (List<RootType*>::iterator type = definedTypes.begin(); type; type = type->Next) {
     if (type->Data->getConstruct() != c_include && type->Data->getConstruct() != c_import) {
       type->Data->printToFile(file);
@@ -333,7 +346,7 @@ void TTCN3Module::generate_with_statement(FILE * file, List<NamespaceType> used_
 
   if (xsi) {
     fprintf(file,
-      "  variant \"controlNamespace \'%s-instance\' prefix \'xsi\'\";\n", XMLSchema.c_str());
+      "  variant \"controlNamespace \'%s-instance\' prefix \'xsi\'\";\n", XMLSchema);
   }
   if (attributeFormDefault == qualified) {
     fprintf(file,
@@ -382,7 +395,7 @@ void TTCN3Module::TargetNamespace2ModuleName() {
   }
   // any character except "A" to "Z", "a" to "z" or "0" to "9" and "_" shall be removed
   for (size_t i = 0; i != res.size(); ++i) {
-    if (!isalpha((const unsigned char) res[i]) && !isdigit((const unsigned char) res[i]) && (res[i] != '_')) {
+    if (!isalpha(static_cast<const unsigned char>( res[i] )) && !isdigit(static_cast<const unsigned char>( res[i] )) && (res[i] != '_')) {
       res.eraseChar(i);
       i--;
     }
@@ -410,7 +423,7 @@ void TTCN3Module::TargetNamespace2ModuleName() {
 
   if (res.empty()) {
     res = "x";
-  } else if (isdigit((const unsigned char) res[0])) {
+  } else if (isdigit(static_cast<const unsigned char>( res[0] ))) {
     res.insertChar(0, 'x');
   }
 
@@ -448,7 +461,11 @@ void TTCN3Module::TargetNamespace2ModuleName() {
 
 void TTCN3Module::dump() const {
   fprintf(stderr, "Module '%s' at %p (from %s)\n",
-    modulename.c_str(), (const void*) this, schemaname.c_str());
+    modulename.c_str(), static_cast<const void*>( this ), schemaname.c_str());
+  
+  for (List<RootType*>::iterator type = constantDefs.begin(); type; type = type->Next) {
+    type->Data->dump(1);
+  }
 
   for (List<RootType*>::iterator type = definedTypes.begin(); type; type = type->Next) {
     type->Data->dump(1);
