@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -17,6 +17,7 @@
 #include "TTCN3Module.hh"
 #include "SimpleType.hh"
 #include "ComplexType.hh"
+#include "converter.hh"
 
 #include "GeneralFunctions.hh"
 
@@ -350,7 +351,7 @@ void XMLParser::startelementHandler(const xmlChar * localname,
 }
 
 void XMLParser::startelementHandlerWhenXSDRead(const xmlChar * localname,
-  int nb_namespaces, const xmlChar ** namespaces, int nb_attributes, const xmlChar ** attributes) {
+  int /*nb_namespaces*/, const xmlChar ** /*namespaces*/, int /*nb_attributes*/, const xmlChar ** /*attributes*/) {
   if (strcmp((const char*)localname, "schema") == 0) {
     // Don't parse beyond the schema tag.
     xmlStopParser(contextCheckingXML);
