@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2016 Ericsson Telecom AB
+ * Copyright (c) 2000-2017 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -28,10 +28,12 @@
 #include <unistd.h> // for using "getopt" function
 #include <sys/stat.h>
 
+#include "converter.hh"
+
 bool c_flag_used = false;
 int  d_flag_used = 0;
 bool e_flag_used = false;
-bool f_flag_used = false;
+bool f_flag_used = false; // also J flag
 bool g_flag_used = true;
 bool h_flag_used = false;
 bool m_flag_used = false;
@@ -54,7 +56,7 @@ static int checkSyntax(const bool not_verbose, const int first_module, const int
   const char * const * const module_names);
 static int validate(int const first_module, int const last_module,
   const char * const * const module_names);
-static int get_xsd_module_names(const bool quiet, const int first_module, 
+static int get_xsd_module_names(const int first_module, 
   const int last_module, const char * const * const module_names);
 static int generateCode(const bool quiet, const bool need_predefined,
   const int first_module, const int last_module,
@@ -72,7 +74,7 @@ int main(int argc, char **argv) {
   char c;
   opterr = 0;
 
-  while ((c = getopt(argc, argv, "cdef:ghmnpqstvwxz")) != -1) {
+  while ((c = getopt(argc, argv, "cdef:ghJ:mnpqstvwxz")) != -1) {
     switch (c) {
       case 'c':
         c_flag_used = true;
@@ -84,6 +86,7 @@ int main(int argc, char **argv) {
         e_flag_used = true;
         break;
       case 'f':
+      case 'J':
         f_flag_used = true;
         from_file = optarg;
         break;
@@ -199,7 +202,7 @@ int main(int argc, char **argv) {
     }
     
     if (n_flag_used) {
-      get_xsd_module_names(q_flag_used, first_module, last_module, module_names);
+      get_xsd_module_names(first_module, last_module, module_names);
       return EXIT_SUCCESS;
     }
 
@@ -216,6 +219,13 @@ int main(int argc, char **argv) {
     }
     return EXIT_FAILURE;
   }
+  
+  if (f_flag_used) {
+    for (int i = 0; i < last_module; ++i) {
+      Free(module_names[i]);
+    }
+    Free(module_names);
+  }
 
   if (XMLParser::getNumWarnings() > 0 ||
     TTCN3ModuleInventory::getNumErrors() > 0 ||
@@ -234,13 +244,13 @@ static void printProductinfo() {
 
 static void printUsage(const char * argv0) {
   fprintf(stderr, "\n"
-    "usage: %s [-ceghmpstVwx] [-f file] schema.xsd ...\n"
+    "usage: %s [-ceghmpstVwx] [-f file] [-J file] schema.xsd ...\n"
     "	or %s -v\n"
     "\n"
     "OPTIONS:\n"
     "	-c:		disable the generation of comments in TTCN-3 modules\n"
     "	-e:		disable the generation of encoding instructions in TTCN-3 modules\n"
-    "	-f file:	the names of XSD files are taken from file instead of the command line\n"
+    "	-f|J file:	the names of XSD files are taken from file instead of the command line\n"
     "	-g:		generate TTCN-3 code disallowing element substitution\n"
     "	-h:		generate TTCN-3 code allowing type substitution\n"
     "	-m:		generate only the UsefulTtcn3Types and XSD predefined modules"
@@ -389,7 +399,7 @@ static int checkSyntax(const bool not_verbose, const int first_module, const int
   return EXIT_SUCCESS;
 }
 
-static int get_xsd_module_names(const bool quiet, const int first_module, 
+static int get_xsd_module_names(const int first_module, 
   const int last_module, const char * const * const module_names) {
   TTCN3ModuleInventory& modules = TTCN3ModuleInventory::getInstance();
   for (int i = first_module; i < last_module; ++i) {
